function altermed,inar,w,thresh,mask=mask,med_clean=med_clean,fora=fora,old=old
;+
; NAME:
;	ALTERMED
; PURPOSE:
;      cosmic particles removal for an image array
; DESCRIPTION:
;      procedure is used when the overall amount of images is in the range 2-5, most effective then.
;      For every image a median-smoothed (optional) mean image "alim" is created, and renormed to satisfy the average 
;      flux level for the frame analysed. Every point that exceed the level of alim+{thresh}*sighma(Poissonian) and
;      has two or more neighbours exceeding too the level mentioned, is removed (i. e. replaced with the corresponding
;      "alim" level).
;      This allows to distinguish cosmic hits and, on the one hand, the object features, and, on the other hand, 
;      normal pixels from the tail of the statistics. Also all the points that exceed higher level alim+2*{thresh}*sighma
;      are replaced.
;
;
; CALLING SEQUENCE:
;	ALTERMED,INAR,W,thresh,mask=mask,med_clean=med_clean,fora=fora
;
; CATEGORY:
;	reduction MPFS-data
;
; INPUTS:
;        INAR - image array (X,Y,N_image)
;        W - window size for median smoothing (if 0, no smothing is made)
;        thresh - threshold level for cosmic alimination
;
; OUTPUTS:
;       result is the cleaned average array, renormed to the integral flux of the frame array 
;
;
; OPTIONAL OUTPUT:
;	mask - difference between the sum of images and resulting frame
;
; OPTIONAL INPUT KEYWORDS:
;	med_clean  -  additional median cleaning of the resulting frame
;       fora - masking most luminous rows and columns to avoid object degradation
;
; RESTRICTIONS:
;	no
; NOTES:
;	no
;
; PROCEDURES USED:
;	Function :   --
;	Procedures:  -
;
; MODIFICATION HISTORY:
;       Written by Pavel Abolmasov, Special Astrophysical Observatory RAS, Jan 2005
;-

  inim=total(inar,3)

  a=size(inar)
  outar=inar
  nim=a[3]  &  Nx=a[1]  & Ny=a[2]
  pwr=total(inar(100:Nx-100,100:Ny-100,*))
;  for k=0,a[3]-1 do outar(*,*,k)=inar(*,*,k) ;/total(total(inar(*,*,k),2),1)
 ;  outar=inar
  mask=replicate(0.,a[1],a[2])
  for k=0,a[3]-1 do begin
    im=reform(outar(*,*,k))
    pim=total(im(100:Nx-100,100:Ny-100))
    alim0=(total(inar,3)-im)*pim/(pwr-pim)
    if w lt 1 then alim=alim0 else alim=med_ext2(alim0,w)  ;*pim/(pwr-pim)
    alim1=alim*0.
    for kx=0,Nx-1 do alim1(kx,*)=alim1(kx,*)+med_ext(reform(alim(kx,*)),10)/2. ;+med_ext(reform(im(kx,*)),10)/2.
    for ky=0,Ny-1 do alim1(*,ky)=alim1(*,ky)+med_ext(reform(alim(*,ky)),15)/2. ;+med_ext(reform(im(*,ky)),15)/2.
;    alim1=smooth(alim1,5) ;!!!!!!!!!!!!!!!

     www=where(im gt alim1+sqrt(alim1)*thresh,cww)

if keyword_set(old) then begin
www=where(im gt alim+sqrt(alim > 50.)*thresh,cww)
    www1=where(alim1(www) le 2.*median(alim1),cww1)
    if cww*cww1 gt 0 then www=www(www1)
    cww=n_elements(www ge 0)
    www2=where(im gt alim0+(alim0 > 100.)*5.*thresh,cww2)
    if cww gt 0 then www=[www,www2] $
        else www=www2
    if cww gt 0 then www3=where(alim1(www) le 5.*median(alim1),cww3)
    if cww*cww3 gt 0 then www=www(www3)
  ;  www=([www,where(im gt 20.*(alim0 > 10.))])
 ;   www=www(uniq(www))
    cww=n_elements(www ge 0)
endif else begin
  www=where(im gt alim1+(alim)*1.*thresh,cww) ; !! 0.9 --> 1.0
endelse

    mask1=mask*0.
    mask1(www)=1.
www1=where(mask1 gt 1.)
if keyword_set(old) then mask1=(shift(mask1,1,1)+shift(mask1,-1,1)+shift(mask1,1,-1)+shift(mask1,-1,-1)+$
    shift(mask1,0,1)+shift(mask1,-1,0)+shift(mask1,1,0)+shift(mask1,0,-1)+$
shift(mask1,2,1)+shift(mask1,-2,1)+shift(mask1,1,-2)+shift(mask1,-1,-2))*mask1 else $
mask1=(shift(mask1,1,1)+shift(mask1,-1,1)+shift(mask1,1,-1)+shift(mask1,-1,-1)+$
    shift(mask1,0,1)+shift(mask1,-1,0)+shift(mask1,1,0)+shift(mask1,0,-1)+$
shift(mask1,2,1)+shift(mask1,-2,1)+shift(mask1,1,-2)+shift(mask1,-1,-2))*mask1 ;; added mask1

    www1=where(mask1 gt 1. or (shift(mask1,1,1)>shift(mask1,-1,1)>shift(mask1,-1,-1)>shift(mask1,1,-1)>$
shift(mask1,-1,0)>shift(mask1,0,-1)>shift(mask1,1,0)>shift(mask1,0,1)) gt 2. or $
(shift(mask1,2,1)>shift(mask1,-2,1)>shift(mask1,-1,-2)>shift(mask1,1,-2)>$
shift(mask1,-1,2)>shift(mask1,2,-1)>shift(mask1,1,2)>shift(mask1,-2,1)) gt 3.,cww1)

; www1=[www1,where(im gt (alim1>10.)*20.,cww)] ;; !!
    www1=www1(uniq(www1))
    alim1=alim1*median(im)/median(alim1)
    if total(www1) gt 0 then im(www1)=alim1(www1)
    outar(*,*,k)=im  ;*total(total(inar(*,*,k),2),1)
    message,'sigma = '+string(sqrt(pwr/double(n_elements(inar))))+'used',/cont
    message,'image N'+string(k)+'clear, '+string(cww)+'... no,'+string(cww1)+'points replaced',/cont
    if cww1 eq 0 then message,'process ineffective!',/cont
    if double(cww1) ge double(n_elements(inar))/2. then message,'image spoiled by the procedure!'
    if total(www1) gt 0 then mask(www1)=mask(www1)+1.
  endfor

  outim=total(outar,3)

if keyword_set(fora) then begin
  alimx=total(alim,2)/a[2]
  alimy=total(alim,1)/a[1]
  forwx=where(smooth(alimx/med_ext(alimx,50),50) gt 1.+.2*sqrt(double(a[1]*a[2])/pwr),fnx)
  forwy=where(smooth(alimy/med_ext(alimy,50),50) gt 1.+2.*sqrt(double(a[1]*a[2])/pwr),fny)
 ; if fnx gt 0 then for i=0,fnx-1 do outim(forwx[i],*)=inim(forwx[i],*)
 ; if fny gt 0 then for j=0,fny-1 do outim(*,forwy[j])=inim(*,forwy[j])
  if fny*fnx gt 0. then begin
    for k=0,fnx-1 do for j=0,fny-1 do outim(forwx[k],forwy[j])=inim(forwx[k],forwy[j])
  endif
  message,'/FORA: '+strtrim(string(fnx*fny),2)+'points replaced',/cont
endif

if keyword_set(med_clean) then begin
  outim1=med_ext2(outim,3)
  w1=where((outim>shift(outim,1,1)>shift(outim,1,-1)>shift(outim,-1,1)>shift(outim,-1,-1)>$
   shift(outim,0,1)>shift(outim,0,-1)>shift(outim,-1,0)>shift(outim,1,0)) gt outim1+sqrt((2.*outim1)>100.)*thresh*5.,n1)
  if n1 gt 0 then outim(w1)=outim1(w1)
endif
;  mask=inim-outim
return,outim
end
