pro analine,infile,l,wdir, w=w, dv=dv,field=field,losq=losq,colt=colt,smoo=smoo,trub=trub,dne=dne,sb=sb
;+
; NAME:
;	anaLINE
; PURPOSE:
;	obtaining flux, velosity and fwhm maps from a datacube
; DESCRIPTION: 
;	fits line profile with a gaussian and computes fluxes etc. from its parameters
;
; CALLING SEQUENCE:
;	ANALINE,INFILE,LAMBDA,WDIR,w=w,dv=dv,field=field
; CATEGORY:
;	MPFS-data analysis		
;
; INPUTS:
;	INFILE - fits-file containing initial primarily-reducted data  
;	LAMBDA - rest-frame wavelength (redshift z for distant galaxies can be taken from the header, or 
;		added by keyword dv in km/s)
;	WDIR - output directory
; OUTPUTS:
;	output fts and ps files with names of the types: [lambda]_{fv,w}.ps and [lambda]_{f,v,w,c}.fts
;	containing fluxes (f), velosities (v) and fwhm in velosity units (w)
;		
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;	w - spectral interval halfwidth (in Angstroms, by default set to 10.)
;	dv - additional velosity shift
;	field - field size in pxls ([16,16] by default)
;       colt - colour table (0 by default)
;       losq - quasi-logarythmic scale scale linear for LN(flux+median(flux))
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;	
;	Procedures:  ?
;
; MODIFICATION HISTORY:
;       Written by P. Abolmasov, Special Astrophysical Observatory RAS, Jul 2004
;-

  if not(keyword_set(w)) then w=50.
  if not(keyword_set(colt)) then colt=0
  if not(keyword_set(dv)) then dv=0.
  if not(keyword_set(field)) then field=[16,16]
  out=wdir+strtrim(string(round(l)),2)

loadct,27

  obj=double(readfits(infile,h))
  a=size(obj)
  dl=sxpar(h,'CDELT1')
  l0=sxpar(h,'CRVAL1')
  d=round(w/dl)                                     ; window in pixels
  area=dblarr(2*d+1,a[2])
  larea=dblarr(2*d+1)
  nx=a(1)                          ;  number of wavelengths 
  ny=a(2)			   ; number of fibers
     if ny ne field[0]*field[1] then message,'WRONG DIMENTIONS OF THE FIELD!'

  lambda=l0+dl*dindgen(nx)		; wavelength grid
  z=double(sxpar(h,'REDSHIFT'))
  uni=sxpar(h,'BUNIT')

  ccor=round((l*(z+1.)-l0)/dl)
  area=obj(ccor-d:ccor+d,*)  
  larea=lambda(min(ccor)-d:max(ccor)+d)  
  set_plot,'x'
aa=size(area)                                        ; aa[2] = a[2]  !!!

    w1=where(larea gt l*(z+1.)-w/10. and larea lt l*(z+1.)+w/10.)
    mest=max(med_ext(total(area(w1,*),2),3),mcest)
    cest=larea(w1[mcest])
    west=where(larea lt l*(z+1.)+w*.3 and larea gt l*(z+1.)-w*.3)
    nest=where(larea gt l*(z+1.)+w*.3 or larea lt l*(z+1.)-w*.3)
  plot,larea,total(area,2)
  afit=gaussfit(larea(west),(total(area(west,*),2))/total(area(west,*)),g0,nterms=4,$
        estimates=[(max(total(area(west,*),2))-min(total(area(west,*),2)))/total(area(west,*)),l*(z+1.),w/10.,min(total(area(west,*),2))]) 
  oplot,larea(west),afit*total(area(west,*)),color=1000
  g0[0]=g0[0]*total(area(west,*))
  g0[1]=l*(z+1.)  &  g0[2]=2.5
;  g0[2]=3.
  flux=dblarr(ny)
  dflux=dblarr(ny)
  rms=dblarr(ny)
  fwhm=dblarr(ny)
  cont=dblarr(ny)
  cent=dblarr(ny)
  dfwhm=dblarr(ny)
  dcont=dblarr(ny)
  dcent=dblarr(ny)
  var=dblarr(ny)
  velt=dblarr(ny)
  var0=dblarr(ny)

   arasm=dblarr(field[0]*field[1],aa[1])
   for k=0,aa[1]-1 do arasm(*,k)=area(k,*)

  for i=0,ny-1 do begin
    if not(keyword_set(smoo)) then vec=area(*,i) else vec=smooth(area(*,i),3)

    area(*,i)=vec
    plot,larea,vec
    wplus=where(vec gt 0.,nplus)
    if nplus lt 1 then begin
      flux[i]=0.
      cent[i]=0.
      fwhm[i]=0.
      continue
    endif

    w1=where(larea gt l*(z+1.)-w/5. and larea lt l*(z+1.)+w/5.)
    mest=max(med_ext(vec(w1),3),mcest)
    cest=larea(w1[mcest])
    west=where(larea lt l*(z+1.)+w*.3 and larea gt l*(z+1.)-w*.3)
    nest=where(larea gt l*(z+1.)+w*.3 or larea lt l*(z+1.)-w*.3 and vec le abs(2.*median(vec)) )

    contpars=goodpoly(larea(nest),vec(nest),1,2.,yfit)   ; continuum level
    cont=poly(larea,contpars)

    afit=gaussfit(larea(west),(vec(west)-cont(west))*1.e+17,g,$
              nterms=3,estimates=[g0[0]*1.e+17,g0(1:2)],sigma=sighma)
    arasm(i,*)=cont
    arasm(i,west)=afit*1.e-17+cont(west)
    oplot,larea,arasm(i,*),color=1000
    flux[i]=g[0]*abs(g[2])*sqrt(!DPI*2.)*1.e-17
    cent[i]=g[1]
    fwhm[i]=g[2]*sqrt(8.*alog(2.))
    rms[i]=stdev(area(west,i)-arasm(i,west))
    dflux[i]=flux[i]*sqrt((sighma[0]/g[0])^2+(sighma[2]/g[2])^2)                       ;fwhm[i]*rms[i]/dl
    dcent[i]=sighma[1]                        ;fwhm[i]^2*rms[i]/sqrt(8.*alog(2.)*total(arasm(i,west)^2*(larea(west)-cent[i])^2))
    if flux[i] gt 10.*total(area(west,i)-cont(west)) or rms[i] gt flux[i] or abs(cent[i]-l*(z+1.)) gt w/2. then flux[i]=0.
  endfor

  flux_map=reform(flux,field[0],field[1])
  fwhm_map=reform(fwhm,field[0],field[1])
  cent_map=reform(cent,field[0],field[1])
  dcent_map=reform(dcent,field[0],field[1])
;  cont_map=reform(cont,field[0],field[1])

c=2.9979e+5                       ; c in km/s 
  
velt=(cent-l)*c/l-double(dv)
  wv=fwhm*c/l
  dv=abs(dcent)*c/l
  vel_map=reform(velt,field[0],field[1])
  wv_map=reform(wv,field[0],field[1])
  dv_map=reform(dv,field[0],field[1])

flux1=reform(smooth(flux_map,3),field[0]*field[1])
 wbad=where(flux le rms*1.,nbad)
  if nbad gt 0 then wbad=[wbad,where(flux1 le 1.*median(flux))] else wbad=where(flux1 le 1.*median(flux),nbad)
 wbad=wbad(uniq(wbad))
 ; vel_map=med_ext2(vel_map,3)

; vel_map(wbad)=sqrt(-1)
; flux(wbad)=0.
  flux_map=reform(flux,field[0],field[1])

if keyword_set(trub) then begin
   flux_map(0,*)=0.  &  flux_map(*,0)=0.  &  flux_map(field[0]-1,*)=0.  &  flux_map(*,field[1]-1)=0.
endif

intflux=total(flux_map) & dinf=sqrt(total(dflux^2))

posang=sxpar(h,'POSANG')

set_plot,'ps'
loadct,colt

xa=rebin(dindgen(field[1]),field[0],field[1],/sample)
ya=transpose(xa)
 
velf=vel_map(where(finite(vel_map) and abs(vel_map-c*z) le 2.*c*w/(l)))
vellev=round((median(velf)+(median(velf(where(velf gt median(velf))))-median(velf(where(velf lt median(velf)))))*(dindgen(20)-10.)*.5)*.1)*10.
vellev=vellev(uniq(vellev))
vellev=-310.+10.*dindgen(200)

posar=posang*!DPI/180.
ra0=sxpar(h,'RA') & dec0=sxpar(h,'DEC')
ra='20 35 0.75'  &  dec='+60 11 30.9'

xch=-(ratos(ra)-ratos(ra0))*cos(posar)*cos(detos(dec0)/206265.)+(detos(dec)-detos(dec0))*sin(posar)+double(field[0])/2.
ych=(ratos(ra)-ratos(ra0))*sin(posar)*cos(detos(dec0)/206265.)+(detos(dec)-detos(dec0))*cos(posar)+double(field[1])/2.

print,xch,ych

if not(keyword_set(losq)) then begin                           ; logarythmic scale 
  flux_plot=-rotate(reform(flux_map)/median(flux(where(flux gt 0.))),7)
  flev=reverse(-.1*dindgen(1600))
  flev=flev(where(flev gt min(1.1*flux_plot)))
endif else begin
  flux_plot=-alog(rotate(reform(flux_map+median(flux_map)),7))
  flev=min(flux_plot(where(finite(flux_plot))))+.01*dindgen(100)
endelse

vel_plot=rotate(reform(vel_map),7)

post=[3.,3.,3.+field[0],3.+field[1]]*1000.

  device,filename=out+'_fv.ps',xsize=field[0]+5.,ysize=field[1]+5.
    contour,flux_plot,xa,ya,levels=flev,charsize=2.5,/fill,charthick=3.,/device,$ ;title=sxpar(h,'OBJECT')+', wavelength = '+strtrim(string(l),2)+string("305B),$
             xstyle=1,ystyle=1,yrange=[min(ya),max(ya)],pos=post
;    contour,rotate(flux_map,7),/overplot,levels=10^(dindgen(60)/8.-6.),c_linestyle=replicate(2,40)
;    tvscl,flux_plot,1.,3.,xsize=field[0],ysize=field[1],/centim
   if keyword_set(dne) then drawne,.8,.8,posang,r=.1,colo=sb
loadct,0
    contour,vel_plot,xa,ya,/downhill,/overplot,levels=vellev,xstyle=5,ystyle=5,$
        c_annotation=' '+strtrim(string(vellev,format='(F8.1)'),2)  ;,yrange=[max(ya),min(ya)]
loadct,colt

;    contour,rotate(vel_map(lk,*,*),7),/overplot,levels=round(median(vel_map)/10.)*10.+5.*dindgen(20)-50.,c_linestyle=replicate(2,40)
;    drawne,.8,.8,posang,r=.1
;    xyouts,5,2,'velosities in km/s',/device
;    circle_plot,xch,ych,1.,field=field
   plotsym,0,4,color=-1,thick=10.
   plots,xch,ych,psym=8,/data,color=-1,symsize=1.
 device,/close

   device,filename=out+'_f.ps',xsize=field[0]+5.,ysize=field[1]+5.
   if colt gt 0. then flux_plot=-flux_plot
    tvscl,flux_plot,3.,3.,xsize=field[0],ysize=field[1],/centim
    contour,flux_plot,xa,ya,charsize=2.5,/nodata,charthick=3.,/device,pos=post,$  ;  title=sxpar(h,'OBJECT')+', wavelength = '+strtrim(string(l),2)+string("305B),$
             xstyle=1,ystyle=1,yrange=[min(ya-.5),max(ya+.5)],xrange=[min(ya-.5),max(ya+.5)],/noer
loadct,0
   if keyword_set(dne) then drawne,.75,.7,posang,r=.1,colo=sb
;    xyouts,5,2,'integral flux '+strtrim(string(intflux),2)+'+/-'+strtrim(string(dinf),2)+'erg/cm^2 s',/device
;   loadct,0
   plotsym,0,4,color=-1,thick=10.
;   plots,(xch+1.)/21.,(ych+3.)/21.,psym=8,/norm,color=-1,symsize=1.
   plots,xch,ych,psym=8,/data,color=-1,symsize=1.
;   scalp,flux,uni,losq=losq,/expf
 device,/close
 device,filename=out+'_W.ps',xsize=16,ysize=16
    contour,rotate(reform(wv_map),7),/downhill,levels=round(dl*c/l)*(dindgen(5)+1.)*2.,c_annotation=strtrim(string(round(dl*c/l)*(dindgen(5)+1.)*2.,format='(F8.1)'),2),$
        title=sxpar(h,'OBJECT')+', wavelength = '+strtrim(string(l),2)+'Angstrom, redshift used z='+sxpar(h,'REDSHIFT'),/noerase
    drawne,.75,.7,posang,r=.1,colo=sb
    xyouts,0,-1,'line halfwidths in km/s'
  device,/close
set_plot,'x'

  h2=h
  fileout=out+'_F.FTS'
  sxaddpar,h2,'FILE',fileout
  sxaddpar,h2,'INTFLUX',intflux,'erg/cm^2 s'
  sxaddpar,h2,'DFLUX',dinf,'erg/cm^2 s - integral flux uncertainty',after='INTFLUX'
  sxaddpar,h2,'LINE',strtrim(string(l),2),' line wavelength'
  sxaddpar,h2,'IMAGETYP','flux map'
  writefits,fileout,reform(flux_map),h2

  h2=h
  fileout=out+'_dF.FTS'
  sxaddpar,h2,'FILE',fileout
  sxaddpar,h2,'INTFLUX',intflux,'erg/cm^2 s'
  sxaddpar,h2,'DFLUX',dinf,'erg/cm^2 s - integral flux uncertainty',after='INTFLUX'
  sxaddpar,h2,'LINE',strtrim(string(l),2),' line wavelength'
  sxaddpar,h2,'IMAGETYP','flux uncertainty map'
  writefits,fileout,reform(rebin(dflux,field[0],field[1])),h2

  h2=h
  fileout=out+'_W.FTS'
  sxaddpar,h2,'FILE',fileout
  sxaddpar,h2,'IMAGETYP','fwhm map'
  sxaddpar,h2,'LINE',strtrim(string(l),2),' line wavelength'
  writefits,fileout,reform(fwhm_map),h2

  h2=h
  fileout=out+'_V.FTS'
  sxaddpar,h2,'FILE',fileout
  sxaddpar,h2,'IMAGETYP','velocity map'
  sxaddpar,h2,'LINE',strtrim(string(l),2),' line wavelength'
  writefits,fileout,reform(vel_map),h2

  h2=h
  fileout=out+'_dv.FTS'
  sxaddpar,h2,'FILE',fileout
  sxaddpar,h2,'IMAGETYP','velocity map uncertainties'
  sxaddpar,h2,'LINE',strtrim(string(l),2),' line wavelength'
  writefits,fileout,reform(dv_map),h2

  h2=h
  fileout=out+'_cube.FTS'
  sxaddpar,h2,'FILE',fileout
  sxaddpar,h2,'CRVAL2',0.
  sxaddpar,h2,'CDELT2',1.
  sxaddpar,h2,'CRVAL1',(min(larea)-l)*c/l
  sxaddpar,h2,'CDELT1',dl*c/l
  sxaddpar,h2,'IMAGETYP','data cube with line profiles'
  a=size(area)
;  ar1=dblarr(field[0]*field[1],a[1])
;  for k=0,a[1]-1 do ar1(*,k)=reform(area(k,*)-min(area),field[0]*field[1])
  writefits,fileout,area,h2

  h2=h
  fileout=out+'_fit.FTS'
  sxaddpar,h2,'FILE',fileout
  sxaddpar,h2,'CRVAL1',(min(larea)-l)*c/l
  sxaddpar,h2,'CDELT1',dl*c/l
  sxaddpar,h2,'CRVAL2',0.
  sxaddpar,h2,'CDELT2',1.
  sxaddpar,h2,'IMAGETYP','data cube with line profiles'
;  a=size(arasm)
  ar1=dblarr(a[1],field[0]*field[1])
  for k=0,a[1]-1 do ar1(k,*)=reform(arasm(*,k),field[0]*field[1])  
  writefits,fileout,ar1,h2

fin:
loadct,0
end
