pro analine_faint,infile,l,wdir, w=w, dv=dv,field=field,smoo=smoo,trub=trub,dne=dne,colt=colt,$
   losc=losc,fixl=fixl,z=z
;+
; NAME:
;	anaLINE_faint
; PURPOSE:
;	obtaining flux, velosity and fwhm maps from a datacube
; DESCRIPTION: 
;	fits line profile with a gaussian and computes fluxes etc. from its parameters
;
; CALLING SEQUENCE:
;	ANALINE,INFILE,LAMBDA,WDIR,w=w,dv=dv,field=field
; CATEGORY:
;	MPFS-data analysis		
;
; INPUTS:
;	INFILE - fits-file containing initial primarily-reducted data  
;	LAMBDA - rest-frame wavelength (redshift z for distant galaxies can be taken from the header, or 
;		added by keyword dv in km/s)
;	WDIR - output directory
; OUTPUTS:
;	output fts and ps files with names of the types: [lambda]_{fv,w}.ps and [lambda]_{f,v,w,c}.fts
;	containing fluxes (f), velosities (v) and fwhm in velosity units (w)
;		
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;	w - spectral interval halfwidth (in Angstroms, by default set to 10.)
;	dv - additional velosity shift
;	field - field size in pxls ([16,16] by default)
;       smoo - median smoothing 3X3 for flux maps
;       trub - truncate borders
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;	
;	Procedures:  ?
;
; MODIFICATION HISTORY:
;       Written by P. Abolmasov, Special Astrophysical Observatory RAS, Jul 2004
;-

  if not(keyword_set(w)) then w=50.
  if not(keyword_set(colt)) then colt=0.
  if not(keyword_set(dv)) then dv=0.
  if not(keyword_set(field)) then field=[16,16]
  out=wdir+strtrim(string(round(l)),2)

  sent=readfits(wdir+'sent.fts',hs)            ; Spectral sensitivity -- what for?
  
loadct,27

  obj=double(readfits(infile,h))
  a=size(obj)
  uni=sxpar(h,'BUNIT')
  dl=sxpar(h,'CDELT1')
  l0=sxpar(h,'CRVAL1')
  d=round(w/dl)
  area=dblarr(2*d+1,a(2))
  larea=dblarr(2*d+1)
  nx=a(1)                          ;  number of wavelengths 
  ny=a(2)			   ; number of fibers

  lambda=l0+dl*dindgen(nx)		; wavelength grid
  if not(keyword_set(z)) then z=double(sxpar(h,'REDSHIFT'))

  ccor=round((l*(z+1.)-l0)/dl)
  area=obj(ccor-d:ccor+d,*)  
  larea=lambda(ccor-d:ccor+d)  
     if ny ne field[0]*field[1] then begin
         message,'WRONG DIMENTIONS OF THE FIELD!',/cont
         area=area(*,a[2]-field[0]*field[1]:a[2]-1)
         ny=field[0]*field[1]
     endif
aa=size(area)
sence=median(sent(ccor-d:ccor+d))/sxpar(h,'GAIN')

if keyword_set(smoo) then begin
  for i=0,aa[2]-1 do begin
   ; slice=reform(area(i,*),field[0],field[1])
;    area(*,i)=smooth(area(*,i),3)
  endfor
endif
  set_plot,'x'

  flux=dblarr(ny)
  dflux=dblarr(ny)
  rms=dblarr(ny)
  fwhm=dblarr(ny)
  cont=dblarr(ny)
  cent=dblarr(ny)
  dfwhm=dblarr(ny)
  dcont=dblarr(ny)
  dcent=dblarr(ny)
  var=dblarr(ny)
  velt=dblarr(ny)
  var0=dblarr(ny)
  ston=dblarr(ny)
   arasm=dblarr(field[0]*field[1],aa[1])
   for k=0,aa[1]-1 do arasm(*,k)=area(k,*)

  for i=0,ny-1 do begin
;    plot,larea,area(*,i)
    wplus=where(area(*,i) gt 0.,nplus)
    if nplus lt 1 then begin
      flux[i]=0.
      cent[i]=0.
      dcent[i]=w
      fwhm[i]=0.
      continue
    endif
    w1=where(larea gt l*(z+1.)-w/5. and larea lt l*(z+1.)+w/5.)
    mest=max(smooth(med_ext(area(w1,i),3),3),mcest)
    cest=larea(w1[mcest])
    if keyword_set(fixl) then cest=l*(z+1.)
    west=where(larea lt cest+w*.2 and larea gt cest-w*.2)
    nest=where(abs(larea-cest) ge w*.2 and area(*,i) le median(area(*,i)))

;    cont=lowess(larea(nest),area(nest,i),w/4.,1)
    contpars=goodpoly(larea(nest),area(nest,i),1,1.,afit)
    cont=poly(larea,contpars)
;    oplot,larea,cont,color=10000
 ;   gfit=gaussfit(larea(west),(area(west,i)-cont(west))/mest,gf,nterms=3,est=[mest,cest,6.])*mest
    flux[i]=int_tabulated(larea(west),area(west,i)-cont(west),/double)
    cent[i]=int_tabulated(larea(west),larea(west)*(area(west,i)-cont(west)),/double)/flux[i]
    fwhm[i]=sqrt(total((larea(west)-cent[i])^2*(area(west,i)-cont(west) > 0.)/flux[i]))*sqrt(8.*alog(2.))
    var[i]=stdev(area(nest,i)-cont(nest))*double(n_elements(west))         ;fwhm[i]/dl
    ston[i]=flux[i]/var[i]
    dcent[i]=fwhm[i]/ston[i]
  endfor

  flux_map=reform(flux,field[0],field[1])
  cent_map=reform(cent,field[0],field[1])
;  cont_map=reform(cont,field[0],field[1])

c=2.9979e+5                       ; c in km/s 

velt=(cent-l)*c/l   ;-double(dv)
  vel_map=reform(velt,field[0],field[1])
dvelt=dcent*c/l
  dvel_map=reform(dvelt,field[0],field[1])

flux1=reform(smooth(flux_map,3),field[0]*field[1])

wbad=where(flux le 1.*median(flux1),nbad)

 if nbad gt 0 then vel_map(wbad)=sqrt(-1)
; vel_map=med_ext2(vel_map,3)
; flux_map(wbad)=0.

if keyword_set(trub) then begin
  flux_map(0,*)=0.  &  flux_map(*,0)=0.  &  flux_map(field[0]-1,*)=0.  &  flux_map(*,field[1]-1)=0.
  vel_map(0,*)=sqrt(-1)  &  vel_map(*,0)=sqrt(-1)  &  vel_map(field[0]-1,*)=sqrt(-1)  &  vel_map(*,field[1]-1)=sqrt(-1)
endif

intflux=total(flux_map) & dinf=stdev(flux_map)

vmean=total(flux_map*vel_map)/intflux

posang=sxpar(h,'POSANG')

set_plot,'ps'

xa=rebin(dindgen(field[1]),field[0],field[1],/sample)
ya=transpose(xa)
 
velf=vel_map(where(finite(vel_map) and abs(vel_map-c*z) le c*w/l))
vest=((2.+z)*z/(1.+(z+1.)^2))*c
vellev=10.*round(vest*.1)+dindgen(30)*10.-150.
vellev=vellev(uniq(vellev))

posar=posang*!DPI/180.
ra0=sxpar(h,'RA') & dec0=sxpar(h,'DEC')
; ra='20 35 0.75'  &  dec='+60 11 30.9'
ra='03 45 55.77'  &  dec='68 04 55.2'

xch=-(ratos(ra)-ratos(ra0))*cos(posar)*cos(detos(dec0)/206265.)+(detos(dec)-detos(dec0))*sin(posar)+double(field[0])/2.
ych=(ratos(ra)-ratos(ra0))*sin(posar)*cos(detos(dec0)/206265.)+(detos(dec)-detos(dec0))*cos(posar)+double(field[1])/2.

flux_plot=-rotate(reform(flux_map)/median(flux(where(flux gt 0.))),7)
; if keyword_set(smoo) then flux_plot=smooth(med_ext2(flux_plot,3),3)
if keyword_set(losc) then flev=(-10.^(2.5-.1*dindgen(25)))
; flev=flev(where(flev gt min(flux_plot*1.2)))
vel_plot=rotate(reform(vel_map),7)
if keyword_set(smoo) then vel_plot=med_xy(reform(vel_plot,field[0],field[1]),smoo*2+1)
if keyword_set(smoo) then begin
 flux_plot=med_xy(reform(flux_plot,field[0],field[1]),smoo*2+1)
; flux_plot(where(abs(flux_plot-flux_plot1) ge 5*median(abs(flux_plot-flux_plot1))))=$
 ; flux_plot1(where(abs(flux_plot-flux_plot1) ge 5*median(abs(flux_plot-flux_plot1))))
endif
 flev=reverse(-(dindgen(10000))*(max(vel_plot)-min(vel_plot))/double(5000))+2.*max(vel_plot)
loadct,0
post=[3.,3.,3.+field[0],3.+field[1]]*1000.
  device,filename=out+'_fv.ps',xsize=field[0]+5.,ysize=field[1]+5.
    contour,flux_plot,xa,ya,levels=flev,charsize=2.,/fill,$
             xstyle=1,ystyle=1,pos=post,/device,charthick=3.  ;,title=sxpar(h,'OBJECT')+', wavelength = '+strtrim(string(l),2)+string("305B)  ;,yrange=[max(ya),min(ya)]
;    contour,rotate(flux_map,7),/overplot,levels=10^(dindgen(60)/8.-6.),c_linestyle=replicate(2,40)
;    tvscl,flux_plot,1.,3.,xsize=field[0],ysize=field[1],/centim
    if keyword_set(dne) then drawne,.75,.8,posang,r=.1,colo=sb
   contour1,vel_plot,xa,ya,vellev,post  ;,ylim=16
  ;  contour,vel_plot,xa,ya,/downhill,/overplot,levels=vellev,xstyle=5,ystyle=5,$
  ;      c_annotation=' '+strtrim(string(vellev,format='(F8.1)'),2)
;    contour,rotate(vel_map(lk,*,*),7),/overplot,levels=round(median(vel_map)/10.)*10.+5.*dindgen(20)-50.,c_linestyle=replicate(2,40)
;    xyouts,5,2,'velosities in km/s',/device
;    circle_plot,xch,ych,1.,field=field
   plotsym,0,4,color=-1,thick=3.
   plots,xch,ych,psym=8,/data,color=-1,symsize=1.
 device,/close

  device,filename=out+'_f.ps',xsize=field[0]+5.,ysize=field[1]+5.
    loadct,colt
    if colt ne 0. then flux_plot=-flux_plot
     if keyword_set(losc) then tvscl,rotate(-alog(flux_map+median(flux_map)),7),3.,3.,xsize=field[0],ysize=field[1],/centim else $
    tvscl,flux_plot,3.,3.,xsize=field[0],ysize=field[1],/centim
;    tvscl,flux_plot,3.,3.,xsize=field[0],ysize=field[1],/centim
    contour,flux_plot,xa,ya,charsize=2.5,/nodata,charthick=3.,pos=post,/dev,$  ;title=sxpar(h,'OBJECT')+', wavelength = '+strtrim(string(l),2)+string("305B),$
             xstyle=1,ystyle=1,yrange=[min(ya-.5),max(ya+.5)],xrange=[min(ya-.5),max(ya+.5)],/noer
    if keyword_set(dne) then drawne,.75,.8,posang,r=.1,colo=sb
;    xyouts,5,2,'integral flux '+strtrim(string(intflux),2)+'+/-'+strtrim(string(dinf),2)+'erg/cm^2 s',/device
   loadct,0
   plotsym,0,4,color=-1,thick=10.
;   plots,(xch+1.)/21.,(ych+3.)/21.,psym=8,/norm,color=-1,symsize=1.
   plots,xch,ych,psym=8,/data,color=1,symsize=1.
;   scalp,flux,uni,/expf
 device,/close

set_plot,'x'

message,'mean radial velosity: '+strtrim(string(vmean)),/cont

  h2=h
  fileout=out+'_F.FTS'
  sxaddpar,h2,'FILE',fileout
  sxaddpar,h2,'INTFLUX',intflux,'erg/cm^2 s'
  sxaddpar,h2,'DFLUX',dinf,'erg/cm^2 s - integral flux uncertainty',after='INTFLUX'
  sxaddpar,h2,'MEANVEL',vmean,'km/s'
  sxaddpar,h2,'LINE',strtrim(string(l),2),' line wavelength'
  sxaddpar,h2,'IMAGETYP','flux map'
  writefits,fileout,reform(flux_map),h2

  h2=h
  fileout=out+'_V.FTS'
  sxaddpar,h2,'FILE',fileout
  sxaddpar,h2,'LINE',strtrim(string(l),2),' line wavelength'
  sxaddpar,h2,'IMAGETYP','velocity map'
  writefits,fileout,reform(vel_map),h2

  h2=h
  fileout=out+'_dv.FTS'
  sxaddpar,h2,'FILE',fileout
  sxaddpar,h2,'LINE',strtrim(string(l),2),' line wavelength'
  sxaddpar,h2,'IMAGETYP','velocity uncertainties map'
  writefits,fileout,reform(dvel_map),h2

  h2=h
  fileout=out+'_cube.FTS'
  sxaddpar,h2,'FILE',fileout
  sxaddpar,h2,'CRVAL2',0.
  sxaddpar,h2,'CDELT2',1.
  sxaddpar,h2,'CRVAL1',(min(larea)-l)*c/l
  sxaddpar,h2,'CDELT1',dl*c/l
  sxaddpar,h2,'LINE',strtrim(string(l),2),' line wavelength'
  sxaddpar,h2,'IMAGETYP','data cube with line profiles'
  a=size(area)
;  ar1=dblarr(field[0]*field[1],a[1])
;  for k=0,a[1]-1 do ar1(*,k)=reform(area(k,*)-min(area),field[0]*field[1])
  writefits,fileout,area,h2

fin:
loadct,0
end
