function gau_ext,x,a
  y=gaussian(x,a,pder)
  return,[y,pder[0],pder[1],pder[2]]
end
;--------------------------------------------------------------------------------------------

pro CREA_DI2,LOGFILE,FWHM,N_deg,YFIT=yfit,PLOT=plot,manual=manual
;+
; NAME:
;	CREA_DI2 (fromer "CREA_DISPER")
; PURPOSE:
;      Identefication comparison spectra & creation array coefficient dispersion curve
; DESCRIPTION:
;	The routine compare position lines in  2D-array comparison MPFS-spectrum
;	(from file 'neon_s.fts') with tabulated wavelength from file 'LINES.TAB',
;	 and create after 2-dimensional
;	 polynomial approximation array dispersion coefficients
;
; CALLING SEQUENCE:
;	CREA_DISPER,LOGFILE,FWHM,N_deg,PLOT=plot
;
; CATEGORY:
;	reduction MPFS-data
;
; INPUTS:
;	LOG = String scalar of file name  FITS-header from the LOG observation
;	FWHM = width spectral lines in px
;	N_deg = degree of 2D-polynomial approximation dispersion curve
;
; OUTPUTS:
;	 2-dimensional ( N_deg+1) x N_spectra float array coefficient dispersion curve,
;        saved to the disk in  working directory with  standard name 'disper.fts' and
;	 print in file 'disper.txt'.
;	 Last value in every string is rms approximation i px
;
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;	PLOT - if present 2-d errors of approximation plotted to display,
;	else save plot to POSTSCRIPT file in working directory with name 'err_line.ps'
;
; RESTRICTIONS:
;	no
; NOTES:
;	no
;
; PROCEDURES USED:
;	Function :   DEF_WDIR,DEF_RDIR,DEF_NAME,READ_FTS,INP_WAVE
;	Procedures:  CRSPROD,GOODPOLY
;
; MODIFICATION HISTORY:
;       Written by Victor Afanasiev, Special Astrophysical Observatory RAS, Jul 1999
;       Removed some bags : A.Moiseev, Mar 2000
;-

on_error,0
message,'creation dispersion curve',/cont
wdir=DEF_WDIR(LOGFILE)
name_neon=DEF_NAME(LOGFILE,'NEON',N_name)
dir=DEF_RDIR(LOGFILE)
out_str='comparison spectrum files: '+DEF_RDIR(LOGFILE)

for k=0,N_name-1 do begin
  out_str=out_str+name_neon(k)+','
endfor
!p.multi=0
wave=INP_WAVE(LOGFILE)
lambda_c=wave(0)  & d_lambda=wave(1)

; definition level separation lines

if d_lambda lt 1. then lev=0
if d_lambda gt 1 and d_lambda le 2 then lev=1
if d_lambda gt 2 and d_lambda le 4 then lev=2
if d_lambda gt 4 then lev=3
tresh=2.

sw=0 & if keyword_set(plot) then sw=1                                ; sw -- plotting flag
 lev=lev-1.
; reading neon data
neon=READ_FTS(wdir+'neon_sf.fts',/sil)                              ;,SILENT='silent')

a=size(neon)
Nx=a(1)  &  Ny=a(2)
x=dindgen(Nx)

for y=0,Ny-1 do begin
  neon_tmp=neon(*,y)
  wtmp=where(neon_tmp le 2.*median(neon_tmp))
  neofit=goodpoly(x(wtmp),neon_tmp(wtmp),4,2.,neon_bgd,newx,newy)
  neon_bgd=poly(x,neofit)
  neon(*,y)=neon(*,y)-neon_bgd
endfor

; reading lines from table

openr,1,'/usr/local/rsi/idl_6.0/external/mpfs_2K.lib/lines.tab';/users/lfz/standards/lines.tab'
a=dblarr(3)
readf,1,a
table=a
lines=a[0]
weigth=a[1]
intensity=a[2]
WHILE NOT EOF(1) do begin
  readf,1,a
  table=[table,a]
  lines=[lines,a[0]]
  weigth=[weigth,a[1]]
  intensity=[intensity,a[2]]
ENDWHILE
close,1
a=size(table)

lambda=x*d_lambda+lambda_c-(Nx/2.)*d_lambda
lambda_beg=lambda(0)+50.*d_lambda
lambda_end=lambda(Nx-1)-d_lambda*50.
index=WHERE(lines gt lambda_beg and lines lt lambda_end)

lines=lines(index)
weigth=weigth(index)
intensity=intensity(index)

; creation full table spectrum in observed coverage
pos_previus=(lines-lambda_c)/d_lambda+Nx/2.
s=size(pos_previus)
N_lines=s(1)
message,'Number lines in observed spectral coverage'+string(n_lines),/cont

;****************************************
; creation tabulated comparison spectra
sigma=fwhm/2.35
x=dindgen(Nx)
neon_tab=dblarr(Nx)

; creation spectrum, filtered by LEVEL
 index=WHERE(weigth gt lev)   ; we use only lines with weight greater then level
 lines=lines(index)
 weigth=weigth(index)
 intensity=intensity(index)
 pos_previus=(lines-lambda_c)/d_lambda+Nx/2.
 s=size(pos_previus)
 N_lines=s(1)
 message,'Number lines used for dispersion curve creation'+string(n_lines),/cont
 message,'Spectral range:'+strcompress(string(min(lines))+'..'+string(max(lines))),/cont

 ;creation tabulated comparison spectra
 sigma=fwhm/2.35
 x=dindgen(Nx)
 neon_tab=dblarr(Nx)
 for k=0,N_lines-1 do begin
   gauss=(x-pos_previus(k))/sigma
   gauss=intensity(k)*exp(-gauss^2/2)
   neon_tab=neon_tab+gauss
 endfor

;*********************************************************

if keyword_set(manual) then begin
  nec=total(neon(*,fix(Ny/2)-10:fix(Ny/2)+10),2)
  conc=disperman(nec,lines(where(weigth ge 3. and intensity gt .5)),Ndeg=3)
endif
if keyword_set(plot) then loadct,27

;determination shifting central wavelength
m=80
x_shift=dblarr(Ny)

for y=0,Ny-1 do begin
  CRSPROD,neon(100:Nx-101,y),nec(100:Nx-101),m,CORR
  tmp=max(CORR,N_max)
  x_shift(y)=N_max-m
endfor

; Ny=256   ??

y=dindgen(Ny)
f=goodpoly(y,x_shift,2,tresh,Y_fit)
dy=dindgen(Ny)
dy=poly(y,f)
pos_lines=dblarr(N_lines,Ny)

for k=0,N_lines-1 do begin
  ; if not(keyword_set(manual)) then pos_lines(k,*)=pos_previus(k)+dy(*)
   if keyword_set(manual) then pos_lines(k,*)=poly(lines(k),conc)+dy(*) else $
pos_lines(k,*)=pos_previus[k]+dy(*)
endfor

; determination position of lines
spem=total(neon(*,fix(Ny/2)-10:fix(Ny/2)+10),2)
posm0=total(pos_lines,2)/double(Ny)
posm=posm0
; vector_tmp=dblarr(Nx)
nn=1
w=FWHM*nn
vector_tmp=dblarr(Nx*nn)
xx=dindgen(Nx*nn)
; Ny=256
set_plot,'x'
wkiller

for j=0,Ny-1 do begin
  vector_tmp=interpol(reform(med_ext(neon(*,j),3)),dindgen(Nx),dindgen(Nx*nn)/double(nn),/quadratic)     ; for given y 
;  vector_tmp=smooth(vector_tmp,2*nn+1)
  for k=0,N_lines-1 do begin
    pos=round(pos_lines(k,j)*nn)<(n_elements(vector_tmp)-1);!!!!!!!!!!!!! Olya -----
    pos=pos>0
    tmp=max(vector_tmp((pos-2*w)>0:(pos+2*w)<n_elements(vector_tmp)-1),Nmax)
    pos1=(Nmax+min(xx((pos-2*w)>0:(pos+2*w)<n_elements(vector_tmp)-1)))
    xx1=xx(pos1-3*w>0:pos1+3*w<n_elements(vector_tmp)-1)
    vec=vector_tmp(pos1-3*w>0:pos1+3*w<n_elements(vector_tmp)-1)
    xx2=xx(where(abs(xx-pos1) ge 3*w and abs(xx-pos1) le 5.*w))
    vec2=vector_tmp(where(abs(xx-pos1) ge 3*w and abs(xx-pos1) le 5.*w))
    lf=goodpoly(xx2,vec2,1.,2.)         
    g=[max(vec-poly(xx1,lf)),double(pos1),FWHM/2.35]
;    gf=gaussfit(xx1,vec-poly(xx1,lf)>0.,g,nterms=3,est=[max(vec-poly(xx1,lf)),double(pos1),(FWHM/2.35)*(lines[k]/5000.)])
    gf=lmfit(xx1,vec-poly(xx1,lf)>0.,g,fita=[1.,1.,0.],function_name='gau_ext',/double,convergence=conv,itmax=50,sigma=sig)
;   gau_neon=lmfit(x(wp),vec(wp)/max(vec(wp)),g,fita=[1.,1.,0.],function_name='gau_ext',/double,convergence=conv,itmax=50,sigma=sig)
;    if g[0] gt 0. and abs(g[1]-xx[pos]) le w and abs(g[2]) le w*5.*(lines[k]/5000.) then $
;             pos_lines(k,j)=g[1]  else pos_lines(k,j)=pos1
;;;;plot,xx1,vec-poly(xx1,lf)
;;;;oplot,xx1,gf,color=1000 
       if  conv eq 1 then pos_lines(k,j)=g[1] else pos_lines(k,j)=pos1
        

  endfor
endfor
pos_lines=pos_lines/double(nn)

y_out=Ny/2
vector=neon(*,y_out)*1000./max(neon(*,y_out))

set_plot,'ps'
device,filename=wdir+'neon_lines.ps',/landscape,xsize=25,ysize=20
plot,x,vector,xst=1,yst=1,yrange=[min(vector(100:Nx-100)),1.2*max(vector(100:Nx-100))],xtitle='position,pxls',ytitle='relat. intensity'
for k=0,N_lines-1 do begin
  pos=pos_lines(k,y_out)
  oplot,[pos,pos],[vector(pos)*1.01,vector(pos)*1.01+total(vector)/double(n_elements(vector))]
  xyouts,pos,vector(pos)*1.01+total(vector)/double(n_elements(vector)),string(lines(k),format='(F7.2)'),$
	/data,orientation=90,charsize=.6
endfor
device,/close
set_plot,'x'

rms=dblarr(Ny)
pos_lines_fit=pos_lines

;****************************************************
;creation array coefficients dispersion curves
disper_par=dblarr(N_deg+2,Ny)
rms_good=rms
ngood=intarr(Ny)

rmsl=dblarr(N_lines)
for kl=0,N_lines-1 do begin
  ftmp=goodpoly(y,pos_lines_fit(kl,*),2,2.,ftmp_fit)
  rmsl[kl]=stdev(pos_lines_fit(kl,*)-ftmp_fit)
endfor

plot,lines,rmsl
index0=where(rmsl le 1.*median(rmsl))

for y1=0,Ny-1 do begin
  f=goodpoly(double(pos_lines_fit(index0,y1)),double(lines(index0)),N_deg,tresh,ffit,xpk,ypk)
  disper_par(0:N_deg,y1)=f(*)
  ; lines_fit=0.
  lines_fit=poly(lines,f)
  rms(y1)=stdev(lines_fit-lines)
  rms_good(y1)=stdev(ypk-poly(xpk,f))
  ngood[y1]=n_elements(xpk)
endfor
;*************
if keyword_set(yfit) then begin
  y=indgen(Ny)
  for kkk=1,N_deg do begin
    dispars=goodpoly(double(y),disper_par(kkk,y),2,1.,ffit)
    disper_par(kkk,y)=poly(double(y),dispars)
  endfor
endif else begin
  y=indgen(Ny)
  for kkk=1,N_deg do begin
    diss=smooth(med_ext(reform(disper_par(kkk,y)),10),10)
 ;   disper_par(kkk,y)=replicate(median(disper_par(kkk,y)),Ny)
  endfor
endelse

rms=rms/d_lambda & rms_good=rms_good/d_lambda
disper_par(N_deg+1,*)=rms(*)
rms_mean=total(rms)/Ny & rms_mean_good=total(rms_good)/Ny
x_hist=dindgen(21)*0.025
; calculation and plot errors approximation
N_lin=N_elements(lines)
lines_calc=dblarr(N_lin,Ny)
for y=0,Ny-1 do begin
  ; lines_calc(*,y)=poly(lines,disper_par(0:N_deg,y))
  lines_calc(*,y)=interpol(poly(x,disper_par(0:N_deg,y)),x,pos_lines(*,y),/spline)
endfor

dy=2.0
index1=where(weigth ge 4.,num1)
set_plot,'PS'
device,file=wdir+'err_line.ps',/portrait,xsize=21,ysize=27,yoffset=0
plot,[0,Ny],[-1,num1]*dy,xst=1,yst=1,/nodata,charsize=0.75,$
	position=[0.03,0.1,0.7,0.95],/norm,$
	title='errors approximation dispersion curve  mean r.m.s='+$
	string(rms_mean*d_lambda,format='(F7.3)')+'('+string(rms_mean_good*d_lambda,format='(F7.3)')+') A',$
	xtitle='number of spectra',ytitle='!4Dk, !3px'

xyouts,0.1,0.05,out_str,/norm
xyouts,0.1,0.98,'number lines '+string(N_lines,format='(I2)')+$
	'(--> '+strtrim(string(double(total(ngood))/double(Ny)),2)+$
         ')  central wavelength '+string(lambda_c,format='(I4)')+$
	' A  dispersion '+string(d_lambda,format='(F5.2)')+$
	' A/px'+' Ndeg='+string(N_deg,format='(I2)'),/norm

for k=0,num1-1 do begin
  rms=stdev(-lines_calc(index1[k],*)+lines(index1[k]),mean)
  oplot,dindgen(Ny),(-lines_calc(index1[k],*)+lines(index1[k])-mean)/d_lambda+dy*k,psym=0,symsize=0.5,color=0
  oplot,[0,Ny],[dy,dy]*k,color=0
  xyouts,Ny*1.02,dy*k,string(lines(index1[k]),format='(F9.2)')+$
		string(mean,format='(F9.4)')+' !9+!3'+$
		string(rms,format='(F8.4)'),charsize=1.,/data
endfor
 xyouts,Ny*1.03,dy*(num1-0.5),'!4k, !3A',charsize=0.75,/data
 xyouts,Ny*1.12,dy*(num1-0.5),'!4Dk, !3A',charsize=0.75,/data
if sw eq 0 then device,/close
if sw eq 0 then begin
  if os_family() eq 'unix' then set_plot,'X' else set_plot,'WIN'
endif
;save dispersion curve
writefits,wdir+'disper.fts',double(disper_par)
a=size(disper_par)
;out_format='(F10.4,F12.10,'+string(a(1)-3,format='(I1)')+'E18.8,F8.4)'
Openw, UNIT,wdir+'disper.txt', /GET_LUN
for k=0,a[2]-1 do begin
  printf,UNIT,disper_par(*,k) ;,format=out_format
endfor
close, UNIT
message,'mean error dispersion curves ='+string(rms_mean,format='(F5.2)')+' px',/cont
disper_par=0
cont:
loadct,0
close,/all
END
