pro CREA_TRA,LOGFILE,ysh=ysh,star=star,old=old

;+
; NAME:
;	CREA_TRA
; PURPOSE:
;	creation traectory for every spectra
; DESCRIPTION: 
;	Routine interpolated traectory etalon spectra from file 'eta_tra.fts'
;	in working directory in position every fiber. Routine used table
;	position etalon fiber from file 'etalon.pos' and spectra fiber
;	(file 'fiber.pos'). Result save in file 'fib_tra.fts'
;	
; CALLING SEQUENCE:
;	CREA_TRA, LOGFILE
; CATEGORY:
;	reduction MPFS-data		
;
; INPUTS:
;	LOGFILE = file name of LOG observation (in format FITS-header)
;
; OUTPUTS:
;	saved in file 'fib_tra.fts'
;		
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;	no	
;
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;	Function :	DEF_WDIR
;
; MODIFICATION HISTORY:
;       Written by Victor Afanasiev, Special Astrophysical Observatory RAS, Jul 1999
;       Revised by students of the Stellar Physics Laboratory, SAO RAS
;-

on_error,0
message,'creation trajectory for every fiber',/cont
wdir=DEF_WDIR(LOGFILE)
eta_tra=READ_FTS(wdir+'eta_tra.fts',SILENT='silent')
if keyword_set(star) then eta_tra=READ_FTS(wdir+'eta_star_tra.fts',SILENT='silent')
eta_sig=READ_FTS(wdir+'eta_sig.fts',SILENT='silent')
if keyword_set(star) then eta_sig=READ_FTS(wdir+'eta_star_sig.fts',SILENT='silent')
c=size(eta_tra)
Nx=c(1)  

if not(keyword_set(old)) then OPENR, UNIT, '/bufm/mpfs.lib_2006/etalon.pos', /GET_LUN else OPENR, UNIT, '/bufm/mpfs.lib_2006/etalon_1.pos', /GET_LUN
a=' '
READF,UNIT,a
eta_pos=a                          ; eta_tra - traectories according to etalon
WHILE NOT EOF(UNIT) do begin        ; eta_pos - traectories according to 'ETALON.pos'
  READF,UNIT,a
  eta_pos=[eta_pos,a]
ENDWHILE
close, UNIT
FREE_LUN, UNIT
eta_pos=double(eta_pos)
a=size(eta_pos)

if not(keyword_set(old)) then OPENR, UNIT, '/bufm/mpfs.lib_2006/fiber_new.pos', /GET_LUN else OPENR, UNIT, '/bufm/mpfs.lib_2006/fiber_1.pos', /GET_LUN
a=double(0)
READF,UNIT,a
fib_pos=a
WHILE NOT EOF(UNIT) do begin
  READF,UNIT,a
  fib_pos=[fib_pos,a]
ENDWHILE
close, UNIT
FREE_LUN, UNIT

a=size(fib_pos)
N_fib=a(1)

if not(keyword_set(old)) then OPENR, UNIT, '/bufm/mpfs.lib_2006/etalon.pos', /GET_LUN else OPENR, UNIT, '/bufm/mpfs.lib_2006/etalon_1.pos', /GET_LUN
a=' '
READF,UNIT,a
sky_pos=a
WHILE NOT EOF(UNIT) do begin
  READF,UNIT,a
  sky_pos=[sky_pos,a]
ENDWHILE
close, UNIT
FREE_LUN, UNIT
sky_pos=double(sky_pos)
a=size(sky_pos)
N_sky=a(1)

traectory=dblarr(Nx,N_fib)  & tra_sky=dblarr(Nx,N_sky)
tra_flat=dblarr(Nx,N_fib+c(2))
traectory_sig=dblarr(Nx,N_fib)  & tra_sky_sig=dblarr(Nx,N_sky)
tra_flat_sig=dblarr(Nx,N_fib+c(2))


for x=0,Nx-1 do begin
 traectory(x,*)=INTERPOL(eta_tra(x,*),eta_pos,fib_pos)   ;,/spline interpolation from eta_pos to fib_pos
 traectory_sig(x,*)=INTERPOL(eta_sig(x,*),eta_pos,fib_pos)   ; ,/splineinterpolation from eta_pos to fib_pos
 tra_sky(x,*)=eta_tra(x,*)
 tra_sky_sig(x,*)=eta_sig(x,*)
 tra_flat(x,*)=[reform(eta_tra(x,*)),reform(traectory(x,*))]
 tra_flat(x,*)=tra_flat(x,sort(tra_flat(x,*)))
 tra_flat_sig(x,*)=[reform(eta_sig(x,*)),reform(traectory_sig(x,*))]
 tra_flat_sig(x,*)=tra_flat_sig(x,sort(tra_flat_sig(x,*)))
 ;if (x/100-x/100.) eq 0 then print,x
endfor

 fl=readfits(wdir+'flat_i.fts',hf,/sil)

af=size(fl)
Ny=af[2]

 flint=total(fl(floor(af[1]/4.):floor(af[1]/4.)*3,*),1)
 flint=flint/smooth(flint,10)
 fls=total(fl(floor(af[1]/4.):floor(af[1]/4.)*3,*),2)
 w=5.

if keyword_set(ysh) then begin
; obj=readfits(wdir+'obj_i.fts',hf)
; oint=total(obj,1)
 y=dindgen(Ny)
 dy=dindgen(N_fib+N_sky)
 for k=0,N_fib+N_sky-1 do begin
   mey=total(tra_flat(500:1500,k)*fls)/total(fls)   ;  mean maximum position
   yy=y(where(y gt mey-w and y lt mey+w))
   fey=smooth(flint(where(y gt mey-w and y lt mey+w)),3)
   plot,yy,fey
   maxf=max(fey,wmax)
   ceny=yy[wmax]
 ;  if abs(ceny-mey) le w/2. then 
   dy[k]=ceny-mey ; else dy[k]=0.
 end
; dyps=goodpoly(dindgen(N_fib+N_sky),med_ext(dy,10),3,3.,dyfit)
dyfit=smooth(med_ext(dy,20),20)
plot,dy
oplot,dyfit,color=1000
dy=dyfit
 dy_obj=dy(where(indgen(N_fib+N_sky) mod 17 ne 0))
 dy_sky=dy(where(indgen(N_fib+N_sky) mod 17 eq 0))

for xk=0,Nx-1 do begin
 tra_flat(xk,*)=tra_flat(xk,*)+dy 
 traectory(xk,*)=traectory(xk,*)+dy_obj
 tra_sky(xk,*)=tra_sky(xk,*)+dy_sky
endfor
endif
fsim=flint*0.
sigma=w/2.35
y=dindgen(Ny)

for k=0,N_fib+N_sky-1 do fsim=fsim+exp(-.5*((y-tra_flat(1000,k))/sigma)^2)

;          set_plot,'ps'
;           if not(keyword_set(star)) then device,filename=wdir+'tra_cor.ps',/color,/port else $
;                   device,filename=wdir+'tra_cor_star.ps',/color,/port
             loadct,0
            if not(keyword_set(old)) then  !p.multi=[0,1,3]
             plot,flint,xrange=[450,650]
             loadct,27
              oplot,fsim*(max(flint(450:650))-min(flint(450:650)))+min(flint(450:650)),color=1000
             loadct,0
if not(keyword_set(old)) then begin ;!!!!!!!!!!!!!!!!!!!!!!!!!!!!
             plot,flint,xrange=[950,1150],yst=16
             loadct,27
              oplot,fsim*(max(flint(950:1150))-min(flint(950:1150)))+min(flint(950:1150)),color=1000
             loadct,0
             plot,flint,xrange=[1250,1450]
             loadct,27
              oplot,fsim*(max(flint(1250:1450))-min(flint(1250:1450)))+min(flint(1250:1450)),color=1000
             loadct,0
endif
             !p.multi=0
;           device,/close
;          set_plot,'x'
; endif

if not(keyword_set(star)) then begin
 writefits,wdir+'fib_tra.fts',traectory
 writefits,wdir+'flat_tra.fts',tra_flat
 writefits,wdir+'sky_tra.fts',tra_sky
 writefits,wdir+'fib_sig.fts',traectory_sig
 writefits,wdir+'flat_sig.fts',tra_flat_sig
 writefits,wdir+'sky_sig.fts',tra_sky_sig
endif else begin
 writefits,wdir+'fib_tra_star.fts',traectory
 writefits,wdir+'flat_tra_star.fts',tra_flat
 writefits,wdir+'sky_tra_star.fts',tra_sky
 writefits,wdir+'fib_sig_star.fts',traectory_sig
 writefits,wdir+'flat_sig_star.fts',tra_flat_sig
 writefits,wdir+'sky_sig_star.fts',tra_sky_sig
endelse

traectory=0
end
