pro decont,incube,wdir,plot=plot,gf,sighmas,ndeg=ndeg,wint=wint,field=field,lrange=lrange

;+
; NAME:
;	DECONT
; PURPOSE:
;	creating simple polynomial approximation for object continuum; as a by-product, program calculates atmospheric
;           dispertion and overall spectrum of a source
; DESCRIPTION:
;       procedure calculates using approximation with 2d-gaussian the position of the object in continuum, and all the parameters     
;	for all the spectral range are extimated by polynomials from wavelength
;
; CALLING SEQUENCE:
;	DECONT,incube,wdir,out,plot=plot,gf,sighmas
; CATEGORY:
;	analizing MPFS-data
;
; INPUTS:
;	incube - initial file (normally 'obj_abs.fts')
;	wdir - working directory
;
; OUTPUTS:
;	results are sent to wdir+'obj_decont.fts' and 'cont.fts' - object without continuum and continuum, respectively.
;	Also, overall spectrum is plotted in 'flux.ps', center positions of monochromatic images are plotted in 'centers.ps'
;	and object monochromatic halphwidths - into 'width.ps', is keyword /plot is set.
;	
;	gf - polynomial approximations for 2d-gaussian parameters
;	sighmas - rms estimations for gf and for flux in continuum
;
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;	plot - plotting flag for PS-files
;
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;	Function :   READFITS
;
; MODIFICATION HISTORY:
;       Written by Pavel Abolmasov, Special Astrophysical Observatory RAS, Jan 2004
;-
 
if not(keyword_set(field)) then field=[16,16]

  if not(keyword_set(ndeg)) then ndeg=3
  data=double(readfits(wdir+incube,h))
    dl=sxpar(h,'CDELT1')  &  l0=sxpar(h,'CRVAL1')
  a=size(data)
    l=dindgen(a[1])*dl+l0

  cube=reform(data,a[1],field[0],field[1])
  cenx=dblarr(a[1])  &  ceny=dblarr(a[1])  &  dest=dblarr(a[1])  &  flux=dblarr(a[1])
  bgd=dblarr(a[1])
  
x=reform(dindgen(field[0]*field[1]) mod field[0],field[0],field[1])
y=reform((dindgen(field[0]*field[1])-x)/field[0],field[0],field[1])
www=-1
  for k=0,a[1]-1 do begin
    im=reform(cube(k,*,*))
    bgd[k]=bormed(im)  &  flux[k]=total(im-bgd[k])
    cenx[k]=total((im-bgd[k])*x)/flux[k]  &  ceny[k]=total((im-bgd[k])*y)/flux[k]
    dest[k]=sqrt(total(((x-cenx[k])^2+(y-ceny[k])^2)*im/flux[k]))
    imfit=gauss2dfit(im/total(im),g)
    bgd[k]=g[0]  &  cenx[k]=g[4]  &  ceny[k]=g[5]
    dest[k]=sqrt((g[2])^2+(g[3])^2)/sqrt(2.)
    if finite(dest[k]) and dest[k] lt max([field[0],field[1]])/2. and dest[k] gt 0. and flux[k] gt 0. then if min(www) ge 0 then www=[www,k] else www=k
  endfor
  
www1=www(where(abs(l(www)-6563.) gt 30.))
ll=interpol(l(www1),double(www1),double(www))
fluxl=interpol(flux(www1),double(www1),double(www))

 flux_fit=lowess(ll,fluxl,200,1)
 cenx_pars=goodpoly(ll,cenx(www1),2,3.,cenx_fit0,newx,newy)
 ceny_pars=goodpoly(ll,ceny(www1),2,3.,ceny_fit0,newx,newy)
 dest_pars=goodpoly(ll,dest(www1),2,3.,dest_fit0,newx,newy)
 cenx_fit=poly(l,cenx_pars) & ceny_fit=poly(l,ceny_pars)
 dest_fit=poly(l,dest_pars)  

  outcube=cube

  for i=0,a[1]-1 do outcube(i,*,*)=$
   exp(-(((x-cenx_fit[i])/dest_fit[i])^2+((y-ceny_fit[i])/dest_fit[i])^2)*.5)*flux_fit[i]/(!DPI*2.*dest_fit[i]^2)
;  cube=reform(cube,a[1],a[2])
  outcube=reform(outcube,a[1],a[2])
  writefits,wdir+'obj_decont.fts',data-outcube,h
  
  writefits,wdir+'cont.fts',outcube,h
end
