function destroy,array,maska,level1,level2,plot=plot,thresh1,thresh2,wdir=wdir
;+
; NAME:
;	DESTROY
; PURPOSE:
;       removing hits from spectra avoiding spectral lines
;	
; DESCRIPTION:
;       all the considered area is devided into three regions, defined by the current running 
;       median value of the raw object.
; CALLING SEQUENCE:
;	DESTROY, ARRAY, mask, level1, level2
;
; CATEGORY:
;	reduction MPFS-data
;
; INPUTS:
;	ARRAY - input 2D array (raw image)
;       mask - the array that should contain the residuals 
;       level1 > level2, the CH-candidate situated in the area with average intensity > level1
;       is omitted, otherwise it will be cleaned by harder or softer filter depending on the 
;       average intencity
; OUTPUTS:
;	cleaned array
;       mask - array of esiduals
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;       no
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;       med_ext2
; MODIFICATION HISTORY:
;       Written by Pavel Abolmasov, Special Astrophysical Observatory RAS, Jul 2003
;      revised by A.Shipulin Special Astrophysical Observatory RAS, Jul 2003
;
;-

 mar=total(array)/double(n_elements(array))  ;;; mean value
; dar=sqrt(total((array-mar)^2)/double(n_elements(array)))

; darray=array
 a=size(array)

sar=sfit(array,2)

;; +-shaped spatial filter

 medar=array-sar
 for k=0,a[2]-1 do medar(*,k)=smooth(med_ext(reform(array(*,k)),20)/2.,20.)
 for i=0,a[1]-1 do medar(i,*)=medar(i,*)+smooth(med_ext(reform(array(i,*)),20)/2.,20.)
; medar=medar-smooth(medar,50)

; dar=sqrt(total((array-medar)^2)/double(n_elements(array)))
mint=total(medar-median(medar)>0)/double(n_elements(medar))
                        ; Wohl koennen wir besser Medianfilter zu machen?
darray=array
wall=where(array gt medar+thresh1*sqrt(medar > 10.) and medar le level1*mint)
wall=[wall,where(array gt medar+thresh2*sqrt(medar) and medar gt level1*mint and $
            medar le level2*mint )]

; *** control output

ww=0.*darray
if n_elements(where(medar ge level2*mint)) gt 1 then ww(where(medar ge level1*mint))=10
if n_elements(where(medar ge level2*mint)) gt 1 then ww(where(medar ge level2*mint))=20

set_plot,'ps'
 device,filename=wdir+'destroy.eps',/color,/enc,xoffset=0,yoffset=28,/land,ysize=20,xsize=28
pos0=[1.,1.,27.,19.]*1000.

loadct,0,/sil
 contour,rebin(ww(10:10*floor(a[1]/10.-1)-1,10:10*floor(a[2]/10.-1)-1),floor(a[1]/10.-2),$
   floor(a[2]/10.-2)),indgen(floor(a[1]/10.-2))*10+10.,indgen(floor(a[2]/10.-2))*10+20.,$
   pos=pos0,/dev,lev=[0,5,15],/xst,/yst,/fill,charthick=3.,charsize=1.5
loadct,27,/sil
 contour,rebin(sar(10:10*floor(a[1]/10.-1)-1,10:10*floor(a[2]/10.-1)-1),floor(a[1]/10.-2),$
   floor(a[2]/10.-2)),indgen(floor(a[1]/10.-2))*10+10.,indgen(floor(a[2]/10.-2))*10+20.,$
   pos=pos0,/dev,xst=5,yst=5,nlev=15,/noer,c_color=190,c_thick=3.,/down
loadct,0,/sil
 device,/close
set_plot,'x'
sar=0.
; ***

ww=0.*darray
ww(wall)=1.
 ww=ww+(shift(ww,-1,-1)+shift(ww,1,-1)+shift(ww,-1,1)+shift(ww,1,1)+shift(ww,0,-1)+shift(ww,1,0)+shift(ww,0,1)+shift(ww,-1,0))
ww=ww+shift(ww,-1,-1)+shift(ww,1,-1)+shift(ww,-1,1)+shift(ww,1,1)+shift(ww,0,-1)+shift(ww,1,0)+shift(ww,0,1)+shift(ww,-1,0)

wall=[where(ww gt 0.),where(array gt medar+10.*thresh2*sqrt(medar > 20.))]
wall=wall(uniq(wall))
darray(wall)=medar(wall)

maska=array-darray
; ** under reconstruction!!  **

 return,darray
end
