function diaspect,datacube,w,lambda=lambda,lampar=lampar,nosh=nosh,wira=wira,adr=adr
;+
; NAME:
;	DIASPECT
; PURPOSE:
;       extracting single spectrum from a diaphragm with center given by x and y and
;	radius r (polynomial approximation for shifting due to atmosphaeric dispertion is used if keyword nosh 
;	not set)
;	
; DESCRIPTION:
;       no comments
; CALLING SEQUENCE:
;	result = diaspect(datacube,w)
;
; CATEGORY:
;	reduction MPFS-data
;
; INPUTS:
;	datacube - 2D array
;       x and y are the diaphragm center coordinates, w is it's radius 
; OUTPUTS:
;	1D spectrum
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;       no
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;        
; MODIFICATION HISTORY:
;       Written by Pavel Abolmasov, Special Astrophysical Observatory RAS, Jul 2003
;-      revised by A.Shipulin Special Astrophysical Observatory RAS, Jul 2003

on_error,0
  if not(keyword_set(nosh)) then ifsh=1. else ifsh=0.

  a=size(datacube)
  if a[0] lt 3 then datacube=reform(datacube,a[1],16,a[2]/16)
  a=size(datacube)
  if not(keyword_set(lampar)) and keyword_set(lambda) then message,'dispersion parameters needed'
   if keyword_set(lampar) then begin
      l0=lampar[0] 
      dl=lampar[1]
   endif
  
  xs=0
if not(keyword_set(wira)) then begin
   if not(keyword_set(lambda)) then read,xs,prompt='input slice position (pxls, 0 to '+strtrim(string(a[1]-1))+')'$
 else xs=(lambda-l0)/dl
endif else begin
  ra=(wira-l0)/dl
endelse

n=20.
wkiller
if not(keyword_set(wira)) then slice0=total(datacube(xs-5:xs+5,*,*),1) else slice0=total(datacube(ra[0]:ra[1],*,*),1)
; slice00=med_ext2(slice0,5)
; slice0(where(finite(slice0,/inf)))=slice00(where(finite(slice0,/inf)))
slice0=rotate(rebin(slice0,a[2]*n,a[3]*n),7)
window,/free,xsize=n*a[2],ysize=n*a[3]
slice0_norm=double((slice0-min(slice0)))/double(max(slice0-min(slice0)))
contour,smooth(slice0_norm,2)/max(smooth(slice0_norm,3)),dindgen(n*a[2])/double(n),dindgen(n*a[3])/double(n),color=-1,levels=dindgen(100)/double(100.),/downhill,/fill
;tvscl,rotate(smooth(slice0_norm,2),7)

x=double(0)  &  y=double(0)

    ret:
     cursor,x,y,4,/data
     m=!MOUSE
     sw=long(m.button)                                         ; mouse click-type  
  
     if sw eq 0 then goto,ret
     if sw eq 1 then begin
       c=[x,y]
       message,'got it! '+strcompress(string(c[0])+string(c[1])),/cont   
       goto,ret
     endif

if sw eq 2 then message,'spectrum extraction interrupted'

x0=c[0] & y0=c[1]

x0=n*x0  & y0=y0*n
spect=dblarr(a[1])
xa=rebin(dindgen(a[3]*n),a[2]*n,a[3]*n,/sample)
ya=transpose(xa)
dx=dblarr(a[1])  & dy=dblarr(a[1])

slice0_int=slice0
sf_tmp=sfit(slice0,1)
slice0_int(where((xa-x0)^2+(ya-y0)^2 lt (3.*w*double(n))^2))=sf_tmp(where((xa-x0)^2+(ya-y0)^2 lt (3.*w*double(n))^2))
slice0=slice0-sfit(slice0_int,1)
fit0=gauss2dfit(slice0,g0)

www=where(indgen(a[1]) mod 20 eq 5)

for xl=0,a[1]-1 do begin
  slice=smooth(rotate(rebin(reform(datacube(xl,*,*)),a[2]*n,a[3]*n),7),15)
  slice_int=slice
 if xl mod 20 eq 5 then begin
  sf_tmp=sfit(slice,1)
  slice_int(where((xa-x0)^2+(ya-y0)^2 lt (3.*w*double(n))^2))=sf_tmp(where((xa-x0)^2+(ya-y0)^2 lt (3.*w*double(n))^2))
  slice=slice-sfit(slice_int,1)
  fit=gauss2dfit(slice,g)
  dx[xl]=double(g[4]-g0[4])
  dy[xl]=double(g[5]-g0[5])
  print,xl,':',dx[xl],dy[xl]
 endif
endfor
dx=interpol(dx(www),double(www),dindgen(a[1]))
dy=interpol(dy(www),double(www),dindgen(a[1]))

fdx=goodpoly(dindgen(a[1]),dx,2,3.,dxfit,newzx,newdx)
fdy=goodpoly(dindgen(a[1]),dy,2,3.,dyfit,newzy,newdy)
fdx=goodpoly(newzx,newdx,1,1.,dxfit,newz,newdx)
fdy=goodpoly(newzy,newdy,1,1.,dyfit,newz,newdy)
dxfit=poly(dindgen(a[1]),fdx)
dyfit=poly(dindgen(a[1]),fdy)
for xl=0,a[1]-1 do begin
  slice=rotate(rebin(reform(datacube(xl,*,*)),a[2]*n,a[3]*n),7)
;  slice=slice-bormed(slice)
  x=x0+dxfit[xl]*ifsh  &  y=y0+dyfit[xl]*ifsh
  r=(xa-x)^2+(ya-y)^2
  slice=slice-median(slice(where(r gt (w*double(n))^2)))
  spect[xl]=total(slice(where(r le (w*double(n))^2)))/double(n)^2
  tvscl,slice*((r lt (w*double(n)+1.)^2)+(r gt (w*double(n)-1.)^2))+100.*(r lt (w*double(n)+1.)^2 and r gt (w*double(n)-1.)^2)
  if keyword_set(lampar) then xyouts,.8,.05,strtrim(string(l0+dl*double(xl),format='(F10.2)'),2)+'A',/norm,charsize=1.,color=-1
endfor
; spect=med_ext(spect,5)
 adr=sqrt(fdx[1]^2+fdy[1]^2)
 return,spect
end
