function diaspect,datacube,w,lambda=lambda,lampar=lampar,nosh=nosh,wira=wira,adr=adr,adang=adang,cent=cent,$
       nobgd=nobgd,wdir=wdir,ptb=ptb,ndeg=ndeg,old=old
;+
; NAME:
;	DIASPECT
; PURPOSE:
;       extracting single spectrum from a diaphragm with center given by x and y and
;	radius r (polynomial approximation for shifting due to atmosphaeric dispertion is used if keyword nosh 
;	not set)
;	
; DESCRIPTION:
;       no comments
; CALLING SEQUENCE:
;	result = diaspect(datacube,w)
;
; CATEGORY:
;	reduction MPFS-data
;
; INPUTS:
;	datacube - 2D array
;       x and y are the diaphragm center coordinates, w is it's radius 
; OUTPUTS:
;	1D spectrum
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;       no
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;        
; MODIFICATION HISTORY:
;       Written by Pavel Abolmasov, Special Astrophysical Observatory RAS, Jul 2003
;-      revised by A.Shipulin Special Astrophysical Observatory RAS, Jul 2003

on_error,0
;if not(keyword_set(ptb)) then begin
  f_tmp=readfits(wdir+'obj-sky.fts',htmp)
 if not(keyword_set(ptb)) then ptb=sxpar(htmp,'ROTANGLE')
 if not(keyword_set(ndeg)) then ndeg=2
;endif
z=sxpar(htmp,'REDSHIFT')
ang=(37.-ptb)*!DPI/180.

if not(keyword_set(old)) then field=[16,16] else field=[16,15]
  if not(keyword_set(nosh)) then ifsh=1. else ifsh=0.
  if not(keyword_set(wdir)) then wdir='/bufm/mpfs_tmp/ngc6946/'
  a=size(datacube)
  if a[2] ne field[0]*field[1] and a[0] lt 3 then field[1]=nint(a[2]/field[0])
  if a[0] lt 3 then datacube=reform(datacube(*,a[2]-field[0]*field[1]:a[2]-1),a[1],field[0],field[1])
   if a[0] eq 3 then field=[a[2],a[3]]
  a=size(datacube)
  if not(keyword_set(lampar)) and keyword_set(lambda) then message,'dispersion parameters needed'
   if keyword_set(lampar) then begin
      l0=lampar[0] 
      dl=lampar[1]
   endif

if not(keyword_set(old)) then field=[16,16] else field=[16,15]
  

  xs=0
if not(keyword_set(wira)) then begin
   if not(keyword_set(lambda)) then read,xs,prompt='input slice position (pxls, 0 to '+strtrim(string(a[1]-1))+')'$
 else xs=(lambda*(z+1.)-l0)/dl
endif else begin
  ra=(wira-l0)/dl
endelse

lev=[1.e-4,1.e-3,1.e-2,1.e-1,0.3,0.5,0.8,1.,1.5,2.]
lev=smooth(rebin(lev,n_elements(lev)*2),3)
lev=lev(uniq(lev))

n=20.
wkiller
if not(keyword_set(wira)) then slice0=total(datacube(xs-5:xs+5,*,*),1)-total(datacube((xs-10)>0:(xs+10)<(a[1]-11),*,*),1)/10. else slice0=total(datacube(ra[0]:ra[1],*,*),1)
; slice00=med_ext2(slice0,5)
; slice0(where(finite(slice0,/inf)))=slice00(where(finite(slice0,/inf)))
slice0=rotate(rebin(slice0,a[2]*n,a[3]*n),7)
loadct,33
window,/free,xsize=n*a[2],ysize=n*a[3],retain=2
slice0_norm=double((slice0-min(slice0)))/double(max(slice0-min(slice0)))
contour,smooth(slice0_norm,3),dindgen(n*a[2])/double(n),dindgen(n*a[3])/double(n),levels=lev,$
  /fill,xst=1,yst=1,pos=[0,0,n*a[2],n*a[3]],/dev
;tvscl,rotate(smooth(slice0_norm,2),7)
plotsym,0,4.*(w),color=-1,thick=3
x=double(0)  &  y=double(0)

    ret:
     cursor,x,y,4,/data
     m=!MOUSE
     sw=long(m.button)                                         ; mouse click-type  
  
     if sw eq 0 then goto,ret
     if sw eq 1 then begin
       c=[x,y]
;       plots,x,y,/data
contour,smooth(slice0_norm,3),dindgen(n*a[2])/double(n),dindgen(n*a[3])/double(n),levels=lev,/downhill,$
  /fill,xst=1,yst=1,pos=[0,0,n*a[2],n*a[3]],/dev
loadct,33
      ; oplot,[c[0]],[c[1]],color=1000,psym=8
plotsym,0,4.*(w),color=-1,thick=3
  plots,c[0],c[1],psym=8,/data,color=200
plotsym,0,4.*(w)*2.,thick=3
  plots,c[0],c[1],psym=8,/data,color=50000
plotsym,0,4.*(w)*3.,thick=3
  plots,c[0],c[1],psym=8,/data,color=50000
plotsym,0,4.*(w),color=-1,thick=3
loadct,0
       message,'got it! '+strcompress(string(c[0])+string(c[1])),/cont   
       goto,ret
     endif

if sw eq 2 then message,'spectrum extraction interrupted'

x0=c[0] & y0=c[1]

x0=n*x0  & y0=y0*n
spect=dblarr(a[1])
xa=rebin(dindgen(a[2]*n),a[2]*n,a[3]*n,/sample)
ya=transpose(xa)
dx=dblarr(a[1])  & dy=dblarr(a[1])
if not(keyword_set(nosh)) then begin
 slice0_int=slice0
 sf_tmp=sfit(slice0,1)
 slice0_int(where((xa-x0)^2+(ya-y0)^2 lt (3.*w*double(n))^2))=sf_tmp(where((xa-x0)^2+(ya-y0)^2 lt (3.*w*double(n))^2))
 slice0=slice0-median(slice0)            ;-sfit(slice0_int,1)
 fit0=gauss2dfit(slice0,g0)

 www=where(indgen(a[1]) mod 20 eq 5)

 for xl=0,a[1]-1 do begin
   ; slice_tmp=med_ext2(reform(datacube(xl,*,*)),3)
   slice_tmp=reform(datacube(xl,*,*))
   slice=smooth(rotate(rebin(slice_tmp,a[2]*n,a[3]*n),7),1.*n)
   slice_int=slice
  if xl mod 20 eq 5 then begin
   sf_tmp=sfit(slice,1)
   slice_int(where((xa-x0)^2+(ya-y0)^2 lt (3.*w*double(n))^2))=sf_tmp(where((xa-x0)^2+(ya-y0)^2 lt (3.*w*double(n))^2))
 ;  slice=slice-sfit(slice_int,1)
   slice=slice-median(slice(where((xa-x0)^2+(ya-y0)^2 ge (w*double(n))^2)))
   slice(where((xa-x0)^2+(ya-y0)^2 ge (2.*w*double(n))^2))=0.
   fit=gauss2dfit(slice,g)
   dx[xl]=double(g[4]-g0[4])
   dy[xl]=-double(g[5]-g0[5])
   print,xl,':',dx[xl],dy[xl]
  endif
 endfor
 dx=interpol(dx(www),double(www),dindgen(a[1]))
 dy=interpol(dy(www),double(www),dindgen(a[1]))

dd=-dx*sin(ang)+dy*cos(ang)

;dxfit=dx
;dyfit=dx
;www=www(where((dx(www))^2+(dy(www))^2 le 20.*n^2))
;fdx=goodpoly(double(www),dx(www),1,3.,ddxfit,newzx,newdx)
;fdy=goodpoly(double(www),dy(www),1,3.,ddyfit,newzy,newdy)
;fdx=goodpoly(newzx,newdx,1,2.,ddxfit,newzx,newdx)
;fdy=goodpoly(newzy,newdy,1,2.,ddyfit,newzy,newdy)

fdd=goodpoly(double(www),dd(www),2,2.,ddxfit,newzx,newdx)

;dxfit=poly(dindgen(a[1]),fdx)
;dyfit=poly(dindgen(a[1]),fdy)

ddfit=poly(dindgen(a[1]),fdd)/n

dxfit=-ddfit*sin(ang)
dyfit=ddfit*cos(ang)

endif else begin
  dxfit=replicate(0.,a[1])
  dyfit=replicate(0.,a[1])
endelse

loadct,0

slice_arr=dblarr(a[2]*n,a[3]*n,3)
; openw,1,'~/trash/dia.jpg'
; window,/free,xsize=n*a[2],ysize=n*a[3]
;dia=mpeg_open([a[2],a[3]]*n,mot=3,qua=100)
slitmp=dblarr(3,a[2]*n,a[3]*n)

;set_plot,'ps'
;device,filename='~/trash/dia.ps',/color,xsize=a[2],ysize=a[3],/port
; loadct,33
 for xl=0,a[1]-1 do begin
  slice=rotate(rebin(reform(datacube(xl,*,*)),a[2]*n,a[3]*n),7)
;  slice=slice-bormed(slice)
  x=x0+dxfit[xl]  &  y=y0+dyfit[xl]
  r=(xa-x)^2+(ya-y)^2
  if not(keyword_set(nobgd)) then slice=slice-median(slice(where(r ge (2.*w*double(n))^2 and r le (3.*w*double(n))^2)))  ;; colechko
  spect[xl]=total(slice(where(r le (w*double(n))^2)))/double(n)^2
  if xl mod 20 eq 0 then begin
;slitmp(0,*,*)=double(xl)*slice*((r lt (w*double(n)+1.)^2)+(r gt (w*double(n)-1.)^2))+100.*(r lt (w*double(n)+1.)^2 and r gt (w*double(n)-1.)^2)
;slitmp(1,*,*)=double(abs(xl-a[1]))*slitmp(0,*,*)
;slitmp(2,*,*)=double(abs(xl-a[1]))*slitmp(0,*,*)/double(xl)
;  mpeg_put,dia,ima=bytscl(slitmp),frame=round(xl/20.),/color
;   tvscl,slice*((r lt (w*double(n)+1.)^2)+(r gt (w*double(n)-1.)^2))+100.*(r lt (w*double(n)+1.)^2 and r gt (w*double(n)-1.)^2),$
;       .05,.05,xsize=.9,ysize=.9,/norm
;   contour,slice,xa/n,ya/n,/nodata,/noerase,pos=[.05,.05,.95,.95]*1000.
;  if keyword_set(lampar) then xyouts,.8,.05,strtrim(string(l0+dl*double(xl),format='(F10.2)'),2)+'A',/norm,charsize=1.
 ;  slice_arr(*,*,0)=slice
 ;  slice_arr(*,*,1)=slice ;median(slice^2)/slice
 ;  slice_arr(*,*,2)=slice ;^2/median(slice)
 ;  write_jpeg,unit=1,bytscl(slice_arr),true=3
 endif
endfor
;mpeg_save,dia,file="/users/maryeva/V532/mp041112res/mpres"

;mpeg_close,dia

;device,/close
;set_plot,'x'
; close,1
; spect=med_ext(spect,5)
 if keyword_set(adr) then begin
   adr=[fdd[1],fdd[2]]/n         ; [sqrt(fdx[1]^2+fdy[1]^2),sqrt(fdx[2]^2+fdy[2]^2)*(sign(fdx[2])+sign(fdy[2]))/2.]/double(n)
 endif

if not(keyword_set(nosh)) then begin 
set_plot,'ps'
device,filename=wdir+'ADR.ps',xsize=20,ysize=25,/portrait
  !p.multi=[0,1,3]
  loadct,3
; tvscl,slice0,xsize=16,ysize=16,/cent
 contour,smooth(-slice0,5),xa/n,ya/n,/fill,/iso,/irr,xst=1,yst=1,nlevels=20.,max_value=2.*max(smooth(-slice0,5))
   plotsym,0,4,color=1
   plots,x0/n,y0/n,psym=8,/data,color=-1,symsize=2.5,thick=3.

 arrow,(x0+dxfit[0]*5.)/double(n),(y0+dyfit[0]*5.)/double(n),(x0+dxfit[a[1]-1]*5.)/double(n),$
         (y0+dyfit[a[1]-1]*5.)/double(n),/data,/solid,color=-1
 drawaz,12,12,-37.+ptb,r=2.    

loadct,0
  plot,www,dx(www)/double(n),psym=2,xtitle='pixel no',ytitle='dx, arcsec',yrange=[min(dxfit),max(dxfit)],title='x-direction ADR approximation'
  oplot,dxfit

  plot,www,dy(www)/double(n),psym=2,xtitle='pixel no',ytitle='dy, arcsec',yrange=[min(dyfit),max(dyfit)],title='y-direction ADR approximation'
  oplot,dyfit
  
  if keyword_set(adang) then xyouts,.5,.9,'adr angle '+strtrim(string(adang*!radeg,format='(F7.2)'),2),/norm
  !p.multi=0
device,/close
set_plot,'x'
endif
cent=[x,y]/double(n)
 return,spect
end
