pro FLAT_CUBE,LOGFILE,image_type,post  ;,scatter=scatter
;
;+
; NAME:
;	FLAT_REDUCTION
; PURPOSE:
;	reduction MPFS-image at flat field
; DESCRIPTION:
;	Routine read in working directory flat-field frame from file 'flat_s.fts' (extracted flat-field datacube)
;	Image with type '*' read from file '*_s.fts'
;	Then the image is divided by the normalized flat-field. 
;
;
;
; CALLING SEQUENCE:
;	FLAT_REDUCTION, LOGFILE, image_type,post
;
; CATEGORY:
;	reduction MPFS-data
;
; INPUTS:
;	LOGFILE = file name of LOG observation (in format FITS-header)
;	image_type = string scalar type exposure (values: 'obj','star','flat',
;		   'eta','star','test');
;	scatter - flag for scattered light level determination
;	post - postfix '_s', '_ns' or '_os' determining which fibers are considered
; OUTPUTS:
;	Result saved to disk in working directory in file '*'+post+'f.fts'
;
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;	no
;
; RESTRICTIONS:
;	no
; WARNING:
;	scattered light on the current stage was treated as a source of constant brightness, 
;	not depending on the origin of the light and the brightness of the source. 
;	Evidently, it's not a very reasonable approach. INDEEEEEED! Now is all over, the problem of scattered light being
;	yet present. author supposes it'll be useful to estimate scattered light level if every frame separately,
;       using the shape extracted from the file 'eta_i', renormed to current image using strips of scattered light below 
;	and above fiber trajectpries.
;
; PROCEDURES USED:
;	Function  DEF_WDIR,READ_FTS, SCATTER
;	Procedure SXADDPAR
;
; MODIFICATION HISTORY:
;
;	Written for datacubes by P. Abolmasov and Antony Shi-Pulin in 2003-2005.
;-

on_error,0
MESSAGE,'reduction '+image_type+' at flat field',/cont
wdir=DEF_WDIR(LOGFILE)
if image_type eq 'star' or image_type eq 'test' then flat=READFITS(wdir+'flatstar'+post+'.fts',hflat) $
  else flat=READFITS(wdir+'flat'+post+'.fts',hflat)

a=size(flat)
Nx=a(1)  & Ny=a(2)
vector=dblarr(Nx)
w=2
N_pos=Nx/20.
x_pos=(dindgen(N_pos)+1)*20.
y_pos=dblarr(N_pos)
b=where(flat le 0,index)
if index ne 0 then flat(b)=1 & message,string(index)+' unreliable points in flat-field!',/cont

flame=flat*0.
for y=0,a[2]-1 do begin
  flat(*,y)=med_ext(flat(*,y),10)
  flame(*,y)=smooth(flat(*,y),20)
endfor
for x=0,a[1]-1 do flat(x,*)=flat(x,*)/max(flame(x,*))

if strpos(image_type+post,'starstar') lt 0 then image=READFITS(wdir+image_type+post+'.fts',himage) else $
  image=READFITS(wdir+post+'.fts',himage)
imexp=double(sxpar(himage,'EXPTIME'))

; if image_type ne 'neon' then 
image=image/flat
sxaddpar,himage,'FILE',image_type+post+'f.fts'
writefits,wdir+image_type+post+'f.fts',image,himage
set_plot,'X'
; window,1
; plot,flat(500,*),xst=1,yst=1,tit='Normalised FLAT'
end
