pro FLAT_REDUCTION,LOGFILE,image_type,scatter=scatter
;
;+
; NAME:
;	FLAT_REDUCTION
; PURPOSE:
;	reduction MPFS-image at flat field
; DESCRIPTION:
;	Routine read in working directory flat-field frame from file 'flat_i.fts'
;	From the etalon image scattered light is extracted, end frame normalised.
;	Image with type '*' read from file '*_i.fts', scattered light is subtracted 
;	from the image, and then the image is divided by the normalized flat-field. 
;	Routine use for calculation scattering ligth file 'eta_tra.fts'
;
;
; CALLING SEQUENCE:
;	FLAT_REDUCTION, LOGFILE, image_type
;
; CATEGORY:
;	reduction MPFS-data
;
; INPUTS:
;	LOGFILE = file name of LOG observation (in format FITS-header)
;	image_type = string scalar type exposure (values: 'obj','star','flat',
;		   'eta','star','test');
;	scatter - flag for scattered light level determination
; OUTPUTS:
;	Result saved to disk i working directory in file '*_n.fts'
;
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;	no
;
; RESTRICTIONS:
;	no
; WARNING:
;	scattered light on the current stage is treated as a source of constant brightness, 
;	not depending on the origin of the light and the brightness of the source. 
;	Evidently, it's not a very reasonable approach. INDEEEEEED!
;
; PROCEDURES USED:
;	Function  DEF_WDIR,READ_FTS, SCATTER
;	Procedure SXADDPAR
;
; MODIFICATION HISTORY:
;       Written by Victor Afanasiev, Special Astrophysical Observatory RAS, Jul 1999
;       Modified by Pavel Abolmasov and Antony Shipulin, Special Astrophysical Observatory RAS, Aug 2003
;       Integral flux normalization added by P. Abolmasov and A. Shipulin, Special Astrophysical Observatory RAS, Jan 2004
;-

on_error,0
MESSAGE,'reduction '+image_type+' at flat field',/cont
wdir=DEF_WDIR(LOGFILE)
etalon=READFITS(wdir+'eta_i.fts',heta)
flat=READFITS(wdir+'flat_i.fts',hflat)

if image_type eq 'star' or image_type eq 'test' then begin
 flat=READFITS(wdir+'flatstar_i.fts',hflat)
 etalon=READFITS(wdir+'eta_star_i.fts',heta)
endif

;eta_traectory=READFITS(wdir+'eta_tra.fts',hetra)
etexp=double(sxpar(heta,'EXPTIME'))
flexp=double(sxpar(heta,'EXPTIME'))
;subtraction scattered light in flat image

scalight=SCATTER(etalon)     ; scattered light (obtained from the etalon image, only)

writefits,'flat_sca.fts',scalight
;normalisation flat field

if keyword_set(scatter) then flat=flat-scalight*(total(flat)/total(etalon))

if total(where(flat le 0)) ge 0 then message,'FUCKING ZERO FLAT FIELD!!! (don`t tell me that`s unprintable)',/cont

;flat=flat+abs(flat)

a=size(flat)
Nx=a(1)  & Ny=a(2)
vector=dblarr(Nx)
w=10.
N_pos=Nx/20.
x_pos=(dindgen(N_pos)+1)*20.
y_pos=dblarr(N_pos)
b=where(flat le 0,index)
if index ne 0 then flat(b)=1

for x=0,Nx-1 do begin
  im=reform(flat(x,Ny/2-2*w:Ny/2+2*w))
  vector(x)=total(median(im,5))/(4*w+1)            ; averaged flat-field value in the central row of the image...
endfor

for x=0,N_pos-1 do begin
  y_pos(x)=total(vector(x_pos(x)-w/2:x_pos(x)+w/2))/(w+1)     ; flat-field value for the interpolation points
endfor

x=dindgen(Nx)
f=interpol(y_pos,x_pos,x)

for y=0,Ny-1 do begin
  flat(*,y)=double(flat(*,y)*max(vector)/vector)
  ;flat(*,y)=float(median(reform(flat(*,y)),5)*max(vector)/vector)
endfor

;flat=flat/max(flat)
mp=max(vector,mpos)
flat=flat/max(median(reform(flat(mpos,*)),20))

;reduction image at flat field
image=READFITS(wdir+image_type+'_i.fts',himage)
imexp=double(sxpar(himage,'EXPTIME'))

if keyword_set(scatter) then image=image-scalight*(total(image)/total(etalon))

if image_type ne 'neon' then image=image ;/flat
sxaddpar,himage,'FILE',image_type+'_n.fts'
writefits,wdir+image_type+'_n.fts',image,himage
window
plot,flat(500,*),xst=1,yst=1,tit='Normalised FLAT'
end
