function halpha_deblend,w_n,w_b,wint,n_cont,wdir,infile

;+
; NAME:
;	HALPHA_DEBLEND
; PURPOSE:
;	approximation of H_alpha line and two NII lines 6548,6583 by 4 gaussians
; DESCRIPTION:
;       procedure uses multiple gauss-analysis for approximation of the observed line profile.
;
; CALLING SEQUENCE:
;	HALPHA_DEBLEND,INETRACK,w_n,w_b,wint,n_cont,wdir
; CATEGORY:
;	analizing MPFS-data
;
; INPUTS:
;	infile - initial file (normally 'obj_abs.fts')
;	wdir - working directory
;	n_cont - number of pixels used for continuum approximation
;	w_n - estimated fwhm of the narrow component (and NII lines, also)
;	w_b - estimated fwhm of the broad component
;
; OUTPUTS:
;	the output parameters of the gaussian (see description for multigaus.pro)
;
; OPTIONAL OUTPUT:
;	the result of the fitting is saved to 'halpha.ps'
;
; OPTIONAL INPUT KEYWORDS:
;	no
;
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;	Function :   MULTIGAUS
;
; MODIFICATION HISTORY:
;       Written by Pavel Abolmasov, Special Astrophysical Observatory RAS, Jan 2004
;-

  p1=double(6548.1-6562.8)
  p2=double(6583.4-6562.8)
  obj=double(readfits(wdir+infile,h))
  mn=max(obj)
  obj=obj/mn
  a=size(obj)
  ;larea=dblarr(2*w+1)
  dl=sxpar(h,'CDELT1')
  l0=sxpar(h,'CRVAL1')
  pint=round((wint-l0)/dl)                   ; pixels interval used...

  lambda0=l0+dl*dindgen(a[1])		; wavelength grid
  lambda=lambda0(pint[0]:pint[1])
  layer=dblarr(n_elements(lambda),a(2))
  layer=obj(pint[0]:pint[1],*)
  pc=intarr(4)
  pc[0]=pint[0]-n_cont
  pc[1]=pint[0]
  pc[2]=pint[1] 
  pc[3]=pint[1]+n_cont

  cont=[obj(pc[0]:pc[1],*),obj(pc[2]:pc[3],*)]
  contl=total(cont,2)/double(a[2])
  contlev=median(contl)

  layer=layer ;-contlev
  b=size(layer)
  layert=dblarr(b[1])
  for k=0,b[1]-1 do begin
    slice=reform(layer(k,*),16,16)
    sfit=gauss2dfit(slice,gs)
    layert[k]=sqrt(2.*!DPI)*gs[0]*gs[2]
  endfor
  ml=min(layert)
  layert=layert-min(layert)
; ** preliminary determination of halpha center **
  max=max(layert,x)
  wc=20.
  ;f=gaussfit(lambda(x-round(wc/dl):x+round(wc/dl)),layert(x-round(wc/dl):x+round(wc/dl)),G0,nterms=3)
  lmax=total(lambda(x-round(wc/dl):x+round(wc/dl))*layert(x-round(wc/dl):x+round(wc/dl)))/total(layert(x-round(wc/dl):x+round(wc/dl)))

  ;lmax=G0(1)
  print,lmax-lambda(x)
  pos=[lmax,lmax+p2,lmax]
  w_breal=w_n
  rms=0.

  sighma=dblarr(n_elements(pos))

  IF n_elements(w_b) eq 1 THEN BEGIN
   res=multigaus(lambda,layert,pos,fwhm=[w_n,w_n,w_b],doubl_len=[0.,-35.3,0.],doubl_ratio=[0.,.333,0.],/fixratio,/FIXfwhm)
   fit=replicate(ml,n_elements(lambda))
     for i=0,n_elements(pos)-1 do begin
       sighma(i)=res(i).fwhm/(2.*sqrt(2.*alog(2.)))
       fit=fit+res(i).max*exp(-((lambda-pos(i))/sighma(i))^2/2.)
     endfor
    fit=fit+.333*res(1).max*exp(-((lambda-(lmax+p1))/sighma(1))^2/2.)
  ENDIF ELSE BEGIN
   for k=0,n_elements(w_b)-1 do begin
    res=multigaus(lambda,layert-contlev,pos,fwhm=[w_n,w_n,w_b(k)],doubl_len=[0.,-35.3,0.],doubl_ratio=[0.,.333,0.],/fixpos,/fixratio,/fixfwhm)
    fit=replicate(contlev,n_elements(lambda))
     for i=0,n_elements(pos)-1 do begin
       sighma(i)=res(i).fwhm/(2.*sqrt(2.*alog(2.)))
       fit=fit+res(i).max*exp(-((lambda-pos(i))/sighma(i))^2/2.)
     endfor
    fit=fit+.333*res(1).max*exp(-((lambda-(lmax+p1))/sighma(1))^2/2.)
    rms0=rms
    rms=total((layert-fit)^2)/n_elements(layert-fit)
    if rms ne 0. and rms lt rms0 then w_breal=w_b(k)
    if k eq 0 then w_breal=w_b[0]
    print,w_breal
   endfor

    res=multigaus(lambda,layert-contlev,pos,fwhm=[w_n,w_n,w_breal],doubl_len=[0.,-35.3,0.],doubl_ratio=[0.,.333,0.],/fixratio)
    fit=replicate(ml,n_elements(lambda))
     for i=0,n_elements(pos)-1 do begin
       sighma(i)=res(i).fwhm/(2.*sqrt(2.*alog(2.)))
       fit=fit+res(i).max*exp(-((lambda-pos(i))/sighma(i))^2/2.)
     endfor
    fit=fit+.333*res(1).max*exp(-((lambda-(lmax+p1))/sighma(1))^2/2.) 
  ENDELSE

  layert=layert*mn
  res(0).max=res(0).max*mn
  res(1).max=res(1).max*mn
  res(2).max=res(2).max*mn
  res(0).flux=res(0).flux*mn
  res(1).flux=res(1).flux*mn
  res(2).flux=res(2).flux*mn

  fit=fit*mn

  set_plot,'X'

  plot,lambda,layert+ml*mn,psym=2,/ylog,yrange=[ml*mn,max(layert+ml*mn)]
  oplot,lambda,fit,psym=0
  oplot,lambda,res(0).max*exp(-((lambda-pos(0))/sighma(0))^2/2.),psym=0,linestyle=2,color=100000000
  oplot,lambda,res(1).max*exp(-((lambda-pos(1))/sighma(1))^2/2.),psym=0,linestyle=2,color=960000
  oplot,lambda,.333*res(1).max*exp(-((lambda-(lmax+p1))/sighma(1))^2/2.),psym=0,linestyle=2,color=960000
  oplot,lambda,res(2).max*exp(-((lambda-pos(2))/sighma(2))^2/2.),psym=0,linestyle=2, color=130
  xyouts,6560,8.e-15,'H_alpha:'
  xyouts,6560,6.e-15,'narrow comp.: flux = '+strtrim(string(res(0).flux),2)+'ergs/(cm^2 Angstrom s)'   
  xyouts,6573,5.e-15,'fwhm = '+strtrim(string(res(0).fwhm),2)+'Angstrom'
  xyouts,6560,4.e-15,'broad comp.: flux = '+strtrim(string(res(2).flux),2)+'ergs/(cm^2 Angstrom s)'   
  xyouts,6573,3.4e-15,'fwhm = '+strtrim(string(res(2).fwhm),2)+'Angstrom'
  xyouts,6575,2.e-15,'[NII] (6583):'
  xyouts,6575,1.5e-15,'flux = '+strtrim(string(res(1).flux),2)+'ergs/(cm^2 Angstrom s)'
  xyouts,6575,1.e-15,'fwhm = '+strtrim(string(res(1).fwhm),2)+'Angstrom'
  set_plot,'PS'
  device,file=wdir+'halpha.ps',/landscape,xsize=24,ysize=18
  plot,lambda,layert+ml*mn,psym=2,/ylog,yrange=[ml*mn,max(layert+ml*mn)], $
    title='Halpha and [NII] - fitting with three gaussians',xtitle='Wavelength, Angstroms', ytitle='overall flux, ergs/(cm^2 Angstrom s)'
  oplot,lambda,fit,psym=0
  oplot,lambda,res(0).max*exp(-((lambda-pos(0))/sighma(0))^2/2.),psym=0,linestyle=2
  oplot,lambda,res(1).max*exp(-((lambda-pos(1))/sighma(1))^2/2.),psym=0,linestyle=4,color=9
  oplot,lambda,.333*res(1).max*exp(-((lambda-(lmax+p1))/sighma(1))^2/2.),psym=0,linestyle=4,color=9
  oplot,lambda,res(2).max*exp(-((lambda-pos(2))/sighma(2))^2/2.),psym=0,linestyle=3, color=3
  xyouts,6560,8.e-15,'H_alpha:'
  xyouts,6560,6.e-15,'narrow comp.: flux = '+strtrim(string(res(0).flux),2)+'ergs/(cm^2 Angstrom s)'   
  xyouts,6573,5.e-15,'fwhm = '+strtrim(string(res(0).fwhm),2)+'Angstrom'
  xyouts,6560,4.e-15,'broad comp.: flux = '+strtrim(string(res(2).flux),2)+'ergs/(cm^2 Angstrom s)'   
  xyouts,6573,3.4e-15,'fwhm = '+strtrim(string(res(2).fwhm),2)+'Angstrom'
  xyouts,6575,2.e-15,'[NII] (6583):'
  xyouts,6575,1.5e-15,'flux = '+strtrim(string(res(1).flux),2)+'ergs/(cm^2 Angstrom s)'
  xyouts,6575,1.e-15,'fwhm = '+strtrim(string(res(1).fwhm),2)+'Angstrom'
  device,/close
  set_plot,'X'
return,res
end
