function linear,spectr,disper_par,N_lin,lambda_0,d_lambda
;+
; NAME:
;	LINEAR
; PURPOSE:
;	linearisation spectra
; DESCRIPTION: 
;	
; CALLING SEQUENCE:
;	Result = LINEAR ( spectr, disper_par, N_lin, lambda_0, d_lambda)
;
; CATEGORY:
;	reduction MPFS-data		
;
; INPUTS:
;	spectr = 2D array MPFS extracted spectra
;	disper_par = 2D array polynomial coefficits dispersion curve for every fiber
;	N_lin = numper ponts in linearised spectra
;	lambda_0 = beginning wavelength in A linearised spectra
;	d_lambda = dispersion in A/px linearised spectra
;
; OUTPUTS:
;	Result = 2D array spectra
;		
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;	no	
;
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;	no
;
; MODIFICATION HISTORY:
;       Written by Victor Afanasiev, Special Astrophysical Observatory RAS, Jul 1999
;-

on_error,2
a=size(spectr)
Nx=a(1)
Ny=a(2)
a=size(disper_par)
N_deg=a(1)-2
N_y=a(2)

if N_y ne Ny then message,'number spectra end dispersion curve not equal',/cont

lambda=findgen(N_lin)*d_lambda+lambda_0

x=dindgen(Nx)
spectr_lin=fltarr(N_lin,Ny)
for y=0,Ny-1 do begin

lambda_tmp=fltarr(Nx)
for j=0,N_deg do begin
lambda_tmp=lambda_tmp+disper_par(j,y)*x^j
endfor
spectr_lin(*,y) = INTERPOL(spectr(*,y),lambda_tmp,lambda)
endfor
return,spectr_lin
end
