pro LINER_f,logfile,out,mul=mul
;
;+
; NAME:
;	LINER_f
; PURPOSE:
;	linearisation spectra OBJ/STAR ( for filter transmition)
;
; CALLING SEQUENCE:
;	LINERISATION, logfile
;
; CATEGORY:
;	reduction MPFS-data
;
; INPUTS:
;	LOGFILE = file name of LOG observation (in format FITS-header)
;	image_type = string scalar type exposure (values: 'obj','star','neon',,'test')
;
;
; OUTPUTS:
;	Result saved to disk i working directory in file '*_lin.fts'
;
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;	no
;
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;	Function  DEF_WDIR,READ_FTS, OUT_WAVE
;	Procedure SXADDPAR,SXADDHIST
;
; MODIFICATION HISTORY:
;       Written by Victor Afanasiev, Special Astrophysical Observatory RAS, Jul 1999
;       add sky spectra linerisation ( A.Moiseev) SAO RAS, Apr 2000
; -     Carefully analyzed by P.Abolmasov, Aug 2003

on_error,2

message,'linerisation spectra ',/cont
if not(keyword_set(mul)) then mul=1. ; normalization factor
wdir=DEF_WDIR(LOGFILE)
image1=READ_FTS(wdir+'obj_s.fts',head_i,SILENT='silent')
image2=READ_FTS(wdir+'flat_s.fts',SILENT='silent')
image=image1/image2
disper_par=READ_FTS(wdir+'disper.fts',SILENT='silent')
a=size(image)
Nx=a(1)
Ny=a(2)
a=size(disper_par)
N_deg=a(1)-2
N_y=a(2)
;determination parameter of linerisation
out_param=OUT_WAVE(LOGFILE)

if N_y ne Ny then message,'number spectra and dispersion curve not equal',/cont
if N_elements(out_param) gt 1 then param=out_param
if N_elements(out_param) eq 1 then begin
  out_param=float([0,0,Nx])
  for k=0,N_deg do begin
    out_param(0)=out_param(0)+disper_par(k,Ny/2)*(double(Nx/2.)^k)
  endfor
  out_param(1)=disper_par(1,Ny/2)
  out_param(0)=out_param(0)-Nx*out_param(1)/2
endif

lambda_0=double(out_param(0))
d_lambda=out_param(1)
N_lin=out_param(2)
message,string(lambda_0)+string(d_lambda)+string(N_lin),/cont
lambda=findgen(N_lin)*d_lambda+lambda_0

x=dindgen(Nx)
image_lin=fltarr(N_lin,Ny)

for y=0,ny-1 do begin
  lambda_tmp=fltarr(Nx)
  for j=0,N_deg do begin
    lambda_tmp=lambda_tmp+disper_par(j,y)*x^j
  endfor
  image_lin(*,y) = INTERPOL(image(*,y),lambda_tmp,lambda)
endfor

image_lin=image_lin/mul
;*************

dev_rms=stdev(disper_par(N_deg+1,*),mean_rms)

;saving data
sxaddhist,'mean error dispersion curves ='+$
	string(mean_rms,format='(F5.2)')+' px',head_i
sxaddpar,head_i,'NAXIS1' ,N_lin,' NUMBER ELEMENTS IN SPECTRA'
sxaddpar,head_i,'NAXIS2' ,Ny,' NUMBER OF SPECTRA'
sxaddpar,head_i,'FILE',out
sxaddpar,head_i,'CRVAL1' ,out_param(0),before='HISTORY',' WAVELENGTH 1-st ELEMENT, ANGSTROM'
sxaddpar,head_i,'CDELT1' ,out_param(1),after='CRVAL1',' DISPERSION, ANGSTROM/PX'

writefits,wdir+out,image_lin,head_i
end
