pro LINERISATION,logfile,image_name,CORRECT=CORRECT,sky=sky
;
;+
; NAME:
;	LINERISATION
; PURPOSE:
;	linearisation spectra
; DESCRIPTION:
;	Routine read extracted MPFS-spectra with type '*' from file '*_s.fts'
;	in working directory and lineriarised. If parameter 'OUT_WAVE' in LOG
;	observation is non-zero, routine use parameters linearisation from LOG,
;	else use mean parameter linearisation (beginning wavelength and dispersion)
;	calculated by file 'disper_par.fts'. Routine modifies FITS-header of result
;
; CALLING SEQUENCE:
;	LINERISATION, logfile, image_name
;
; CATEGORY:
;	reduction MPFS-data
;
; INPUTS:
;	LOGFILE = file name of LOG observation (in format FITS-header)
;	image_type = string scalar type exposure (values: 'obj','star','neon','test')
;
;
; OUTPUTS:
;	Result saved to disk in the working directory in file '*_lin.fts'
;
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;	no
;
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;	Function  DEF_WDIR,READ_FTS, OUT_WAVE
;	Procedure SXADDPAR,SXADDHIST
;
; MODIFICATION HISTORY:
;       Written by Victor Afanasiev, Special Astrophysical Observatory RAS, Jul 1999
;       add sky spectra linerisation ( A.Moiseev) SAO RAS, Apr 2000
;	
;

on_error,0

message,'linerisation spectra '+image_name,/cont
wdir=DEF_WDIR(LOGFILE)
if image_name ne 'neon' then image=READ_FTS(wdir+image_name+'_osf.fts',head_i,SILENT='silent') else $
        image=READ_FTS(wdir+image_name+'_sf.fts',head_i,SILENT='silent')
if image_name ne 'neon' then sky=READ_FTS(wdir+image_name+'_nsf.fts',head_i,SILENT='silent')
; disper_par=READ_FTS(wdir+'disper.fts',SILENT='silent')

a=size(image)
Nx=a(1)
Ny=a(2)
a=size(sky)
N_sky=a(2)

if image_name ne 'neon' then disper_par=redis(wdir+'disper.txt',Ny+N_sky) else disper_par=redis(wdir+'disper.txt',Ny) 
a=size(disper_par)
N_deg=a(1)-2
N_y=a(2)
;determination parameter of linerisation
out_param=def_par(LOGFILE,'INP_WAVE')  ;double(strsplit(def_par(LOGFILE,'INP_WAVE'),',',/ext))

if N_y ne Ny+N_sky then message,'number spectra end dispersion curve not equal',/cont
if N_elements(out_param) gt 1 then param=out_param
if N_elements(out_param) eq 1 then begin
  out_param=[INP_WAVE(LOGFILE),Nx]
  out_param(0)=out_param(0)-out_param(1)*out_param(2)/2
endif
message,'Initial wavelength:'+string(out_param(0))+'; +dispertion:'+string(out_param(1)),/cont

lambda_0=double(out_param(0))
d_lambda=out_param(1)
N_lin=out_param(2)
message,string(lambda_0)+string(d_lambda)+string(N_lin),/cont
lambda=dindgen(N_lin)*d_lambda+lambda_0

x=dindgen(Nx)
image_lin=dblarr(N_lin,Ny)
if image_name ne 'neon' then yn=where(indgen(Ny+N_sky) mod 17 ne 0) else yn=indgen(Ny)
for y=0,Ny-1 do begin
 ; lambda_tmp=replicate(double(0),Nx)
  lambda_tmp=poly(x,disper_par(0:3,yn[y]))
  image_lin(*,y)=interpol(image(*,y),lambda_tmp,lambda,/spline)
endfor

;*************
; Correction on residual shifting between  probes
 IF keyword_set(CORRECT) THEN BEGIN

   if image_name NE 'neon' then new_shift=read_FTS(wdir+'neon_shi.fts') $
     ELSE BEGIN  ; determination offset shifting between probes:
 	   new_shift=dblarr(Ny)

   	   center=reform(image_lin(300:N_lin-200,NY/2)) ; central probe
	   for y=0,ny-1 do new_shift(y)=DEF_COR(reform(image_lin(300:N_lin-200,y)),center)
                                                         
   		 window,15,tit='Residual shifting'
   		 plot,indgen(ny),new_shift,psym=1,xst=1,yst=1,$
               tit='Residual shifting after linerisation',xtit='Lens',ytit='Shift [px]',color=-1
   		mkhdr,header_s,new_shift
   		sxaddhist,'NEON residual shifts',header_s
   		writefits,wdir+'neon_shi.fts',new_shift,header_s
     ENDELSE

  for y=0,ny-1 do image_lin(*,y)=vecshift(image_lin(*,y),dx=new_shift(y))
 ENDIF

dev_rms=stdev(disper_par(N_deg+1,*),mean_rms)

; SKY spectra linearization:
if image_name NE 'neon' then begin
                                   ;determination dispersion curve for sky spectra
IF keyword_set(SKY) THEN BEGIN
 traectory=READ_FTS(wdir+'eta_tra.fts',headt)
  N_sky=sxpar(headt,'NAXIS2')                    ; =16
  disper_sky=dblarr(N_deg+2,N_sky)

  ydis=indgen(ny)
;  ydis=17+indgen(ny)
  ysky=dindgen(n_SKY)*N_sky
  Ny_deg=2 ; polynom degree
  ;disper_sky=disper_par(*,where((dindgen(ny+N_sky) mod N_sky) eq 0 ))
  for j=0,N_deg do begin
     ; fitting across dispersion
     fitdis=goodpoly(ydis,disper_par(j,ydis),Ny_deg,2.,fit,newx,newy)
 ;    disper_sky(j,*)=interpol(reform(disper_par(j,ydis)),double(ydis),ysky,/spline)
     disper_sky(j,*)=poly(double(ysky),fitdis)             ;median(disper_par(j,*))
  endfor

  ; read SKY image
  sky_im=READ_FTS(wdir+image_name+'_nsf.fts')
 ; linearisation
 sky_lin=dblarr(N_lin,N_sky)
 for y=0,n_sky-1 do begin
   lambda_tmp=replicate(0.,Nx)
   for j=0,N_deg do lambda_tmp=lambda_tmp+disper_sky(j,y)*x^j
   sky_lin(*,y) = INTERPOL(sky_im(*,y),lambda_tmp,lambda)
 endfor
  new_im=dblarr(N_lin,ny+N_sky)
  new_im(*,n_sky:ny+n_sky-1)=image_lin                     ; first 17 rows are filled with sky spectra
  new_im(*,0:n_sky-1)=sky_lin

  image_lin=new_im
  
  sky_im1=dblarr(N_lin,ny+N_sky)
  sky_im1(*,0:n_sky-1)=sky_lin
  ;lambda=dindgen(N_lin)*d_lambda+lambda_0
  for x=0,N_lin-1 do sky_im1(x,n_sky:*)=INTERPOL(sky_lin(x,*),ysky,ydis,/spline)  
 ENDIF
endif

;saving data
sxaddhist,'mean error dispersion curves ='+$
	string(mean_rms,format='(F5.2)')+' px',head_i
sxaddpar,head_i,'NAXIS1' ,N_lin,' NUMBER ELEMENTS IN SPECTRA'
sxaddpar,head_i,'NAXIS2' ,Ny,' NUMBER OF SPECTRA'
sxaddpar,head_i,'FILE',image_name+'_lin.fts'
sxaddpar,head_i,'CRVAL1' ,out_param(0),before='HISTORY',' WAVELENGTH 1-st ELEMENT, ANGSTROM'
sxaddpar,head_i,'CDELT1' ,out_param(1),after='CRVAL1',' DISPERSION, ANGSTROM/PX'
sxaddpar,head_i,'CRPIX1' ,1,after='CDELT1'
sxaddpar,head_i,'CRVAL2' ,1,after='CRPIX1'
sxaddpar,head_i,'CDELT2' ,1,after='CRVAL2'
sxaddpar,head_i,'CRPIX2' ,1,after='CDELT1'
IF keyword_set(CORRECT) THEN sxaddhist,'Corrected on neon_shi.fts',head_i
if image_name NE 'neon' then begin
   sxaddpar,head_i,'NAXIS2' ,Ny+N_sky,' NUMBER OF SPECTRA'
IF keyword_set(SKY) THEN    sxaddpar,head_i,'COMMENT','with sky probes!'
endif

  writefits,wdir+image_name+'_lin.fts',image_lin,head_i
  IF keyword_set(SKY) THEN writefits,wdir+image_name+'_lsky.fts',sky_im1,head_i
  if image_name eq 'neon' then writefits,wdir+'neonsp.fts',total(image_lin,2),head_i
  if image_name eq 'obj' then writefits,wdir+'osp.fts',total(image_lin,2),head_i

end
