pro linetrack,infile,wdir,outfile,llist,range,w=w
;+
; NAME:
;	LINETRACK
; PURPOSE:
;	creation barycenter trajectories along line profiles
; DESCRIPTION:
;       procedure calculates using approximation with 2d-gaussian the position of the object in continuum (using polynomial
;       approximation) and in an emission line (w pixels used) with the continuum subtructed. The shift array is saved as a 
;       ps file       
;
; CALLING SEQUENCE:
;	LINETRACK,infile,wdir,outfile,llist,range,w=w
; CATEGORY:
;	analizing MPFS-data
;
; INPUTS:
;	infile - initial file (normally 'obj_abs.fts')
;	wdir - working directory
;	outfile - output .ps file
;	llist - central wavelength
;	range - maximal interval for analysis
;	w - number of pixels used along dispertion for data averaging
;
; OUTPUTS:
;	result sent to wdir+outfile
;
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;	no
;
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;	Function :   READFITS, DECONT, OPLOTERt
;
; MODIFICATION HISTORY:
;       Written by Pavel Abolmasov, Special Astrophysical Observatory RAS, Jan 2004
;-

  if not(keyword_set(w)) then w=1
  decont,infile,wdir,'cont.fts',gf,sighmas,plot=1     ; sighmas=[dc,da,dwx,dwy,dx,dy,dth,df]
						; gf = continuum par-s
  in=readfits(wdir+infile,h)
  con=readfits(wdir+'cont.fts',hc)
  bgd=readfits(wdir+'obj_lin.fts',hb)
;  bgd=bgd(*,17:272)
  gain=sxpar(hb,'GAIN')
  bgd=bgd*gain

  s=size(in)
  dl=sxpar(h,'CDELT1')
  l0=sxpar(h,'CRVAL1')
  lambda=l0+dl*dindgen(s(1))		; wavelength grid
  ccor=round((llist-l0)/dl)
  d=round(range/dl)

  area=dblarr(2*d+1,16,16)
  flux=dblarr(2*d+1)
  cube=reform(in-con*1.,s(1),16,16)
  ccon=reform(con*1.,s(1),16,16)
   area=cube(ccor-d:ccor+d,*,*)
   if w ge 3. then for i=0,2*d do area(i,*,*)=total(cube(ccor-d+i-round((w-1.)/2.):ccor-d+i+round((w-1.)/2.),*,*),1) 
   flux=total(reform(area,2*d+1,s(2)),2)
   fcon=total(con(ccor-d:ccor+d,*),2)
   whr=where(flux gt 0.,cnt)
   if cnt le 1 then goto, fin
   areal=dblarr(cnt,16,16)
   areal=area(whr,*,*)  ;/max(area(whr,*,*))
   ll=dblarr(cnt)
   ll=lambda(ccor-d+whr)
   center=dblarr(cnt,2)
   efw=dblarr(cnt,2)
   chisq=dblarr(cnt)
   g=dblarr(7)
   x=dblarr(16,16)  & y=dblarr(16,16)
   for k=0,14 do x(*,k)=dindgen(16)
;   y=dindgen(256)
   y=16.*dindgen(16)
   flux=dblarr(cnt)
   tilt=dblarr(cnt)
   for i=0,cnt-1 do begin
;     center[i,0]=total(arr*x)/total(arr)
;     center[i,1]=total(arr*y)/total(arr)
     arr=reform(areal(i,*,*),16,16)
     barr=total(bgd(ccor-d+whr[i],*))
     monomap=rebin(arr,320,320)/total(arr)
     if i eq 0 then maparr=monomap else maparr=[maparr,monomap]
;    arr=smooth(arr,2)
     g[0]=0. & g[1]=max(arr) & g[2]=2. & g[3]=2. & g[5]=7. & g[4]=7.
     fitar=gauss2dfit1(arr,g,/tilt, ESTIMATES=[0.,max(arr),2.,2.,total(arr*x)/total(arr),total(arr*y)/total(arr),0.])
     if i eq 0 then farr=rebin(fitar,320,320)/total(fitar) else farr=[farr,rebin(fitar,320,320)/total(fitar)]
     flux[i]=sqrt(2.*!DPI*(g[2]^2+g[3]^2))*g[1]
     chisq[i]=total((arr-fitar)^2)/(256.*flux[i])
     center[i,0]=g[4]-gf[ccor-d+whr[i],4]
     center[i,1]=g[5]-gf[ccor-d+whr[i],5]
     efw[i,0]=g[2]*sqrt(8.*alog(2.)) ;-gf[ccor-d+whr[i],2]
     efw[i,1]=g[3]*sqrt(8.*alog(2.)) ;-gf[ccor-d+whr[i],3]
     tilt[i]=g[6]
     contour,arr-fitar
   endfor

   evarmap1=rebin(total(cube(where(lambda ge llist+3.),*,*),1),320,320,/sample)
   evarmap2=rebin(total(cube(where(lambda le llist-3.),*,*),1),320,320,/sample)
   mfar=dblarr(320*cnt,640)
   mfar(*,0:319)=maparr & mfar(*,320:639)=maparr-farr
   wsq=where(chisq le 5. and (center(*,0)^2+center(*,1)^2 le 200.),cntsq)
   if cntsq le 1 then goto, fin
   center1=dblarr(cntsq,2)
   efw1=dblarr(cntsq,2)
   for q=0,1 do center1(*,q)=center(wsq,q)
   for q=0,1 do efw1(*,q)=efw(wsq,q)

; center0=simulate_line('obj_nph.fts',wdir,llist,range,3.,8.,.22,.5,1.,1.,gf,l0)

   fitmax=max(total(reform(area(whr,*,*),cnt,256),2),nmax)
 ; nmax=nint((gaa[1]-l0)/dl)
   map=dblarr(16,16)
   map=total(area(whr,*,*),1)
   fitar=gauss2dfit(map/max(map),g,/tilt)
   centerx=g[4]-gf[ccor-d+whr[nmax],4]
   centery=g[5]-gf[ccor-d+whr[nmax],5]
   wx=g[2]*sqrt(8.*alog(2.))
   wy=g[3]*sqrt(8.*alog(2.))

   linxy=dblarr(2)
   linxy=linfit(center(*,0),center(*,1))
   
   linrl=dblarr(2)
   linrl=linfit(ll,sqrt((center(*,0))^2+(center(*,1))^2))

   c=double(2.9979e+5)
   vel0=((l0-llist)/llist)*c
   ll=ll(wsq)
   vel=((ll-llist)/llist)*c
   p2=sxpar(h,'PARANGLE')
   ptb=sxpar(h,'ROTANGLE')

if cntsq gt 3 then begin
 ldl=ll-llist
   pars=[centerx,.1,3.]
   xfit=curvefit(ldl,reform(center1(*,0)),flux(wsq),pars,function_name='twosrc',sigmx)
 parsx=pars
   pars=[centery,.2,3.]
   yfit=curvefit(ldl,reform(center1(*,1)),flux(wsq),pars,function_name='twosrc',sigmy)
 parsy=pars

 ; xl=linfit(ll,reform(center1(*,0)),yfit=xfit)
 ; yl=linfit(ll,reform(center1(*,1)),yfit=yfit)

 print,'delta_v = ',c*(parsx[2]+parsy[2])/(2.*llist),'+/-',c*(sigmx[2]+sigmy[2])/(sqrt(8.)*llist),'	km/s'
 print,' (dx, dy)  = (',parsx[1],', ',parsy[1],')','+/- (',sigmx[1],', ',sigmy[2],')'
endif
 set_plot,'ps'
   device,filename=wdir+'slices_'+outfile,/landscape
      tvscl,-mfar
;      oplotert,center(0,*),center(1,*),wx/10.,wy/10.,g[6]/!DTOR 
   device,/close

   device,filename=wdir+'map_'+outfile,/landscape
      contour,evarmap1,color=100,/downhill,levels=10.^(1.*(dindgen(10)-double(9)))*max(evarmap1)
      contour,evarmap2,color=200,/overplot,/downhill,levels=10.^(1.*(dindgen(10)-double(9)))*max(evarmap2)
;      oplotert,center(0,*),center(1,*),wx/10.,wy/10.,g[6]/!DTOR 
      sxaddpar,h,'FILE','map_'+outfile+'.fts'
      sxaddpar,h,'IMAGETYP','flux map'
      writefits,wdir+outfile+'.fts',total(area,1),h
   device,/close

   device,filename=wdir+outfile,/portrait,xsize=20,ysize=30,xoffset=0,yoffset=0
   plot,center1(*,0),center1(*,1), $
  yrange=[centery-.5,centery+.5],xrange=[centerx-.5,centerx+.5],isotropic=1,xtitle='X, arcsec',ytitle='Y, arcsec', $
  thick=2.,charsize=1.5
 ;  oploterr,center1(*,0),center1(*,1),efw1(*,1)/sqrt(((2*w+1)*flux))
 ;  oploterx,center1(*,0),center1(*,1),efw1(*,0)/sqrt(((2*w+1)*flux))
   oplotert,center1(*,0),center1(*,1),efw1(*,0)/sqrt(((2*w+1)*flux)),efw1(*,1)/sqrt(((2*w+1)*flux)),tilt/!DTOR
   oplotert,centerx,centery,wx/50.,wy/50.,g[6]/!DTOR
;   oplot,center0(*,0)+centerx,center0(*,1)+centery,psym=4,thick=2.,symsize=2.
 if cntsq gt 3 then oplot,xfit,yfit,psym=4,thick=2.,symsize=2.
  xyouts,center1(*,0),center1(*,1),' '+strtrim(string(10.*round(vel*.1),format='(F8.0)'),2),charsize=1.5
;   xyouts,center0(*,0)+centerx,center0(*,1)+centery,'['+strtrim(string(10.*round(vel0*.1),format='(F8.0)'),2)+']',charsize=1.5
  drawne,.75,.85,p2-ptb+37,r=.05
   device,/close
 set_plot,'x'
 fin:
end
