PRO MAP_LINE,filein,l0=l0,Name=Name,w_dir=w_dir,vsys=vsys,mul=mul,dv=dv,win=win,contval=contval,ratio=ratio,nx=nx,contlin=contlin

;**********************************
;* Emission line  map from MPFS data cube
;* True integration from gaussian ( one component!)
;* creation files:
;  _F.FTS -- flux
;  _V.FTS -- velocities in km/s
;  _W.FTS -- FWHM in km/s
;  _C.FTS -- local continuum value
;  _CUB.FTS -- emision line image
; L0 -- line wavelength
; Name=       0          1          2          3          4        5       6      7(D)        8(D)     9
;  lambda= 4685.68  4861.32     4958.93     5006.8    6562.78   5577.35 5460.7   6583.37    6717.00  6300.30
; VSYS = system velocity
; dv   = heliocentric correction from ADHOC
; mul  = multiplication factor
; win  = half of window for analysis
; contval = number of channels for continuum determination
; ration  = fixed  line ratio in doublets



;***********************************
loadct,27

  if N_params() LT 1 then begin
   message, /NONAME,'Uses: MAP_LINE,filein,l0=l0,Name=Name,w_dir=w_dir,vsys=vsys,mul=mul,dv=dv,win=win,contval=contval',/cont
   message, /NONAME,'Name=    0          1          2          3           4            5            6           7(D)        8(D)         9      ',/cont
   message, /NONAME,'      [4685.68, 4861.32, 4958.93, 5006.8, 6562.78, 5577.35, 5460.7,  6583.37, 6717.00,   6300.30]',/cont
   return
  endif


; file names:
if not(keyword_set(vsys)) then vsys=0  ; system velocity
if not(keyword_set(ratio)) then ratio=0  ; double ratio
if not(keyword_set(mul)) then mul=1 ; multiplicativ factor ( if input data in ergs)
if not(keyword_set(win)) then  w=6  else w=win ; half of window
if not(keyword_set(contval)) then contval=0    ; points for continuum measuring
if not(keyword_set(dv)) then dv=0        ; correction by Sun motions!
if not(keyword_set(w_dir)) then CD,current=w_dir else w_dir=slash(w_dir)

line= [4685.68,4861.32,4958.93,5006.8, 6562.78,5577.35,5460.7, 6583.37, 6717.00,6300.30]
line2=[4685.68,4861.32,4958.93,4958.93,6562.78,5577.35,5460.7, 6548.06, 6730.00,6300.30]
doublet=line-line2
;Name=  ;  0       1       2       3      4        5     6      7      8
if not(keyword_set(name)) then name=0
if not(keyword_set(l0)) then l0=line(Name)


out=strtrim(string(round(l0)),2)			; truncated wavelength used for filenames creation 

; read data
CD,current=old_dir
CD,w_dir
print,w_dir
obj=read_fts(filein,header)                             ; read OBJ


Sx=sxpar(header,'NAXIS1')
m=sxpar(header,'NAXIS2')                                    ;total number of fibers
ld0 =sxpar(header,'CRVAL1',comment=c1)
dld0=sxpar(header,'CDELT1',comment=c2)
nld=Sx ; wavelength range
lda=ld0+dld0*dindgen(nld)                                    ; wavelength grid
CD,old_dir


; map creation:

IF (name eq 9)or(name eq 5) then begin  dv=0 & m=m-16 & obj=obj(*,16:*) & end ; 5577 or 6300 sky line

;MAPs creation:
if not(keyword_set(nx)) then nx=16
ny=m/nx
c=double(2.9979e5)                                             ; c in km/s
vr=dblarr(nx,ny)
con=dblarr(nx,ny)
fwhm=dblarr(nx,ny)
flux=dblarr(nx,ny)


cub=dblarr(Sx,m)
Lcur=l0*(1+Vsys/c)
FOR i=0,m-1 do begin
    if i mod 50 eq 0 then plot=1 else plot=0
    vec=obj(*,i)

    ; preliminary line position
    rec=where((lda LE Lcur+2*w*dld0)and(lda GE Lcur-2*w*dld0),count)
    if count  EQ 0 then message,'Line position is out of range!!',/contin $
    ELSE BEGIN

     maxprf=max(vec(rec),maxpos)
     maxpos=maxpos+rec(0)

     i1=maxpos-w
     i2=maxpos+w

    if doublet(name) lt 0 then i2=i2-doublet(name)/dld0
    if doublet(name) gt 0 then i1=i1-doublet(name)/dld0

    rec=[rec,rec+doublet(name)/dld0]
    ;continuum determination
    if contval gt 0 then begin
     con1=median(vec(i1-contval:i1))
     con2=median(vec(i2:i2+contval))
     ; linear fitting
      l1=lda(i1-contval)
      l2=lda(i2+contval)

     if keyword_set(contlin) then   con12=con1+(lda-l1)*(con2-con1)/(l2-l1) $
      else    con12=make_array(nld,val=(con1+con2)/2)
    endif else con12=make_array(nld,val=0.)



    res=multigaus(lda(i1:i2),vec(i1:i2)-con12(i1:i2),Lcur(0),fwhm=w(0)/2,plot=plot,/sil, $
            DOUBL_LEN=-doublet(name),DOUBL_RATIO=ratio,fixratio=(ratio gt 0),lim_fwhm=[0.1*w(0),2*w(0)])

    if res(0).center ne 0 then begin
     flux(i)=res(0).flux
     vr(i)=c*(res(0).center-l0)/l0+dv
     fwhm(i)=res(0).fwhm*c/l0
     con(i)=median(con12)
    endif
    cub(*,i)=con12+gaussian(lda,[res(0).max,res(0).center,res(0).fwhm/2.35482])
    if doublet(name) ne 0 then cub(*,i)=cub(*,i)+gaussian(lda,[res(0).max*res(0).ratio,res(0).center-doublet(name),res(0).fwhm/2.35482])
     ENDELSE

    
    if i mod 50 eq 0 then print,'Vector:',Lcur,res(0).center

ENDFOR


;*****  write file

CD,w_dir
FTS_DA3,filein,mul=mul

sxaddhist,'MAP_LINE.PRO : fit by single gaussian from file: '+filein,header
H0=header
fileout=out+'_CUB.FTS'
sxaddpar,header,'FILE',fileout
sxaddpar,header,'IMAGETYP','fit result'
writefits,fileout,cub,header   ; total cub
FTS_DA3,fileout,mul=mul

sxaddhist,'MAP_LINE.PRO : contval '+string(contval),h0
sxaddhist,'MAP_LINE.PRO : half of window '+string(w),h0
sxaddhist,'MAP_LINE.PRO : reference lambda '+string(l0),h0
sxaddpar,h0,'CRVAL1',1
sxaddpar,h0,'CDELT1',1
sxaddpar,h0,'CRVAL2',1
sxaddpar,h0,'CDELT2',1


h1=h0
h2=h0
h3=h0
h4=h0
;***** continuum map *****
fileout=out+'_C.FTS'
sxaddpar,h1,'FILE',fileout
sxaddpar,h1,'IMAGETYP','continuum map'
writefits,fileout,con,h1

;***** FLUX map *****
fileout=out+'_F.FTS'
sxaddpar,h2,'FILE',fileout
sxaddpar,h2,'IMAGETYP','flux map'
writefits,fileout,flux,h2

;***** FWHM map *****
fileout=out+'_W.FTS'
sxaddpar,h3,'FILE',fileout
sxaddpar,h3,'BUNIT','km/s'
sxaddpar,h3,'IMAGETYP','FWHM map'
writefits,fileout,fwhm,h3

;***** Velocities map *****
fileout=out+'_V.FTS'
sxaddpar,h4,'FILE',fileout
sxaddpar,h4,'BUNIT','km/s'
sxaddpar,h4,'IMAGETYP','velocity map'

writefits,fileout,vr,h4

CD,old_dir

window,0,xs=nx*50,ys=ny*50
!p.multi=[0,2,2]
contour,con,/follow,tit=out+'_C.FTS',xtit='X',ytit='Y',nlev=20,charsize=0.5,min_val=0
contour,flux,/follow,tit=out+'_F.FTS',xtit='X',ytit='Y',nlev=20,charsize=0.5,min_val=0
med=median(fwhm,5)
m1=min(med)
m2=max(med)
contour,fwhm,/follow,tit=out+'_W.FTS',xtit='X',ytit='Y',nlev=20,charsize=0.5,min_val=m1,max_val=m2
med=median(vr,5)
m1=min(med)
m2=max(med)

contour,vr,/follow,tit=out+'_V.FTS',xtit='X',ytit='Y',nlev=20,charsize=0.5,min_val=m1,max_val=m2
!p.multi=[0,1,1]
END
