pro muline,infile,l,wdir, w=w, dv=dv,field=field,colt=colt,conp=conp,dne=dne,bot=bot,losc=losc
;+
; NAME:
;	muLINE
; PURPOSE:
;	obtaining flux, velosity and fwhm maps from a datacube
; DESCRIPTION: 
;	fits line profile with a gaussian and computes fluxes etc. from its parameters
;
; CALLING SEQUENCE:
;	ANALINE,INFILE,LAMBDA,WDIR,w=w,dv=dv,field=field
; CATEGORY:
;	MPFS-data analysis		
;
; INPUTS:
;	INFILE - fits-file containing initial primarily-reducted data  
;	LAMBDA - rest-frame wavelength (redshift z for distant galaxies can be taken from the header, or 
;		added by keyword dv in km/s)
;	WDIR - output directory
; OUTPUTS:
;	output fts and ps files with names of the types: [lambda]_{fv,w}.ps and [lambda]_{f,v,w,c}.fts
;	containing fluxes (f), velosities (v) and fwhm in velosity units (w)
;		
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;	w - spectral interval halfwidth (points used for continuum determination included)
;       conp - narrower spectral interval used for approximation per se 
;	dv - additional velosity shift
;	field - field size in pxls ([16,16] by default)
;       colt - specify colour table (0 by default)
;              
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;	
;	Procedures:  ?
;
; MODIFICATION HISTORY:
;       Written by P. Abolmasov, Special Astrophysical Observatory RAS, Jul 2004
;-

  if not(keyword_set(w)) then w=50.
if not(keyword_set(colt)) then colt=0.
if not(keyword_set(bot)) then bot=0
  if not(keyword_set(conp)) then conp=15.

  if not(keyword_set(dv)) then dv=0.
  if not(keyword_set(field)) then field=[16,16]
  out=wdir+strtrim(string(round(l)),2)

; loadct,27

  obj=double(readfits(infile,h))
  a=size(obj)
  dl=sxpar(h,'CDELT1')
  l0=sxpar(h,'CRVAL1')
  d=round(w/dl)
  area=dblarr(2*d+1,a(2))
  larea=dblarr(2*d+1)
  nx=a(1)                          ;  number of wavelengths 
  ny=a(2)			   ; number of fibers
     if ny ne field[0]*field[1] then message,'WRONG DIMENTIONS OF THE FIELD!'

  lambda=l0+dl*dindgen(nx)		; wavelength grid
  z=double(sxpar(h,'REDSHIFT'))
  uni=sxpar(h,'BUNIT')

  nl=n_elements(l)
  ccor=round((l*(z+1.)-l0)/dl)
  area=obj(min(ccor)-d:max(ccor)+d,*)  
  aa=size(area)
  larea=lambda(min(ccor)-d:max(ccor)+d)  
  set_plot, 'x'

  vec=total(area,2)
  contpars0=linfit([larea(0:4),larea(aa[1]-5:aa[1]-1)],[vec(0:4),vec(aa[1]-5:aa[1]-1)])
  cont=poly(larea,contpars0)

set_plot,'ps'
device,filename=out[0]+'_fit.ps',/land
  plot,larea,vec*1.e15,xtitle='!4k!3, '+string("305B),ytitle='flux, 10!E-15!N erg/cm!E2!Ns'+string("305B),$
    charsize=2.,charthick=3.,psym=2.
    res0=multigaus(larea,(vec-cont)/total(vec-cont),l*(z+1.),FWHM=l*4.7/5000.,yfit=afit,lim_fwhm=[3.,15.]) 
    cent0=res0.center &  fwhm0=res0.fwhm
  oplot,larea,(cont+afit*total(vec-cont)/total(afit))*1.e15
device,/close
set_plot,'x'

  flux=dblarr(nl,ny)
  dflux=dblarr(nl,ny)
  rms=dblarr(nl,ny)
  fwhm=dblarr(nl,ny)
  cont=dblarr(nl,ny)
  cent=dblarr(nl,ny)
  dfwhm=dblarr(nl,ny)
  dcont=dblarr(nl,ny)
  dcent=dblarr(nl,ny)
  var=dblarr(ny)
  velt=dblarr(nl,ny)
  var0=dblarr(ny)

aa=size(area)
   arasm=dblarr(field[0]*field[1],aa[1])
   for k=0,aa[1]-1 do arasm(*,k)=area(k,*)
 
   for i=0,ny-1 do begin
    plot,larea,area(*,i)
    wplus=where(area(*,i) gt 0.,nplus)
    if nplus lt 1 then begin
      flux(*,i)=0.
      cent(*,i)=0.
      dcent(*,i)=w
      fwhm(*,i)=0.
      continue
    endif
    nest=where(indgen(aa[1]) lt conp or indgen(aa[1]) gt aa[1]-1-conp)
    west=where(indgen(aa[1]) gt conp and indgen(aa[1]) lt aa[1]-1-conp)
    contpars=goodpoly(larea(nest),area(nest,i),1,2.)
    cont=poly(larea,contpars)
    w1=west
    res=multigaus(larea(w1),(area(w1,i)-cont(w1))/total(area(w1,i)-cont(w1)),cent0,FWHM=fwhm0,yfit=afit,lim_fwhm=[3.,15.],$
        sigma=sighma,/sil)  ;*total(area(w1,i)-cont(w1))
    arasm(i,*)=cont
    arasm(i,w1)=afit*total(area(w1,i)-cont(w1))+cont
  ;  oplot,larea,afit*total(area(*,i)-cont(w1))/total(afit),color=1000
    oplot,larea,arasm(i,*),color=10000
 ;  if i eq 213 then message,'!'
   flux(*,i)=res.flux*total(area(w1,i)-cont(w1))
;    flux(*,i)=res.max*res.fwhm/sqrt(8.D*alog(2.D))
    cent(*,i)=res.center
    fwhm(*,i)=res.fwhm
    dflux(*,i)=sighma.flux*total(area(w1,i)-cont(w1))
    dfwhm(*,i)=sighma.fwhm
    for k=0,nl-1 do rms[k,i]=stdev(area(*,i)-arasm(i,*))*fwhm[k,i]/dl
    dcent(*,i)=sighma.center
  ;  dflux(*,i)=rms(*,i)
  ;  for k=0,nl-1 do dcent[k,i]=fwhm[k,i]^2*rms[k,i]/sqrt(8.*alog(2.)*total(arasm(i,w1)^2*(larea(w1)-cent[k,i])^2))
  endfor
  flux1=flux

  flux_map=reform(flux,nl,field[0],field[1])
  dflux_map=reform(dflux,nl,field[0],field[1])
  fwhm_map=reform(fwhm,nl,field[0],field[1])
  cent_map=reform(cent,nl,field[0],field[1])
;  cont_map=reform(cont,nl,field[0],field[1])

c=double(2.9979e+5)                       ; c in km/s 
  
  for lk=0,nl-1 do velt(lk,*)=(cent(lk,*)-l[lk])*c/l[lk]-double(dv)
  wv=fwhm*c/median(l,/even)
  dv=abs(dcent*c/median(l,/even))
  vel_map=reform(velt,nl,field[0],field[1])
  wv_map=reform(wv,nl,field[0],field[1])
  dv_map=reform(dv,nl,field[0],field[1])

wbad=where(flux le rms*2.,nbad)
for lk=0,nl-1 do begin
  flux1(lk,*)=reform(smooth(reform(flux_map(lk,*,*)),3),field[0]*field[1])
  if nbad gt 0 then wbad=[wbad,where(flux1(lk,*) le 1.5*median(flux(lk,*)))*nl+lk] else wbad=where(flux1(lk,*) le 1.5*median(flux(lk,*)),nbad)*nl+lk
endfor
wbad=wbad(uniq(wbad))

nbad1=0
vel_map1=vel_map
for lk=0,nl-1 do begin
  vel_map1(lk,*,*)=med_ext2(reform(vel_map(lk,*,*)),3)
  maskgood=replicate(1.,field[0],field[1])
  wbk=wbad(where(wbad mod nl eq lk)-lk)/nl
  maskgood(wbk)=0
  maskgood1=maskgood+shift(maskgood,1,1)+shift(maskgood,1,-1)+shift(maskgood,-1,1)+shift(maskgood,-1,-1)+$
    shift(maskgood,1,0)+shift(maskgood,0,-1)+shift(maskgood,0,1)+shift(maskgood,-1,0)
  if nbad1 gt 0 then wbad1=[wbad1,where(maskgood1 gt 0 and maskgood lt 1)*nl+lk] else wbad1=where(maskgood1 gt 0 and maskgood lt 1,nbad1)*nl+lk
endfor

 ; vel_map(wbad)=sqrt(-1)
if max(wbad1) ge 0 then vel_map(wbad1)=vel_map1(wbad1)

; flux(wbad)=0.
  flux_map=reform(flux,nl,field[0],field[1])
intflux=total(total(flux_map,2),2)
dinf=intflux
for lk=0,nl-1 do dinf[lk]=stdev(flux_map(lk,*,*))

posang=sxpar(h,'POSANG')

set_plot,'ps'

xa=rebin(dindgen(field[1]),field[0],field[1],/sample)
ya=transpose(xa)

post=[3.,3.,3.+field[0],3.+field[1]]*1000.

for lk=0,nl-1 do begin
 
vel_tmp=reform(vel_map(lk,*,*))
velf=vel_tmp(where(finite(vel_tmp) and abs(vel_tmp-c*z) le 5.*c*w/l[lk]))

vellev=round((median(velf)+(median(velf(where(velf gt median(velf))))-median(velf(where(velf lt median(velf)))))*(dindgen(150)-100.)*.1)*.1)*10.
vellev=vellev(uniq(vellev))
vellev=(dindgen(100)-50.)*10.+median(vel_map)

posar=posang*!DPI/180.
ra0=sxpar(h,'RA') & dec0=sxpar(h,'DEC')
; ra='20 35 0.75'  &  dec='+60 11 30.9'
ra='03 45 55.77'  &  dec='68 04 55.2'

xch=-(ratos(ra)-ratos(ra0))*cos(posar)*cos(detos(dec0)/206265.)+(detos(dec)-detos(dec0))*sin(posar)+double(field[0])/2.
ych=(ratos(ra)-ratos(ra0))*sin(posar)*cos(detos(dec0)/206265.)+(detos(dec)-detos(dec0))*cos(posar)+double(field[1])/2.

flux_plot=-rotate(reform(flux_map(lk,*,*))/median(flux(lk,where(flux(lk,*) gt 0.))),7)
 flev=reverse(-.2*dindgen(500))
if keyword_set(losc) then flev=-10.^(3.5-.2*dindgen(20))
 flev=flev(where(flev gt min(flux_plot*2.5)))
vel_plot=rotate(reform(vel_map(lk,*,*)),7)
outfv=out[lk]+'_fv.ps'
  if strcmp(out[lk],out[(lk+1) mod nl]) then outfv=out[lk]+'_fv'+strtrim(string(lk),2)+'.ps'

  device,filename=outfv,xsize=field[0]+5.,ysize=field[1]+5.,/color
    loadct,colt,bottom=bot
    sb=1.
    if bot lt 0 then sb=-1.
    if sb lt 0 then flev=reverse(flev)
    contour,flux_plot*sb,xa,ya,levels=flev*sb,charsize=1.,/fill,pos=post,$  ;title=sxpar(h,'OBJECT')+', wavelength = '+strtrim(string(l[lk]),2)+string("305B),$
             xstyle=5,ystyle=5,/device ;,yrange=[max(ya),min(ya)]
loadct,0
    contour,flux_plot*sb,xa,ya,levels=flev*sb,charsize=2.5,charthick=3.,/noer,/nodata,$
             xstyle=1,ystyle=1,pos=post,/device ;,yrange=[max(ya),min(ya)]
;    contour,rotate(flux_map,7),/overplot,levels=10^(dindgen(60)/8.-6.),c_linestyle=replicate(2,40)
;    tvscl,flux_plot,1.,3.,xsize=field[0],ysize=field[1],/centim
;    drawne,.8,.8,posang,r=.1
;    contour,vel_plot,xa,ya,/downhill,/overplot,levels=vellev,xstyle=5,ystyle=5,$
;        c_annotation=' '+strtrim(string(vellev,format='(F8.1)'),2)
 contour1,reform(vel_map(lk,*,*)),xa,ya,vellev,post,ylim=[3.,13.]
;    contour,rotate(vel_map(lk,*,*),7),/overplot,levels=round(median(vel_map)/10.)*10.+5.*dindgen(20)-50.,c_linestyle=replicate(2,40)
    if keyword_set(dne) then drawne,.75,.8,posang,r=.1
;    xyouts,5,2,'velosities in km/s',/device
;    circle_plot,xch,ych,1.,field=field
   plotsym,0,4,color=-1,thick=10.
   plots,xch,ych,psym=8,/data,color=-1*sb,symsize=1.
 device,/close
outf=out[lk]+'_f.ps'
  if strcmp(out[lk],out[(lk+1) mod nl]) then outf=out[lk]+'_f'+strtrim(string(lk),2)+'.ps'

  device,filename=outf,xsize=field[0]+5.,ysize=field[1]+5.,/color
    loadct,colt
     if keyword_set(losc) then tvscl,rotate(-alog(reform(flux_map(lk,*,*))+median(flux_map(lk,*,*)))*sb,7),3.,3.,xsize=field[0],ysize=field[1],/centim else $
    tvscl,flux_plot*sb,3.,3.,xsize=field[0],ysize=field[1],/centim
    contour,flux_plot*sb,xa,ya,/nodata,pos=post,/device,$   ;title=sxpar(h,'OBJECT')+', wavelength = '+strtrim(string(l[lk]),2)+string("305B),$
           xstyle=1,ystyle=1,yrange=[min(ya-.5),max(ya+.5)],xrange=[min(xa-.5),max(xa+.5)],/noer,$
           charsize=2.5,charthick=3.
   loadct,0
   if keyword_set(dne) then drawne,.75,.8,posang,r=.1
;    xyouts,5,2,'integral flux '+strtrim(string(intflux),2)+'+/-'+strtrim(string(dinf),2)+'erg/cm^2 s',/device
   plotsym,0,4,color=-1,thick=5.
;   plots,(xch+1.)/21.,(ych+3.)/21.,psym=8,/norm,color=-1,symsize=1.
   plots,xch,ych,psym=8,color=-1,symsize=1.
;   scalp,flux,uni,/expf
 device,/close
endfor
loadct,0
set_plot,'x'

for lk=0,nl-1 do begin

  h2=h
  fileout=out[lk]+'_F.FTS'
  if strcmp(out[lk],out[(lk+1) mod nl]) then fileout=out[lk]+'_F'+strtrim(string(lk),2)+'.FTS'
  sxaddpar,h2,'FILE',fileout
  sxaddpar,h2,'IMAGETYP','flux map'
  sxaddpar,h2,'INTFLUX',intflux[lk],'erg/cm^2 s'
  sxaddpar,h2,'LINE',strtrim(string(l[lk]),2),' line wavelength'
  sxaddpar,h2,'DFLUX',dinf[lk],'erg/cm^2 s - integral flux uncertainty',after='INTFLUX'
  writefits,fileout,reform(flux_map(lk,*,*)),h2

  h2=h
  fileout=out[lk]+'_df.FTS'
  if strcmp(out[lk],out[(lk+1) mod nl]) then fileout=out[lk]+'_df'+strtrim(string(lk),2)+'.FTS'
  sxaddpar,h2,'FILE',fileout
  sxaddpar,h2,'IMAGETYP','flux uncertainty map'
  sxaddpar,h2,'INTFLUX',intflux[lk],'erg/cm^2 s'
  sxaddpar,h2,'LINE',strtrim(string(l[lk]),2),' line wavelength'
  sxaddpar,h2,'DFLUX',dinf[lk],'erg/cm^2 s - integral flux uncertainty',after='INTFLUX'
  writefits,fileout,reform(dflux_map(lk,*,*)),h2

  h2=h
  fileout=out[lk]+'_W.FTS'
  if strcmp(out[lk],out[(lk+1) mod nl]) then fileout=out[lk]+'_W'+strtrim(string(lk),2)+'.FTS'
  sxaddpar,h2,'FILE',fileout
  sxaddpar,h2,'LINE',strtrim(string(l[lk]),2),' line wavelength'
  sxaddpar,h2,'IMAGETYP','fwhm map'
  writefits,fileout,reform(fwhm_map(lk,*,*)),h2

  h2=h
  fileout=out[lk]+'_V.FTS'
  if strcmp(out[lk],out[(lk+1) mod nl]) then fileout=out[lk]+'_V'+strtrim(string(lk),2)+'.FTS'
  sxaddpar,h2,'FILE',fileout
  sxaddpar,h2,'LINE',strtrim(string(l[lk]),2),' line wavelength'
  sxaddpar,h2,'IMAGETYP','velocity map'
  writefits,fileout,reform(vel_map(lk,*,*)),h2
  h2=h
  fileout=out[lk]+'_dv.FTS'
  if strcmp(out[lk],out[(lk+1) mod nl]) then fileout=out[lk]+'_dv'+strtrim(string(lk),2)+'.FTS'
  sxaddpar,h2,'LINE',strtrim(string(l[lk]),2),' line wavelength'
  sxaddpar,h2,'FILE',fileout
  sxaddpar,h2,'IMAGETYP','velocity map uncertainties'
  writefits,fileout,reform(dv_map(lk,*,*)),h2

  h2=h
  fileout=out[0]+'_fit'+strtrim(string(lk),2)+'.FTS'
  sxaddpar,h2,'FILE',fileout
  sxaddpar,h2,'CRVAL1',(min(larea)-l[lk])*c/l[lk]
  sxaddpar,h2,'CDELT1',dl*c/l[lk]
  sxaddpar,h2,'CRVAL2',0.
  sxaddpar,h2,'CDELT2',1.
  sxaddpar,h2,'IMAGETYP','data cube with line profiles'
;  a=size(arasm)
  ar1=dblarr(aa[1],field[0],field[1])
  for k=0,field[0]-1 do for j=0,field[1]-1 do ar1(*,k,j)=exp(-.5D*((larea-cent_map[lk,k,j])/(fwhm_map[lk,k,j]/sqrt(8.D*alog(2.D))))^2)*$
(flux_map[lk,k,j]/(sqrt(2.D*!DPI)*fwhm_map[lk,k,j]/sqrt(8.D*alog(2.))))
  writefits,fileout,ar1,h2
endfor

  h2=h
  fileout=out[0]+'_cube.FTS'
  sxaddpar,h2,'FILE',fileout
  sxaddpar,h2,'CRVAL2',0.
  sxaddpar,h2,'CDELT2',1.
  sxaddpar,h2,'CRVAL1',(min(larea)-median(l,/even))*c/median(l,/even)
  sxaddpar,h2,'CDELT1',dl*c/median(l,/even)
  sxaddpar,h2,'IMAGETYP','data cube with line profiles'
  a=size(area)
;  ar1=dblarr(field[0]*field[1],a[1])
;  for k=0,a[1]-1 do ar1(*,k)=reform(area(k,*)-min(area),field[0]*field[1])
  writefits,fileout,area,h2

  h2=h
  fileout=out[0]+'_fit.FTS'
  sxaddpar,h2,'FILE',fileout
  sxaddpar,h2,'CRVAL1',(min(larea)-median(l,/even))*c/median(l,/even)
  sxaddpar,h2,'CDELT1',dl*c/median(l,/even)
  sxaddpar,h2,'CRVAL2',0.
  sxaddpar,h2,'CDELT2',1.
  sxaddpar,h2,'IMAGETYP','data cube with line profiles'
;  a=size(arasm)
  ar1=dblarr(a[1],field[0]*field[1])
  for k=0,a[1]-1 do ar1(k,*)=reform(arasm(*,k),field[0]*field[1])  
  writefits,fileout,ar1,h2

fin:
loadct,0
end
