pro REDUCT,LOGFILE,image_type,SHIFT=shift,Thresh=thresh,old=old

;+
; NAME:
;	REDUCTION
; PURPOSE:
;	bias substraction,replace bad pixels, remove cosmic hits and
;	correction flexibility in set MPFS-image with different type
; DESCRIPTION:
;	Routine makes bias subtraction,replaces bad pixels, removes cosmic hits and
;	correction flexibility in set of MPFS-images of the given type.
;	image shifted at different way for different value image_type:
;	obj and neon-images shifted relatively 1-st image in both direction,
;	flat-images shifted relatively reduced obj-image  only in cross-dispersion
;	direction, star,eta and test-images  shifted relatively reduced flat-image
;	in cross-dispersion direction. Routine modified FITS-header saved FITS-files
;
; CALLING SEQUENCE:
;	REDUCT, LOGFILE, image_type, SHIFT=shift, Thresh=thresh
;
; CATEGORY:
;	reduction MPFS-data
;
; INPUTS:
;	LOGFILE = file name of LOG observation (FITS-header format)
;	image_type = string array type of exposure (values: 'bias','obj','star',
;				'flat','eta','neon','test')
;       thresh - threshhold in sighmas for cosmic hits replacement 
;
; OUTPUTS:
;	Result saved at working directory in file '*_i.fts',where '*'=image_type
;
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;	SHIFT  if keyword preset image to be shiftted
;       
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;	Function  DEF_WDIR, DEF_RDIR, DEF_NAME, READFILE, READ_FTS,
;		  COSMETIC, DEF_SHI, SHIFT_IMAGE
;	Procedure SXADDPAR, SXADDHIST, REM_HITS
;
; MODIFICATION HISTORY:
;       Written by Victor Afanasiev, Special Astrophysical Observatory RAS, Jul 1999
;       modificated by A.Moiseev Jan 2000
;       Dec 2000 -- added  overscan calculation
;       Jul 2003 -- began to work properly
;	Jan 2005 -- made up for the mpfs.lib0 by P. Abolmasov
;-

on_error,0
if not(keyword_set(thresh)) then thresh=2.
message,'creation '+image_type+'-image ',/cont
image_type=strlowcase(image_type)
CASE image_type OF
	'bias':   sw=0
	'obj' :	  sw=1
   	'flat':	  sw=2
	'neon':	  sw=3
	'star':   sw=4
	'eta' :	  sw=5
	'test':   sw=6
    'eta_star':	  sw=7
   'flatstar':	  sw=8
ELSE: message,' image type has an illegal value'
ENDCASE
;reading log observation

	openr,UNIT,LOGFILE,/GET_LUN
	a=' '
	readf,UNIT,a
	TABLE=a
	WHILE NOT EOF(UNIT) do begin
     	  readf,UNIT,a
	  TABLE=[TABLE,a]
	ENDWHILE
	FREE_LUN, UNIT

;determination extention files

	EXT=SXPAR(TABLE,'EXT')                        ; = '.fts'

;determination input/output directory

	dir=DEF_RDIR(LOGFILE)
	wdir=DEF_WDIR(LOGFILE)

;determination size input images

if sw eq 0 then begin                                               ; BIAS reduction
	bias=DEF_NAME(LOGFILE,'BIAS',N_bias)
	bias=strtrim(bias,2)
	tmp=READFITS(dir+bias[0]+ext,h)
	a=size(tmp)
	Nx=a(1) & Ny=a(2)
endif
bias_mean=0

if sw gt 0 then begin
	bias_mean=READ_FTS(wdir+'bias_i.fts',SILENT='silent')
	a=size(bias_mean)
	Nx=a(1) & Ny=a(2)
endif

;CREATING REDUCED IMAGE & BAD COLUMNS REPLACEMENT
image_name=DEF_NAME(LOGFILE,image_type,N_image)
image_name=strtrim(image_name,2)

if sw eq 0 then image=intarr(Nx,Ny,N_image) else image=dblarr(Nx,Ny,N_image)
; image_header=strarr(55,N_image)
exps=dblarr(N_image)
posang=dblarr(N_image)

for k=0,N_image-1 do begin
  if sw gt 0 then begin
   history=image_name(k)
  endif
  print,'READ FILE ',image_name(k)

  image(*,*,k)=READFITS(dir+image_name[k]+EXT,h)
  exps[k]=sxpar(h,'EXPTIME')
;  image(*,*,k)=COSMETIC(image(*,*,k),[286,360]-2)-bias_mean	;replace bad column
;  image(*,*,k)=COSMETIC(image(*,*,k),[286,360])-bias_mean	;replace bad column
   image(*,*,k)=image(*,*,k)-bias_mean
  over=overscan(image(*,*,k))
  
  ptb=double(sxpar(h,'ROTANGLE'))  &  p2=double(sxpar(h,'PARANGLE'))
  posang[k]=p2-ptb+37.

  if sw eq 0 and k eq 0  then over0=over
  if sw eq 0 then over=over-over0
  image(*,*,k)=image(*,*,k)-over

  print,'Overscan value=',string(over,format='(F5.2)')
		if k eq 0 then image_header=strarr(n_elements(h),N_image)

     if n_elements(h) gt n_elements(image_header(*,k)) then h=[h(0:n_elements(image_header(*,k))-2),h[n_elements(h)-1]]
     if n_elements(h) lt n_elements(image_header(*,k)) then for i=0,n_elements(image_header(*,k))-n_elements(h)-1 do sxaddpar,h,'COMMENT'+strtrim(string(i),2),'something needed...'     

  image_header(*,k)=h 

endfor

if (abs(max(posang)-min(posang)) mod 360.) gt 10. and sw eq 1 then begin
  message,'IMAGES SIGNIFICANTLY ROTATED!',/cont
  history=history+'IMAGES SIGNIFICANTLY ROTATED!'
endif
if sw eq 1 then history=history+'position angles range: '+strtrim(string(posang),2)

header=ADD_HEADER(image_header)

if keyword_set(shift) then begin

;SHIFTING OBJ & NEON RELATIVE 1-st IMAGE
if keyword_set(shift) then begin
  image_0=image(*,*,0)
  for k=0,N_image-1 do begin
    history=image_name(k)
    if k gt 0 then begin
      dy=DEF_SHI(image_0,image(*,*,k),'y',10,50,100);!!!!!!!!!!!!c=10
      dx=0.
;      dx=DEF_SHI(image_0,image(*,*,k),'x',10,20,1000)
      image(*,*,k)=SHIFT_IMAGE(image(*,*,k),dx,dy)
      print,'shift between '+image_type+'_0 and '+image_type+'_'+string(k,format='(I1)')+$
	' dx='+string(dx,format='(F4.1)')+' dy='+string(dy,format='(F4.1)')
      history=history+' shifted relative '+image_name(0)+' (dx='+string(dx,format='(F4.1)')+$
	     ',dy='+string(dy,format='(F4.1)')+')'
      sxaddhist,history,header
    endif
  endfor
endif

; CROSS-DISPERSION SHIFTING
if sw gt 1  and sw ne 3 then begin
    if (sw eq 2)or(sw eq 6)or(sw eq 8) then begin

	name_0='obj_i'
	image_0=READ_FTS(wdir+name_0+'.fts',SILENT='silent')
		endif
    if ((sw gt 2)and(sw lt 6))or(sw eq 7) then begin
	name_0='flat_i'
	image_0=READ_FTS(wdir+name_0+'.fts',SILENT='silent')
		endif

;if sw ne 4 then begin
for k=0,N_image-1 do begin
history=image_name(k)
dy=DEF_SHI(image_0,image(*,*,k),'y',10,20,200)
dx=DEF_SHI(image_0,image(*,*,k),'x',10,50,200)


print,'Y-shift between ', name_0,' and ',image_type,'_',$
	string(k,format='(I1)'),'=',string(dy,format='(F5.2)')
history=history+' shifted relative '+name_0 + ' (dx='+string(dx,format='(F5.2)')+')'+$
' (dy='+string(dy,format='(F5.2)')+')'

image(*,*,k)=SHIFT_IMAGE(image(*,*,k),dx,dy)

sxaddhist,history,header
endfor
;endif
	endif
endif
;REMOVE COSMIC HITS
filetest=DEF_WDIR(LOGFILE)+image_type+'.tst'       ; test file

gain=sxpar(h,'GAIN')
if N_image gt 1 then image_old=total(image,3)/N_image else image_old=image
if not(keyword_set(old)) then par=[80.,150.] else par=[5.,50.]
if N_image eq 1 then begin 
 ;image_res=image
 pld=0
 if sw eq 1 then pld=1
case sw of
  0 : image_res=DESTROY(image,mask,100.,250.,0.,0.,/plot,wdir=wdir);100 250
  1 : image_res=DESTROY(image,mask,par[0],par[1],thresh,5.*thresh,/plot,wdir=wdir);80 150!!! olya 3*
  2 : image_res=etares(image,mask=mask)
  3 : image_res=rotate(etares(rotate(image,3),mask=mask,mf=thresh/3.),1)
  4 : image_res=DESTROY(image,mask,10.,80.,thresh,thresh*3.,/plot,wdir=wdir)
  5 : image_res=etares(image,mask=mask,mf=thresh)
  6 : image_res=etares(image,mask=mask,mf=thresh)                     ; DESTROY(image,mask,80.,150.,thresh,thresh*3.,/plot,wdir=wdir)
  7 : image_res=etares(image,mask=mask,mf=thresh)
  8 : image_res=etares(image,mask=mask)
endcase
 if sw eq 3 then mask=rotate(mask,1)
endif else begin
if sw eq 0 then image_res=med3d(image,mask=mask)
 if (sw eq 1 or sw eq 5 or sw eq 7 or sw eq 4) and N_image eq 2 then image_res=altermed(image*gain,3,thresh,mask=mask)/gain  ;REM_CH(image,filetest=filetest,T=thresh1) 
 if (sw eq 1 or sw eq 5 or sw eq 7 or sw eq 4) and N_image gt 2 then image_res=altermed(image*gain,3,thresh,mask=mask)/gain ;,/med_clean)/gain ;REM_CH(image,filetest=filetest,T=thresh1) 
;!!!!!!!!!!!! if (sw eq 1 or sw eq 5 or sw eq 7 or sw eq 4) and N_image gt 5 then image_res=med3d(image,mask=mask) ;REM_CH(image,filetest=filetest,T=thresh1) 
 if sw eq 6 or sw eq 2 or sw eq 8 then image_res=flares(image*gain,thresh=thresh,mask=mask)/gain
 if sw eq 3 then begin
   s=size(image)
   image1=dblarr(s[2],s[1],s[3])
   for k=0,N_image-1 do image1(*,*,k)=transpose(reform(image(*,*,k)))
   image_res=transpose(flares(image1*gain,thresh=thresh,mask=mask)/gain)
   mask=transpose(mask)
 endif
endelse

if sw ne 0 then begin

set_plot,'ps'
device,filename=wdir+image_type+'_mask.ps',/landscape,/color,xsize=27, ysize=20
; if N_image gt 1 then tvscl,rebin(image_res-total(image,3) ne 0.,517,518) else $
;          tvscl,rebin(image_res-image ne 0.,517,518)
loadct,16
;  tvscl,rebin(-alog(image_res(0:floor(a[1]/4.)*4-1,0:floor(a[2]/4.)*4-1)),floor(a[1]/4.),floor(a[2]/4.)),2.,3.,xsize=12,ysize=12,/cent
   tv,rebin(bytscl(image_res(0:floor(a[1]/4.)*4-1,0:floor(a[2]/4.)*4-1),quantile(image_res,0.25),quantile(image_res,0.85)),floor(a[1]/4.),floor(a[2]/4.)),2.,3.,xsize=12,ysize=12,/cent
loadct,0
  tvscl,rebin(mask(0:floor(a[1]/4.)*4-1,0:floor(a[2]/4.)*4-1) eq 0.,floor(a[1]/4.),floor(a[2]/4.)),14.,3.,xsize=12,ysize=12,/cent
 xyouts,0,0,'cosmic particles graveyard for '+sxpar(image_header(*,0),'OBJECT')+'; threshold '+strtrim(string(thresh,format='(F5.1)'))+' used',/norm
device,/close
set_plot,'x'

endif

; for BIAS:
if (sw eq 0) and (N_image GE 5 ) then MEAN_BI,image,image_res
if (sw eq 0) and (N_image LE 5 ) then image_res=total(image,3)  ;REM_HITS(image,NOSMOOTH='nosmooth' )
if (sw eq 0) and (N_image LE 5 ) then message,'only '+strtrim(string(N_image),2)+'bias images!',/cont

if (sw eq 0)or(sw eq 2)or(sw eq 3)or(sw eq 5)or(sw eq 7)or(sw eq 8) then image_res=image_res/N_image

if sw eq 0 then begin
  rms_bias=stdev(image_res,mean)*sqrt(N_image)
  history='mean_bias= '+string(mean,format='(F5.2)')+$
	' rms_bias='+string(rms_bias,format='(F5.2)')
  sxaddhist,history,header
endif

header1=header

image=0
data=sxpar(header,'DATE-OBS')
if not(keyword_set(old)) then time=strsplit(sxpar(header,'UT'),':',/ext) else time=strsplit(sxpar(header,'START'),':',/ext)
t=ten(time[0],time[1],time[2])
data=strsplit(data,'/',/extract) ;;,t],',')
juldate,[data[0],data[2],data[1],t],jd 
jd=jd+2.4d6 
sxaddpar,header,'JD',string(jd,for='(F20.7)')
sxaddpar,header,'POSANG',total(posang)/double(N_image),after='ROTANGLE'
if image_type eq 'obj' then sxaddpar,header,'REDSHIFT',read_log(logfile,'Z_SYS'),after='POSANG'
sxaddpar,header,'FILE',image_type+'_i.fts'
; sxaddpar,header1,'FILE',image_type+'_r.fts'
writefits,wdir+image_type+'_i.fts',image_res,header
writefits,wdir+image_type+'_mask.fts',mask,header
; writefits,wdir+image_type+'_r.fts',image_old-image_res,header1
; print,total((image_old-image_res)/image_old)
end
