function REM_HITS,obj_spp,NOSMOOTH=nosmooth,THRESH1=THRESH,thresh2=thresh_sig,file=file

;+
; NAME:
;	REM_HITS
; PURPOSE:
;	remove cosmic hits on exposure frames
; DESCRIPTION:
;
; CALLING SEQUENCE:
;	Result = REM_HITS ( obj_spp, NOSMOOTH=nosmooth)
; CATEGORY:
;	reduction MPFS-data
;
;INPUTS :
;      obj_spp - set of analised images
;
; OUTPUTS:
;	Result = total image cleaned at cosmic hits
;
; OPTIONAL OUTPUT:
;	obj_spp - set of  images cleaned at cosmic hits
;
;
; OPTIONAL INPUT KEYWORDS:
;	NOSMOOTH if keyword present, data not smoothed along dispersion
;   thresh1 - treshold for CH removing (default=4.2)
;   thresh2 - treshold for robomean (default=50)
;   file -- file name for CH image
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;	ROBOMEAN
;
; MODIFICATION HISTORY:
;       Written by Victor Afanasiev, Special Astrophysical Observatory RAS, Jul 1999
;-      Added keywords THRESH,THRESH_SIG and visualisation by A.Moiseev 11.IV.2000
;       Added some bugs by P.Abolmasov and A.Shipulin

on_error,2
a=size(obj_spp)
n=a(1)-1
m=a(2)
if a(0) eq 2 then n_exp=1 else n_exp=a(3) ; number of OBJ frames
n_replace=0
;IF a(0)	eq 2 then goto,cont_1
message,'remove cosmic hits in '+string(n_exp,format='(I2)')+' image',/cont

IF N_exp GE 2 then BEGIN
  if not(keyword_set(thresh_sig)) then thresh_sig=50             ; thresh. for sigma determination
  if not(keyword_set(thresh)) then thresh=4.2; thresh. for first iteration
  im=fltarr(m,n_exp)                                           ; array for each channel
  n_events=max([1,n_exp-2])                                  ; number events for hit removing
  list=bindgen(n_exp)
  mean_1=findgen(n_exp-1)

  old_obj=total(obj_spp,3)

          ;*******
  mul=fltarr(m,n_exp) ; normalization on  mean flux in individual channels
  for i=0,n_exp-1 do mul(*,i)=1./median(obj_spp(*,*,i))
          ;****

 FOR ch=1,n-1 do BEGIN
     im(*,*)=obj_spp(ch,*,*)
  for i=0,n_exp-1 do BEGIN   ; clear CH in each spectra
     mask=0
     newlist=where(list NE i)

         for j=0,n_exp-2 do BEGIN ; mask creating
           resid=im(*,i)*mul(*,i)-im(*,newlist(j))*mul(*,newlist(j))

           robomean,resid,thresh_sig,0.5,mean_1(j),avgdev,sigma
           dif=resid-mean_1(j)
           rec=where(dif GT thresh*sigma)
           mask=[mask,rec]
         ENDFOR        ; mask creating

                       ; mask extention

     mask=[mask,mask-1,mask+1]
     mask=mask>0
     mask=mask<(m*n_exp-1)

     hist=histogram(mask,min=0)
     hit=where(hist GE n_events,count)

         if count GT 0 then BEGIN
           ;  IF N_EXP GT 2 then av=total(im(*,newlist),2)/(n_exp-1)+total(mean_1)/(n_exp-1) else $
              ;     av=im(*,newlist(0))+mean_1(0)
            IF N_EXP GT 2 then av=total(im(*,newlist),2)/(n_exp-1)+total(mean_1/mul(0,*))/(n_exp-1) else $
            av=im(*,newlist(0))+mean_1(0)/mul(0,0)
 

     	     n_replace=n_replace+N_elements(hit)
             obj_spp(ch,hit,i)=av(hit)
         ENDIF
     ENDFOR  ; clear CH in each spectra
   ENDFOR ; ch
     result=total(obj_spp,3)
     goto,cont_3
     cont_1:
     old_obj=obj_spp
     if KEYWORD_SET(nosmooth) then goto,cont_2
     message,' image smoothed along dispersion',/cont
         for  k=0,m-1 do begin
             vector_tmp=float(obj_spp(*,k,0))
             vector_tmp=median(vector_tmp,10)
             obj_spp(*,k,0)=vector_tmp
         endfor

   cont_2:
        result=obj_spp
   cont_3:
       if a(0) ne 2  then message,string(n_replace)+' points replaced',/cont

ENDIF        else begin; N_exp<2

        
      old_obj=obj_spp
               ; ****** N_exp=1 *******
      ;     for i=0,m-1 do begin
      ;          vec=obj_spp(*,i)
      ;          ;for kk=1,2 do begin
      ;          vecm1=median(vec,5)
      ;          vecm=median(vecm1,3)
      ;          dif=vec-vecm
      ;          robomean,dif,thresh_sig,0.5,avg,avgdev,sigma
      ;          rec=where(dif GT thresh*sigma,counts)
      ;          if counts gt 0 then vec(rec)=vecm(rec)
      ;          ;endfor
      ;          obj_spp(*,i)=vec
      ;          if i mod 50 eq 0 then print,'Y:',i
      ;     endfor
        result=medfilt1(obj_spp,5,1)
    ;    result=medfilt(obj_spp,4,1) 
       
       
endelse

       ; determination of the remaints:
   rem=old_obj-result
   result=result+(rem<0)

   rem=old_obj-result

;   window,21,xs=530,ys=530,tit='CH-image'
;   tv,rebin(rem,(n+1)/2,m/2,/sample)

    if keyword_set(file) then begin
        mkhdr,header,rem
        sxaddpar,header,'OBJECT','Cosmic Hits'
        writefits,file,rem,header
    endif

RETURN,result
END
