function SCATTER,image,add=add
;+
; NAME:
;	SCATTER
; PURPOSE:
;	substraction scattering ligth
; DESCRIPTION:
;	Routine estimate level scattering light using binomial approximation
;	for every column
;       
; CALLING SEQUENCE:
;	Result=SCATTER ( image, traectory_eta)
;   add -- additive value for scattering light
; CATEGORY:
;	reduction MPFS-data
;
; INPUTS:
;	image - input MPFS image
;	traectory_tra - 2D array traectories etalon spectra
;
; OUTPUTS:
;	Result = scattering ligth substracted image
;
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;	no
;
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;	GOODPOLY
;
; MODIFICATION HISTORY:
;       Written by Victor Afanasiev, Special Astrophysical Observatory RAS, Jul 1999
;       Modified by Pavel Abolmasov, Special Astrophysical Observatory RAS, Aug 2003
;-

a=size(image)
Nx=a(1) &Ny=a(2)
tt=dindgen(Ny)
vector=fltarr(Nx)
nimage=replicate(0.,Nx,Ny)
for x=0,Nx-1 do begin
 robomean,image(x,*),3,.01,mean,dmean,sigma,var,skew,kurt,nfinel,rimage
 q=where(image(x,*) lt 4*sigma+mean)
 nimage(x,q)=image(x,q)
 coeff=goodpoly(q,nimage(x,q),2,3,yfit,newq,newim)
 nimage(x,tt)=coeff(0)+coeff(1)*tt+coeff(2)*tt^2
 print,coeff
 plot,image(x,*),yrange=[-500,2000]
 oplot,nimage(x,*)
endfor
for y=0,Ny-1 do nimage(*,y)=smooth(nimage(*,y),10)

if keyword_set(add) then begin
  f=f+add
  for y=0,Ny-1 do begin
   nimage(*,y)=nimage(*,y)-f
  endfor
endif
tv,bytscl(nimage)
return,nimage
END
