pro SHOW_3,LOGFILE,PS=ps,wdir=wdir
;+
; NAME:
;	SHOW_3
; PURPOSE:
;	visualisation 3 images :  obj_s.fts, obj_lin.fts, obj-sky.fts
; DESCRIPTION:
;
; CALLING SEQUENCE:
;	SHOW_3, LOGFILE, PS=ps
;
; CATEGORY:
;	reduction MPFS-data
;
; INPUTS:
;	LOGFILE = file name of LOG observation (in format FITS-header)
;
; OUTPUTS:
;	no
;
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;	PS - if keyword preset plot image print in POSTSCRIPT file
;	     in working directory (standard name image_3.ps)
;
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;	Function 	- DEF_NAME,
;	Procedure	- SXPAR
;
; MODIFICATION HISTORY:
;       Written by Victor Afanasiev, Special Astrophysical Observatory RAS, Jul 1999
;       rewritten by Pavel Abolmasov, Special Astrophysical Observatory RAS, Apr 2005
;-

  if not(keyword_set(wdir)) then wdir=def_wdir(logfile)
  ima1=readfits(wdir+'obj_sf.fts',h1,/sil)
  ima2=readfits(wdir+'obj_lin2.fts',h2,/sil)
  ima3=readfits(wdir+'obj-sky.fts',h3,/sil)

ima1=med_ext2(ima1,3) 
ima2=med_ext2(ima2,3) 
ima3=smooth(med_ext2(ima3-median(ima3),3)>0.,3)

a1=size(ima1)
a2=size(ima2)
a3=size(ima3)

ima1=-alog(ima1(100:a1[1]-100,*)+1.*median(abs(ima1)))
ima2=-alog(ima2(100:a1[2]-100,*)+1.4*median(abs(ima2)))
ima3=-alog(ima3(100:a3[1]-100,*)+5.*median(abs(ima3)))

  set_plot,'ps'
    device,filename=wdir+'image_3.ps',/land,xsize=27,ysize=21
      tvscl,ima1,1.*1000.,2.*1000.,xsize=25.*1000.,ysize=6.*1000.
      tvscl,ima2,1.*1000.,8.*1000.,xsize=25.*1000.,ysize=6.*1000.
      tvscl,ima3,1.*1000.,14.*1000.,xsize=25.*1000.,ysize=6.*1000.
    device,/close
  set_plot,'x'
end
