 pro SKY_TOTS,name,w_dir,Norm=Norm,PLOT=plot,MASK_TO_SKY=MASK_TO_SKY,NOT_SUB=NOT_SUB,NA=NA,readtrans=readtrans,fon=fon
;+
; NAME:
;	SKY_TOTS
; PURPOSE:
;       SKY spectra subtraction from MPFS-data
;       TOTAL sky spectra construction and masking on "bad" emisission lines
;
; CALLING SEQUENCE:
;	Result = SKY_TOTS, image_name,w_dir,Norm=Norm   ; SKY_TOTS ?
; INPUTS:
;	name = string scalar type of exposure values '*'= 'obj' or 'star'
;			or 'test'
;       w_dir = working directory
;
; OUTPUTS:
;	 saved in file  '*-sky.fts'
; OPTIONAL OUTPUT:
;	no
; OPTIONAL INPUT KEYWORDS:
;       MASK_TO_SKY - masking lines 5577 and 6300 after subtraction
;       NA -- for NaD masking
;       Norm = factor for sky normalization ( default=1)
;       NOT_SUB=NOT_SUB = without subtraction ( only masking 5577)
;	    PLOT - on control plotting
;       readtrans - flag for using transparency map (trans.map)
;	fon - 
;	
; ERROR HANDLING:
;	no
; RESTRICTIONS:
;	no
; NOTES:
;  default files with extended parameters
; 'mask_oh.txt'; mask on the bad raw at the sky-vectors
; 'sky_lens.txt'; good lens with sky spectra
; 'sky_line.txt'; positions sky lines for transparency corrections
; 'sky_pos.txt'; positions for cross-correlation
;
; PROCEDURES USED:
;	Functions:  NUMLINES,def_cor,vecshift,lineflux,gaus_fit
;	Pricedure:  SXADDPAR
; MODIFICATION HISTORY:
;	MODIFIED by A.Moiseev ,SAO RAS  Mar, 2000
;       Analized and criticized by the students of Stellar Physics Lab., SAO RAS Jul 2003
;-

 On_error,0                    ;Return to user

COMMON RAND,seed

; INPUT files :
filein  =name+'_lin.fts'  ; input cube
filemask='mask_oh.txt'    ; mask on the bad raw at the sky-vectors
filelens='sky_lens.txt'   ; good lens with sky spectra
filepos ='sky_pos.txt'    ; positions for cross-correlation
fileline='sky_line.txt'   ; positions sky lines for transparency corrections

; OUTPUT files :

fileout=name+'-sky.fts' ; otput cube
filesky='sky_tot.fts' ; total sky spectra
norms=1

if not(keyword_set(norm)) then norm=1. ; norm factor for sky subtraction
if not(keyword_set(MASK_TO_SKY)) then MASK_TO_SKY=0 else MASK_TO_SKY=1

loadct,0    ; loading color table
Nx=16  ;\cube size without sky probes
Ny=16   ;/
cont=4 ; points for continuum
a=''

CD,w_dir,CURRENT=old_dir

; read cube
im=read_fts(filein,header) ; read OBJ
 ld0 =sxpar(header,'CRVAL1')           ; init. wavelength
 dld0=sxpar(header,'CDELT1')           ; linear dispersion

 Nz=sxpar(header,'NAXIS1')
 Sy=sxpar(header,'NAXIS2')
 lda=findgen(nz)*dld0+ld0            ; wavelength grid

; *** read mask positions [in A] ***
mask=0.
ss2=size(findfile(filemask))
if ss2(0) ne 0 then begin
   num=numlines(filemask)-1
   mask=fltarr(2,num)
   openr,1,filemask
   readf,1,a ; skip comment
   readf,1,mask
   close,1
endif else num=0

; *** read good lens positions [in px] ***
ss=size(findfile(filelens))
if ss(0) ne 0 then begin
   num2=numlines(filelens)-1
   rec_lens=fltarr(num2)                                     ; rec_lines - 
   openr,1,filelens
    readf,1,a ; skip comment
    readf,1,rec_lens
    rec_lens=rec_lens-1
   close,1
  endif else begin rec_lens=indgen(nx) & num2=nx & endelse      ; it was The End

; *** read  line positions [in px] ***
ss3=size(findfile(filepos))
IF ss3(0) ne 0 THEN BEGIN
   num3=numlines(filepos)-1
   sky_pos=fltarr(2*num3)
    openr,1,filepos
     readf,1,a ; skip comment
     readf,1,sky_pos
    close,1
 ENDIF ELSE BEGIN num3=1 & sky_pos=[0,nz-1] & ENDELSE

; *** read  sky line positions [in A] for transprency correction***
 ss4=size(findfile(fileline))
 IF ss4(0) ne 0 THEN BEGIN
         num4=numlines(fileline)-1
   line_trans=fltarr(2,num4)
    openr,1,fileline
     readf,1,a ; skip comment
     readf,1,line_trans                              ; line_trans!
    close,1
  ENDIF

; read transparensy ( if need)
if keyword_set(readtrans) then trans_map=readfits('trans.fts') else trans_map=make_array(nx,ny,val=1.)
CD,old_dir

sky=im(*,rec_lens)
; masking Object emission  lines in sky spectra:
mask=round((mask-ld0)/dld0)                        ; convert from  wavelength scale to true position
for i=0,num-1 do for j=0,num2-1 do sky(*,j)=mask_vec(sky(*,j),mask(0,i),mask(1,i),win=5)

; creation numbers of index for cross-correlation between sky-lines
recind=0
FOR i=0,num3-1 do recind=[recind,sky_pos(2*i)+indgen(sky_pos(2*i+1)-sky_pos(2*i))]
recind=recind(1:*)

ref_probe=sky(*,num2/2)
; sky offset shifting
FOR i=0,num2-1 DO begin
    vec=sky(*,i)
    dx=def_cor(vec-med_ext(vec,100),ref_probe-med_ext(ref_probe,100),fwhm=5)
    sky(*,i)=vecshift(vec,dx=dx)
    print,i,'shift:',dx
ENDFOR
ref_probe=sky(recind,num2/2)

;median averaging of the sky lines
tot_sky=fltarr(nz)

if num2 LT 20 then for i=0,nz-1 do tot_sky(i)=median(sky(i,*)) $
   else begin ; robust mean
    for i=0,nz-1 do begin
         robomean,sky(i,*),4,3,avg
         tot_sky(i)=avg
     endfor
   endelse
; sky line flux calculation
IF ss4(0) ne 0 THEN BEGIN
   sky_flx=fltarr(num4)
   for i=0,num4-1 do sky_flx(i)=lineflux(tot_sky,line_trans(0,i),fwhm=line_trans(1,i),cont=cont,lambda=lda)
ENDIF

; preliminary transparency determination:
if not(keyword_set(readtrans)) then begin
 IF ss4(0) ne 0 THEN BEGIN
  for j=nx,ny-1 do begin
   trans=fltarr(num4)
    for i=0,num4-1 do begin
        obj_flx=lineflux(im(*,j),line_trans(0,i),fwhm=line_trans(1,i),cont=cont,lambda=lda)
        trans(i)=obj_flx/sky_flx(i)
     endfor
     ws=(sky_flx>1)^.8 ; weights for averaging
     trans_map(j-nx)=total(trans*ws)/total(ws)
  endfor
  trans_map=med_ext2(trans_map,5)

 ENDIF
endif

; subtraction sky

newobj=im                                                       ; !!!
dx=0
w_s=(n_elements(recind)/10.)>3
for j=nx,ny-1 do begin
  vec=im(*,j)
  IF ss3(0) ne 0 THEN dx=def_cor(vec(recind)-med_ext(vec(recind),w_s),tot_sky(recind)-med_ext(tot_sky(recind),w_s)) $; shifting between sky and object spectra

                 ELSE dx=0.

  ; flux of reference lines determination:
  norms=trans_map(j-nx)
  ;IF ss4(0) ne 0 THEN BEGIN
  ;if keyword_set(readtrans) then norms=tr_map(j-nx) else begin
  ;  trans=fltarr(num4)
  ;  for i=0,num4-1 do begin
  ;;      obj_flx=lineflux(vec,line_trans(0,i),fwhm=line_trans(1,i),cont=cont,lambda=lda)
  ;      trans(i)=obj_flx/sky_flx(i)
  ;   endfor
  ;   ws=(sky_flx>1)^.8
  ; norms=total(trans*ws)/total(ws);norms=total(trans)/num4
  ;endelse
  ;ENDIF  else norms=1

if not(keyword_set(not_sub)) then newobj(*,j)=vec/norms-norm*vecshift(tot_sky,dx=-dx) ; subtraction sky
l5577=round((5577-ld0)/dld0)
lNA=round((5893-ld0)/dld0)
l6300=round((6300-ld0)/dld0)
   if MASK_TO_SKY eq 1 then BEGIN
      ll = RANDOMN(Seed)
      if (l5577 GT 7)and(L5577 LT Nz-7) then newobj(*,j)=mask_vec(newobj(*,j),l5577-7,l5577+7) ; masking residuals from 5577
      if (l6300 GT 7)and(L6300 LT Nz-7) then newobj(*,j)=mask_vec(newobj(*,j),l6300-5,l6300+5) ; masking residuals from 6300

     ;if keyword_set(Na) then if (lNa GT 7)and(LNa LT Nz-7) then newobj(*,j)=mask_vec(newobj(*,j),lna-4,lna+4,win=3) ; masking residuals from Na D
     if keyword_set(Na) then $
         if (lNa GT 7)and(LNa LT Nz-7) then newobj(lna-7:lna+5,j)=med_ext(newobj(lna-7:lna+5,j),5) ; masking residuals from Na D
   ENDIF
  trans_map(j-nx)=norm*norms
ENDFOR

newobj=newobj(*,nx:*)

;window,21,tit='Relative transparency'
;plot,trans_map(*),xst=1,yst=1,xtit='Lens N',ytit='OBJ_LINES/SKY_LINES'


if keyword_set(fon) then begin
 ; correction for oversubtraction in the BLUE region:
 x=findgen(nx*ny)
 for l=10,Nz-10 do begin
   vec=fltarr(nx*ny)
   for k=0,nx-2 do begin          
    vv=newobj(l,k*nx:(k+1)*nx-1)                        ; newobj - object with subtrusted sky
    vv=vv(sort(vv))
    if k eq 0 then kk=8 else kk=2
    vec(k*nx:(k+1)*nx-1)=median(vv(0:kk))
   endfor
   newobj(l,*)=newobj(l,*)-vec+median(vec) 

   fonb=newobj(l,nx:ny*nx-nx)
   fonb=fon(sort(fonb))
   newobj(l,*)=newobj(l,*)-median(fonb(0:3))
  endfor
endif

; writting  data
CD,w_dir
h=header
sxaddpar,header,'FILE',fileout
sxaddpar,header,'COMMENT',' '
sxaddhist,'SKY_TOTS.PRO: Subtraction  sky spectra: '+filesky+' with norm factor '+$
          string(norm,format='(F5.3)'),header

if ss2(0) ne 0 then sxaddhist,'SKY_TOTS.PRO: Masking sky lines from file: '+filemask,header
if ss(0) ne 0 then sxaddhist,'SKY_TOTS.PRO: Sky spectra is sum of good probes from file :'+filelens,header
if ss3(0) ne 0 then sxaddhist,'SKY_TOTS.PRO: Regions for cross-correlation from file :'+filepos,header
if ss4(0) ne 0 then sxaddhist,'SKY_TOTS.PRO: Transparency correction from file :'+fileline,header
if keyword_set(fon) then  sxaddhist,'SKY_TOTS.PRO: background correction in each spectral channel',header

sxaddpar,h,'FILE',filesky
writefits,fileout,newobj,header
; sky spectra
mkhdr,h,tot_sky
sxaddpar,h,'FILE',filesky
sxaddpar,h,'OBJECT','Sky spectra'
l0=sxpar(header,'CRVAL1',comment=c1)
dl=sxpar(header,'CDELT1',comment=c2)
bu=sxpar(header,'BUNIT')
sxaddpar,h,'CRVAL1',l0,c1
sxaddpar,h,'CDELT1',dl,c2
sxaddpar,h,'BUNIT',bu
sxaddhist,'SKY SUB.PRO: Sky spectra from file '+filein,h
writefits,filesky,tot_sky,h

; write transmition map
if not(keyword_set(readtrans)) then writefits,'trans.fts',reform(trans_map,nx,ny)

; *** plot results ....
set_plot,'x'
if  keyword_set(PLOT) then window,xs=600,ys=600,tit='Sky subtraction' $
   else begin ; PS-file
        set_plot,'ps'
        device,file=name+'-sky.ps',xs=20,ys=26,xoff=0,yoff=0,/port
        loadct,0
   endelse

;erase
!p.multi=[0,1,2]
plot,findgen(nz)*dl+l0,tot_sky,xst=1,yst=1,$
    tit='Total sky spectra: '+w_dir+filein,xtit='!7k!3 A',ytit='counts'
ch=120
yr=[0,5*median(im(0.3*sy:0.8*sy,ch))]
plot,lda,im(*,ch),xst=1,yst=1,yr=yr,$
    tit='Sky subtraction: '+w_dir+filein,xtit='!7k!3 A',ytit='counts'
oplot,lda,newobj(*,ch),col=200

;if not(keyword_set(PLOT)) then begin device,/close & set_plot,'win' & end
!p.multi=[0,0,0]
 CD,old_dir

END
