function slice,datacube,lambda,fwhm
;+
; NAME:
;	SLICE
; PURPOSE:
;       making monochromatic image from a datacube
;	
; DESCRIPTION:
;       for the every fiber, flux on the given wavelength with a gaussian weights is calculated,
;	the width of the filter given by fwhm, and the central wavelength by lambda
; CALLING SEQUENCE:
;	result=SLICE(datacube,lambda,fwhm)
;
; CATEGORY:
;	reduction MPFS-data
;
; INPUTS:
;	datacube - fits-file with primary reducted mpfs-data
;       lambda - average wavelength, fwhm - fwhm of the gaussian filter
; OUTPUTS:
;	image in the filter
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;       no
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;	no       
; MODIFICATION HISTORY:
;       Written by Pavel Abolmasov, Special Astrophysical Observatory RAS, Jul 2003
;-      
; function makes a monochromatic image 
  
  cube=readfits(datacube,h)
  image=fltarr(16,15)
  x=indgen(1034)
  dl=sxpar(h,'CDELT1')
  l0=sxpar(h,'CRVAL1')
  sigma=fwhm/sqrt(2*alog(2))
  sigma=sigma/dl
  x0=(lambda-l0)/dl
  x=x(x0-fix(4*sigma):x0+fix(4*sigma))
  for i=0,15 do begin
    for j=0,14 do image(i,j)=total(cube(x,i+16*j)*exp(-.5*((x-x0)/sigma)^2))/sqrt(2*!DPI*sigma^2)
  endfor
  sxaddpar,h,'NAXIS1',16
  sxaddpar,h,'NAXIS2',15
  sxdelpar,h,'CDELT1'
  sxdelpar,h,'CRVAL1'
  sxaddpar,h,'CENTRAL_WAVELENGTH',lambda
  sxaddpar,h,'FILTER_FWHM',fwhm
  writefits,'~/MPFS/trash/obj_'+string(fix(lambda))+'.fts',image,h
  return,image
end
