pro span,insp,lar,av=av,flux=flux,ston=ston,velt=velt,smoo=smoo,fwhm=fwhm,dvelt=dvelt,$
 dflux=dflux,dfwhm=dfwhm,dspfile=dspfile,chisq=chisq,ew=ew,dew=dew,redshift=redshift

;+
; NAME:
;	span
; PURPOSE:
;	line parameters determination from 1D spectrum
; DESCRIPTION: 
;	fits line profile with a gaussian (several gaussians) and computes fluxes etc. from its parameters
;
; CALLING SEQUENCE:
;	span,file,line_array,...
;        span,wdir+'obj_spect.fts',[6717.,6731.];    [4101.33,4685.682,4861.33,4958.91,5006.84,5754.8,5875.75,$
;  6300.23,6363.82,6678.15,5200.7,4471.48,4658.1]
;CI_Cam--[4025.49,4101.737,4233.167,4340.468,4471.477,4508.283,4515.33,4713.143,4731.439,4861.332,5015.675,5169.030,5316.609,5362.864,5534.860]
;
; CATEGORY:
;	spectral analysis		
;
; INPUTS:
;	FILE - 1D fits-file containing the spectrum
;	line_array - rest-frame wavelengths (redshift z for distant galaxies can be taken from the header, or 
;		added by keyword dv in km/s)
;	WDIR - output directory
; OUTPUTS:
;	output fts and ps files with names of the types: [lambda]_{fv,w}.ps and [lambda]_{f,v,w,c}.fts
;	containing fluxes (f), velosities (v) and fwhm in velosity units (w)
;		
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;	
;	Procedures:  ?
;
; MODIFICATION HISTORY:
;       Written by P. Abolmasov, Special Astrophysical Observatory RAS, Jul 2004
;-

  sp=readfits(insp,hsp)

if not(keyword_set(dspfile)) then dspfile=exdir(insp)+'dsp.fts'
dsp=readfits(dspfile,hdsp)

if n_elements(dsp) eq 1 then dsp=sqrt(insp/median(sp))*stdev(sp)

c=double(2.9979e+5)

  wdir=exdir(insp)
; dva=defwl(wdir+'neon_lin.fts',wdir)
  nl=n_elements(lar)
  if keyword_set(smoo) then sp=smooth(sp,2.*smoo+1)
  uni=sxpar(hsp,'BUNIT')
 if not(keyword_set(redshift)) then redshift=sxpar(hsp,'REDSHIFT')
  dl=sxpar(hsp,'CDELT1')
if dl le 0. then dl=sxpar(hsp,'CD1_1')
  l=dindgen(n_elements(sp))*dl+sxpar(hsp,'CRVAL1')
  
;  if keyword_set(av) then ccm_unred,l,sp,av/3.1
  w1=where(l gt min(lar*(redshift+1.))-50 and l lt max(lar*(redshift+1.))+80.)
  l1=l(w1) & sp1=sp(w1) & dsp1=dsp(w1)
  if keyword_set(av) then ccm_unred,l1,sp1,av/3.1
  if keyword_set(av) then ccm_unred,l1,dsp1,av/3.1
; string("261B)  is  "+/-" sign

set_plot,'ps'
device,filename=wdir+'span.ps',/land,xsize=23,ysize=18,xoffset=0
loadct,0
  if nl gt 1 then begin
;   if max(sp1) gt 1.e37 then sp2=sp1*double(1.e-37) else sp2=sp1
;   m=double(1.e-17)
   wl=where(l1-min(lar*(redshift+1.)) le -10. or l1-max(lar*(redshift+1.)) ge 10 and finite(sp1))
    bgd=goodpoly(l1(wl),sp1(wl),1,1.)
    if max(lar) eq min(lar) then resp=multigaus1(l1,(sp1-bgd[0]-bgd[1]*l1),lar*(redshift+1.)+(dindgen(nl)-0.5)*10./double(nl),fwhm=replicate(6.,nl),sigma=sig,yfit=spfit,/plot,lim_fwhm=[1.,30.],chisq=chisq,dy=dsp1) else $
        resp=multigaus1(l1,(sp1-bgd[0]-bgd[1]*l1),lar*(redshift+1.),fwhm=replicate(6.,nl),sigma=sig,yfit=spfit,/plot,lim_fwhm=[1.,30.],chisq=chisq,dy=dsp1)
    spfit=spfit
    flux=resp.flux
    dflux=sig.flux
; dflux=total(dsp(w1(wl)))
    cent=resp.center
    dcent=sig.center   
    fwhm=resp.fwhm
    dfwhm=sig.fwhm
;    if max(sp1) gt 1.e37 then rms=stdev(sp1-spfit)*double(1.e+37) else 
    rms=stdev(sp1-spfit-bgd[0]-bgd[1]*l1)*fwhm/dl
   ; rms0=stdev(sp1(wl))
  ;  dcent=fwhm*dflux/flux/sqrt(n_elements(spfit))

   if keyword_set(ew) then begin
     spnorm=sp1/(bgd[0]+bgd[1]*l1)
     if max(lar) eq min(lar) then resn=multigaus1(l1,1.-spnorm,lar*(redshift+1.)+(dindgen(nl)-0.5)*20./double(nl),fwhm=replicate(10.,nl),sigma=sign,yfit=spfitn,lim_fwhm=[1.,50],chisq=chisq,dy=dsp1/(bgd[0]+bgd[1]*l1),absorp=(indgen(nl) mod 2)) else $
         resn=multigaus1(l1,spnorm,lar*(redshift+1.),fwhm=replicate(6.,nl),sigma=sign,yfit=spfitn,lim_fwhm=[10.,30.],chisq=chisq,dy=dsp1/(bgd[0]+bgd[1]*l1))
      ew=resn.flux
      dew=sign.flux
      fwhm1=resn.fwhm
      dfwhm1=sign.fwhm
      cent1=resn.center
      dcent1=sign.center
dspnorm=dsp1/(bgd[0]+bgd[1]*l1)
velt1=(cent1-lar)*c/lar
dvelt1=dcent1*c/lar
   endif

 endif else begin 
   sp2=smooth(sp1,3)
   sp2=med_ext(sp2,5)
;   m=total(sp1-min(sp2))
   m=double(1.e-17)
   wl=where(abs(l1-replicate(reform(lar*(redshift+1.)),n_elements(l1))) gt 10 and finite(sp1))
   bgd=goodpoly(l1(wl),sp1(wl),1,1.)
   spfit=gaussfit(l1,(sp1-poly(l1,bgd))/m,g0,nterms=3,$ ;measure=dsp1/m,$
       estimates=[max(sp1/m)-min(sp2/m),lar*(redshift+1.),7.],sigma=sig,chisq=chisq)*m
;sig=sig*sqrt(double(n_elements(l1)-3)/chisq)
   rms=stdev(sp1-spfit-poly(l1,bgd))
  ; wgood=where(abs(sp1-spfit-poly(l1,bgd)) le rms)
  ; spfit=gaussfit(l1(wgood),(sp1(wgood)-poly(l1(wgood),bgd))/m,g,nterms=3,estimates=abs(g0))*m
  ; g[2]=abs(g[2])
  ; flux=total(sp1-poly(l1,bgd))*dl            
   g=g0
   flux=sqrt(!DPI*2.)*g[0]*g[2]*m
   cent=g[1]
 ;  spfit=exp(-.5*((l1-cent)/g[2])^2)*g[0]*m
   dflux=flux*sqrt((sig[0]/g[0])^2+(sig[2]/g[2])^2)                 ;stdev(sp1-spfit)*g[2]*sqrt(8.*alog(2.))/dl
; dflux=total(dsp(w1(wl)))
   plot,l1,sp1,psym=2,yrange=[min(spfit+poly(l1,bgd)),max(spfit+poly(l1,bgd))]
   oploterr,l1,sp1,dsp1,2
   oplot,l1,spfit+poly(l1,bgd)
   fwhm=2.35*g[2]
   dfwhm=2.35*sig[2]
   dcent=sig[1]         ;fwhm*dflux/flux
   rms=stdev(sp1-poly(l1,bgd)-spfit)*fwhm/dl

   if keyword_set(ew) then begin
      spnorm=sp1/poly(l1,bgd)
      spfitn=gaussfit(l1,1.+spnorm,g0n,nterms=3, measure=dsp1/poly(l1,bgd),$
        estimates=[max(spnorm)-min(spnorm),lar*(redshift+1.),3.],sigma=sign,chisq=chisq) 
      ew=sqrt(!DPI*2.)*g0n[0]*g0n[2]
      dew=abs(ew*sqrt((sign[0]/g0n[0])^2+(sign[2]/g0n[2])^2))
      fwhm1=2.35*g0n[2]
      dfwhm1=2.35*sign[2]
      cent1=g0n[1]
      dcent1=sign[1]
dspnorm=dsp1/poly(l1,bgd)
velt1=(cent1-lar)*c/lar
dvelt1=dcent1*c/lar
   endif
;   dflux=rms*fwhm/dl
 ;  dcent=fwhm*dflux/flux/sqrt(n_elements(spfit))
;   dfwhm=fwhm*dflux/flux
  endelse

  ston=flux/rms
  velt=(cent-lar)*c/lar
  wv=fwhm*c/lar
  dvelt=dcent*c/lar

sar=sort(cent)
lar=lar(sar)  &  velt=velt(sar)  &  wv=wv(sar)  &  dvelt=dvelt(sar)
flux=flux(sar)  &  dflux=dflux(sar)  &  fwhm=fwhm(sar)

lof=floor(alog10(flux))

for k=0,nl-1 do begin
  xyouts,.2,.9-.04*k,'F!D'+strtrim(string(k+1),2)+'!N = ('+strtrim(string(flux[k]/10.^lof[k],format='(F5.3)'),2)+string("261B)+$
        strtrim(string(dflux[k]/10.^lof[k],format='(F5.3)'),2)+') 10!U'+strtrim(string(lof[k],format='(I5)'),2)+$
        '!Nerg/cm!U2!Ns ',/norm,charsize=1.2,charthick=3.
  xyouts,.65,.8-.04*k,'v!D'+strtrim(string(k+1),2)+'!N = ('+strtrim(string(velt[k],format='(F10.3)'),2)+string("261B)+$
        strtrim(string(dvelt[k],format='(F10.3)'),2)+') km/s ',/norm,charsize=1.2,charthick=3.
  xyouts,.2,.6-.04*k,'FWHM!D'+strtrim(string(k+1),2)+'!N = ('+strtrim(string(fwhm[k],format='(F10.3)'),2)+string("261B)+$
        strtrim(string(dfwhm[k],format='(F10.3)'),2)+')'+string("305B),/norm,charsize=1.2,charthick=3.
;  xyouts,.6,.6-.04*k,'FWHM!D'+strtrim(string(k+1),2)+'!N = ('+strtrim(string(sqrt((fwhm[k]*c/lar[k])^2-dva^2>0.),format='(F10.2)'),2)+string("261B)+$
;        strtrim(string(dfwhm[k]*c/lar[k],format='(F10.2)'),2)+') km/s',/norm,charsize=1.2,charthick=3.
endfor

device,/close
;set_plot,'x'
 
if keyword_set(ew) then begin
 ;set_plot,'eps'
 device,filename=wdir+'span_ew.ps',/land,xsize=23,ysize=18,xoffset=2.5,yoffset=27
  loadct,0  ;,/sil
plot,l1,spnorm,psym=1,yrange=[0,max(spnorm)<10],thick=3,charsize=1.5,charthick=2,/xst
oploterr,l1,spnorm,dspnorm,2
;if nl gt 1 then oplot,l1,1.-spfitn,linestyle=0 else oplot,l1,spfitn,linestyle=0 
oplot,l1,spfitn,linestyle=1;,color=1000
oplot,l1,bgd,linestyle=0

for k=0,nl-1 do begin
 xyouts,.1,.05-.04*k,'EW!D'+strtrim(string(k+1),2)+'!N=('+strtrim(string(ew[k],format='(F10.5)'),2)+string("261B)+$
    strtrim(string(dew[k],format='(F10.5)'),2)+')'+string("305B),/norm,charsize=1.2,charthick=3.
xyouts,.42,.05-.04*k,'FWHM!D'+strtrim(string(k+1),2)+'!N = ('+strtrim(string(fwhm1[k],format='(F10.3)'),2)+string("261B)+$
        strtrim(string(dfwhm1[k],format='(F10.3)'),2)+')'+string("305B),/norm,charsize=1.2,charthick=3.
xyouts,.7,.05-.04*k,'v!D'+strtrim(string(k+1),2)+'!N = ('+strtrim(string(velt1[k],format='(F10.3)'),2)+string("261B)+$
        strtrim(string(dvelt1[k],format='(F10.3)'),2)+') km/s ',/norm,charsize=1.2,charthick=3.
endfor
 device,/close
endif

set_plot,'x'
  rms=stdev(sp1-spfit)
 ; message,'fluxes: '+strtrim(string(resp.flux,format='(E15.5)'),2),/cont
 ; message,'velocities: '+strtrim(string((resp.cent-lar)/lar,format='(F8.2)'),2),/cont
  print,'fluxes: ',flux
  print,'velocities: ',velt   ;(cent-lar)*c/lar
  print,'FWHM: ',fwhm
  print,'S/N: ',flux/rms
  print,'chisq =',chisq
if keyword_set(ew) then print,'EW: ',ew
if keyword_set(ew) then print,'DEW: ',dew
;  flux=resp.flux
end
