function SPEXTR,image,traectory, fwhm
;+
; NAME:
;	SPEXTR
; PURPOSE:
;	extraction spectra in MPFS-image
; DESCRIPTION: 
;	
; CALLING SEQUENCE:
;	Result = SPEXTR ( image , traectory , fwhm )
;
; CATEGORY:
;	reduction MPFS-data		
;
; INPUTS:
;	image - input image (2D float point array)
;	traectory - 2D array traectory spectra
;	fwhm - width of the strobe
;
; OUTPUTS:
;	Result =2D array spectra
;		
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;	no	
;
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;	;
; MODIFICATION HISTORY:
;       Written by Victor Afanasiev, Special Astrophysical Observatory RAS, Jul 1999
;-

w=2*fwhm
;tabulation shifting GAUSS-profile in interval [-w,w]
N=100
gauss=fltarr(2*w+1,N)
sigma=w*0.5/(2*sqrt(2*ALOG(2)))
x=findgen(2*w+1)-w
for k=0,N-1 do begin
dx=k/float(N)
gauss(*,k)=exp(-((x-dx)/sigma)^2/2)
endfor
;extraction spectra
a=size(image)
Nx=a(1)  & Ny=a(2)
b=size(traectory)
Nfib=b(2)

;spectra extraction
spectr=fltarr(Nx,Nfib)
for x=0,Nx-1 do begin
vector=image(x,*)
	for y=0,Nfib-1 do begin
yc=traectory(x,y)
nc=fix(yc)   &   dy=yc-nc
spectr(x,y)=total(vector(nc-w:nc+w)*gauss(*,dy*N))
endfor
;if (x/100-x/100.) eq 0 then print,x
endfor
return,spectr
END
