 pro SUB_NSKY,LOGFILE,image_name,LINE=line,PLOT=plot
;+
; NAME:
;	SUB_NSKY
; PURPOSE:
;	substraction nigth sky spectrum in MPFS data
;
; CALLING SEQUENCE:
;	Result = SUB_NSKY, LOGFILE, image_name, [LINE='line']
; INPUTS:
;	LOGFILE = file name of LOG observation (in format FITS-header)
;	image_name = string scalar type of exposure values '*'= 'obj' or 'star'
;			or 'test'
;	routine read in working directory coefficients dispersion
;	curves (file 'disper.fts'), extracted linearised spectra 
;	object (file '*_lin.fts') and nigth sky probe (file '*_ns.fts)
; OUTPUTS:
;	 saved in file  '*-sky.fts'
; OPTIONAL OUTPUT:
;	no
; OPTIONAL INPUT KEYWORDS:
;	LINE - use separate nigth sky line for determination accuracy
;		sky substraction
;	PLOT - on control plotting
; ERROR HANDLING:
;	no
; RESTRICTIONS:
;	no
; NOTES:
;	no
; PROCEDURES USED:
;	Functions:  SPEXTR, PAR_LINE, LINEAR,READ_FTS
;	Pricedure:  SXADDPAR
; MODIFICATION HISTORY:
;	MODIFIED by Victor Afanasiev, Special Astrophysical Observatory  July, 1999
;-
  On_error,2                    ;Return to user
message,'substraction nigth sky spectra in '+image_name+'-spectra',/cont
wdir=DEF_WDIR(LOGFILE)
traectory=READ_FTS(wdir+'eta_tra.fts',SILENT='silent')
a=size(traectory)
Nx=a(1)  & N_sky=a(2)
disper_par=READ_FTS(wdir+'disper.fts',SILENT='silent')
a=size(disper_par)
N_deg=a(1)-2 & Nfib=a(2)
image_lin=READ_FTS(wdir+image_name+'_lin.fts',head_lin)
a=size(image_lin)
N_linear=a(1)
image_sub=image_lin
;read sky spectra 
sky=READ_FTS(wdir+image_name+'_ns.fts',SILENT='silent')
;determination dispersion curve for sky spectra
disper_sky=dblarr(N_deg+2,N_sky)
disper_sky(*,0)=disper_par(*,0)
disper_sky(*,N_sky-1)=disper_par(*,Nfib-1)
for k=1,N_sky-2 do begin
  disper_sky(*,k)=total(disper_par(*,16*k-1:16*k),2)/2
endfor
param=[SXPAR(head_lin,'CRVAL1'),SXPAR(head_lin,'CDELT1'),N_linear]
;linerisation sky spectrum
sky_lin=LINEAR(sky,disper_sky,N_linear,param(0),param(1))
x_hist=findgen(1001)*10
hh=histogram(sky_lin,min=0,max=10000,binsize=10)
a=max(hh,N_max)
I_max=x_hist(N_max)
sky_average=sky_lin
x_sky=findgen(N_sky)

;goto,cont1
for x=0,N_linear-1 do begin
f=goodpoly(x_sky,sky_lin(x,*),2,2,Yfit)
sky_average(x,*)=Yfit
endfor

	if keyword_set(plot) then begin
window,13,xsize=600,ysize=1000,xpos=0,ypos=0
!P.background=255 & !P.color=0
x_hist=findgen(1001)*10
hh=histogram(sky_lin,min=0,max=10000,binsize=10)
a=max(hh,N_max)
I_max=x_hist(N_max)
wavelength=findgen(N_linear)*param(1)+param(0)
plot,wavelength,total(sky_lin,2)/16,xst=1,yrange=[-5,80],yst=1,$
	NODATA='nodata',position=[0.05,0.05,0.8,0.97],/norm
sky_mean=total(sky_average,2)
	c=50 & m=200
for j=0,N_sky-1 do begin
vector1=rebin(sky_mean,N_linear*c)
vector2=rebin(sky_average(*,j),N_linear*c)
CRSPROD,vector1,vector2,m,CORR
x=findgen(2*m+1)/c-m/c
tmp=max(corr(m+1-2*c:m+1+2*c),Nmax)
res=x(Nmax+m+1-2*c)
oplot,wavelength,sky_lin(*,j)/I_max+j*5,color=100
oplot,wavelength,sky_average(*,j)/I_max+j*5,color=0
xyouts,wavelength(N_linear-1),j*5,string(res,format='(F8.2)'),/data
endfor
	endif
sky_lin=sky_average
cont1:
if keyword_set(line) then begin
;determination rigth position N.S. line in sky spectra
list_pos_line_sky=[5577.,6300.4,6363.]
list_int_line_sky=[   1.,   0.5,   0.1]
b=where(list_pos_line_sky gt param(0) $
	    and list_pos_line_sky lt param(0)+param(1)*(N_linear-1),count)
if count eq 0 then message,'no brigth sky line in used spectral coverage',/cont
if count eq 0 then goto,cont
list_pos_line_sky=list_pos_line_sky(b)
list_int_line_sky=list_int_line_sky(b)
max_int=max(list_int_line_sky,Nmax)
line=list_pos_line_sky(Nmax)
message,'sky line'+string(line,format='(F8.2)')+' used for accuracy sky substraction',/cont
lines_parameter=fltarr(3,Nfib)
pos_line=(line-param(0))/param(1)
tmp=fltarr(3,N_sky)
for k=0,N_sky-1 do begin
tmp(*,k)=par_line(sky_lin(*,k),param,3.,line)
sky_lin(*,k)=shift_s(sky_lin(*,k),pos_line-tmp(1,k))
endfor
sky_parameter=fltarr(N_linear,N_sky-1)
;determination rigth position N.S. line in object spectra and
;correction transmission fiber of object
for k=0,Nfib-1 do begin
ks=k/N_sky
vector_sky=total(sky_lin(*,ks:ks+1),2)/2
tmp=par_line(vector_sky,param,3.,line)
tmp_obj=par_line(image_sub(*,k),param,3.,line)
image_sub(*,k)=shift_s(image_sub(*,k),pos_line-tmp_obj(1))
tmp_obj=par_line(image_sub(*,k),param,3.,line)
image_sub(*,k)=image_sub(*,k)*tmp(0)/tmp_obj(0)
endfor
endif
cont:
;substraction sky spectrum
for k=0,Nfib-1 do begin
ks=k/N_sky
vector_sky=total(sky_lin(*,ks:ks+1),2)/2
image_sub(*,k)=image_sub(*,k)-vector_sky
endfor
;save result
sxaddpar,head_lin,'FILE',image_name+'-sky.fts'
sxaddpar,head_lin,'SKYSUB','YES'
writefits,wdir+image_name+'-sky.fts',image_sub,head_lin
END
