pro SUBSKY,filetype,wdir,norm,trans=trans,mask=mask,median_sky=median_sky,field=field,w=w,lnslist=lnslist,nsky=nsky
;+
; NAME:
;	SUBSKY
; PURPOSE:
;       SKY spectra subtraction from MPFS-data
;       TOTAL sky spectra construction
;
; CALLING SEQUENCE:
;       SUBSKY, image_type_name,wdir,Norm   
; INPUTS:
;	name = string scalar type of exposure values '*'= 'obj' or 'star'
;			or 'test'
;       wdir = working directory
;
; OUTPUTS:
;	 saved in file  '*-sky.fts'
; OPTIONAL OUTPUT:
;	no
; OPTIONAL INPUT KEYWORDS:
;       trans - flag for making transparency correction	
;       median_sky  - avoids polynomial fitting across dispersion and computes median sky spectrum
;
; ERROR HANDLING:
;	no
; RESTRICTIONS:
;	no
; NOTES:
;       no
; PROCEDURES USED:
;	Pricedure:  SXADDPAR, function: goodpoly
; MODIFICATION HISTORY:
;	Written by P.Abolmasov as an alternative to sub_nsky and sky_tots
;-

loadct,27
if not(keyword_set(w)) then w=5.
if not(keyword_set(lnslist)) then lnslist=[5577.345,6300.23,6363.82]
if not(keyword_set(field)) then field=[16,16]
if filetype eq 'obj' then il=readfits(wdir+filetype+'_lin2.fts',hl) else il=readfits(wdir+filetype+'_lin.fts',hl)

if not(keyword_set(nsky)) then nsky=17
hs=hl
a=size(il)

dl=sxpar(hs,'CDELT1')
l0=sxpar(hs,'CRVAL1')

lambda=dindgen(a[1])*dl+l0
sky0=il(*,0:nsky-1)
obj0=il(*,nsky:a[2]-1)
obj=obj0
sky=obj0*0.

; ** preliminary shifting **
sh0=dblarr(nsky)  &  wl=dblarr(nsky)
aline=min(lnslist(where(lnslist gt min(lambda) and lnslist lt max(lambda))))
wall=where(lambda gt aline-3.*w and lambda lt aline+3.*w)
l0=lambda(wall)
vector0=sky0(wall,0)
fi0=gaussfit(l0,vector0,g0,nterms=4)
plot,l0,vector0/max(vector0),title='sky line '+strtrim(string(aline,format='(F8.3)'),2)+'profiles for different fibers',$
       xtitle='wavelength,Angstrom',ytitle='relat. intensity'
for k=1,nsky-1 do begin
  vector=sky0(wall,k)
  fi=gaussfit(l0,vector,g,nterms=4)
  sh0[k]=g[1]-g0[1]
  oplot,l0,(vector/max(vector))*.1+double(k)*.1-.15,color=35
  oplot,l0,(fi/max(vector))*.1+double(k)*.1-.15,color=55
  wl[k]=g[2]*2.35
endfor
 oplot,sh0+g0[1],dindgen(nsky)*.1-.15,color=3
; for k=1,nsky-1 do sky0(*,k)=vecshift(reform(sky0(*,k)),dx=-sh0[k])

; **  -- **

if keyword_set(median_sky) then goto,med else goto,poly

med: 
;  for y=0,nsky-1 do sky0(*,y)=sky0(*,y)*median(sky0)/median(sky0(*,y))
  for x=0,a[1]-1 do begin
         sky(x,*)=median(sky0(x,*))
  endfor
  obj=obj0-sky*norm
  sxaddhist,'sky subtracted using SUBSKY procedure',hs
  sxaddhist,'median average sky spectrum used',hs
goto,fin

poly:
close,1
if not(keyword_set(nsky)) then openr,1,'~/idllib/mpfs.lib/fiber.pos' else openr,1,'/bufm/mpfs.lib_2006/fiber_1.pos'
p=double(0.)
readf,1,p
fibers=p
while not(eof(1)) do begin
  readf,1,p
  fibers=[fibers,p]
endwhile
close,1
if not(keyword_set(nsky)) then openr,1,'/bufm/mpfs.lib_2006/etalon.pos' else openr,1,'/bufm/mpfs.lib_2006/etalon_1.pos'
p=double(0.)
readf,1,p
eta=p
while not(eof(1)) do begin
  readf,1,p
  eta=[eta,p]
endwhile
close,1

; print,eta

f=dblarr(a[1],3)
for x=0,a[1]-1 do begin
  f(x,*)=goodpoly(eta,reform(sky0(x,*)),2,3.,sfit,newy,news)
  sky(x,*)=poly(fibers,f(x,*))
  ; sky(x,*)=interpol(sky0(x,*),eta,fibers,/spline)
endfor
; skysp=total(sky,2)
; for k=0,2 do f(*,k)=smooth(med_ext(f(*,k),50),100)
; for x=0,a[1]-1 do sky(x,*)=skysp[k]*poly(fibers,f(x,*))/total(poly(fibers,f(x,*)))

obj=obj0-sky*norm
sxaddhist,'sky subtracted using SUBSKY procedure',hs
sxaddhist,'polynomial approximation (deg=2) used across dispersion',hs

fin:

; ** shifting sky spectra using brightest night sky lines
ao=size(sky)
dd=20  &  cc=5
sh=dblarr(ao[2])
     for k=0,ao[2]-1 do begin
        vector=total(obj0(wall,k-2>0:k+2<ao[2]-1),2)
        f=gaussfit(l0,vector,g,nterms=5)
        vector_sky=total(sky(wall,k-2>0:k+2<ao[2]-1),2)
        f=gaussfit(l0,vector_sky,g_sky,nterms=5)
        sh[k]=g[1]-g_sky[1]
;	sky(*,k)=vecshift(reform(sky(*,k)),dx=sh[k])
     endfor
 ;  if max(sh) gt min(sh) then shf=goodpoly(dindgen(ao[2]),sh,2,1.,shfit,newx,newy)
 ; shfit=lowess(dindgen(ao[2]),sh,50,1)
;  for k=0,ao[2]-1 do sky(*,k)=vecshift(sky(*,k),dx=shfit[k])
  ; for k=0,ao[2]-1 do sky(*,k)=vecshift(sky(*,k),dx=median(sh))
  print,median(sh)
; **

if keyword_set(trans) then begin
  bord=dblarr(a[1])
  for x=0,a[1]-1 do bord[x]=bormed(reform(obj0(x,*),field[0],field[1]))/bormed(reform(sky(x,*),field[0],field[1]))
  borpolypars=goodpoly(dindgen(a[1]),med_ext(bord,50),3,1.,borfit,newl,newb)
  borfit=poly(dindgen(a[1]),borpolypars)
;  plot,lambda,bord,yrange=[0.,2.],color=-1,xtitle='wavelength,Angstroms',ytitle='relative intensity of border pixels'
;  oplot,lambda,borfit,color=1000
  for x=0,a[1]-1 do obj(x,*)=obj0(x,*)-norm*borfit[x]*sky(x,*)
  sxaddhist,'transparency correction made using border pixels',hs
endif else begin
  obj=obj0-norm*sky
endelse

if keyword_set(mask) then begin
  lnslist=lnslist(where(lnslist gt min(lambda)+3*w and lnslist lt max(lambda)-3*w))  
  for y=0,a[2]-nsky-1 do begin
    for k=0,n_elements(lnslist)-1 do obj(where(lambda gt lnslist[k]-2*w and lambda lt lnslist[k]+2*w),y)=$
      interpol(obj(where(lambda le lnslist[k]-2*w or lambda ge lnslist[k]+2*w),y),lambda(where(lambda le lnslist[k]-2*w or lambda ge lnslist[k]+2*w)),$
       lambda(where(lambda gt lnslist[k]-2*w and lambda lt lnslist[k]+2*w)),/spline)
  endfor
endif

writefits,wdir+filetype+'-sky.fts',obj,hs

sxaddpar,hs,'IMAGETYP','sky spectrum'
if filetype eq 'obj' then writefits,wdir+'sky.fts',sky,hs
loadct,0
end
