function ADD_HEADER,headers
;+
; NAME:
;	ADD_HEADER
; PURPOSE:
;	addition information from FITS-headers MPFS-frames
; DESCRIPTION:
;	The function computes the total exposure, mean value zenit distance
;	and modified FITS header
; CALLING SEQUENCE:
;	Result =ADD_HEADER( headers )
; CATEGORY:
;	reduction MPFS-data
;
; INPUTS:
;	Headers = String array FITS-headers from the MPFS data
;
; OUTPUTS:
;	Header = String array containing the header from the FITS file.
;
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;	no
;
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;
;	Procedures:  SXADDPAR,SXADDHIST
;
; MODIFICATION HISTORY:
;       Written by Victor Afanasiev, Special Astrophysical Observatory RAS, Jul 1999
;-

on_error,2	;return to user

;calculation total exposure object and
;mean value zenit distance

a=size(headers)
N_exp=a(2)
h=headers(*,0)
if a(0) eq 1 then begin
tot_exp=sxpar(h,'EXPTIME')
z_mean=sxpar(h,'Z')
files=sxpar(h,'FILE')
endif
if a(0) gt 1 then begin
tot_exp=0
z_mean=0
files=''
for k=0,N_exp-1 do begin
tot_exp=tot_exp+sxpar(headers(*,k),'EXPTIME')
z_mean=z_mean+sxpar(headers(*,k),'Z')

;prepare information about initial files
file_name=str_sep(sxpar(headers(*,k),'FILE'),'.')
files=files+file_name(0)+' '
endfor
z_mean=float(z_mean)/N_exp
endif

;modification FITS-header
sxaddpar,h,'Z', z_mean,' MEAN ZENIT DISTANCE'
sxaddpar,h,'EXPTIME', tot_exp,' TOTAL EXPOSURE'
;sxaddhist,'initial FITS-files:',h
;sxaddhist,strcompress(files),h
return,h
end
