pro analine_faint_long,infile,l, w=w, ddv=ddv,smoo=smoo,dne=dne,colt=colt,$
   losc=losc,fixl=fixl,z=z,binning=binning
;+
; NAME:
;	anaLINE_faint_long
; PURPOSE:
;	obtaining flux, velosity and fwhm maps from a datacube
; DESCRIPTION: 
;	fits line profile with a gaussian and computes fluxes etc. from its parameters
;
; CALLING SEQUENCE:
;	ANALINE_faint_long,INFILE,LAMBDA,WDIR,w=w,dv=dv
; CATEGORY:
;	long slit data analysis		
;
; INPUTS:
;	INFILE - fits-file containing initial primarily-reducted data  
;	LAMBDA - rest-frame wavelength (redshift z for distant galaxies can be taken from the header, or 
;		added by keyword dv in km/s)
;	WDIR - output directory
; OUTPUTS:
;	output fts and ps files with names of the types: [lambda]_{fv,w}.ps and [lambda]_{f,v,w,c}.fts
;	containing fluxes (f), velosities (v) and fwhm in velosity units (w)
;		
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;	w - spectral interval halfwidth (in Angstroms, by default set to 10.)
;	dv - additional velosity shift
;	field - field size in pxls ([16,16] by default)
;       smoo - median smoothing 3X3 for flux maps
;       trub - truncate borders
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;	
;	Procedures:  ?
;
; MODIFICATION HISTORY:
;       Written by P. Abolmasov, Special Astrophysical Observatory RAS, Jul 2004
;-

  if not(keyword_set(w)) then w=50.
  if not(keyword_set(colt)) then colt=0.
  if not(keyword_set(dv)) then dv=0.

wdir=exdir(infile)
  out=wdir+strtrim(string(round(l)),2)

loadct,27

  obj=double(readfits(infile,h))
  dobj=double(readfits(exdir(infile)+'dobj_norm.fts',h))
  a=size(obj)
  uni=sxpar(h,'BUNIT')
  dl=sxpar(h,'CDELT1')
  l0=sxpar(h,'CRVAL1')
  dy=sxpar(h,'CDELT2')
  y0=sxpar(h,'CRVAL2')
  d=round(w/dl)
  area=dblarr(2*d+1,a[2])
  larea=dblarr(2*d+1)
  nx=a[1]                          ;  number of wavelengths 
  ny=a[2]			   ; number of fibers

  lambda=l0+dl*dindgen(nx)		; wavelength grid
  y=y0+dy*dindgen(ny)		; coordinate
  if not(keyword_set(z)) then z=double(sxpar(h,'REDSHIFT'))
if not(keyword_set(yrange)) then yrange=[min(y),max(y)]
nbins=double(ny)

if keyword_set(binning) then begin
  data=obj   &   ddata=dobj
  nbins=floor(double(ny)/double(binning))
  data1=dblarr(nx,nbins)   &   ddata1=dblarr(nx,nbins)
  y1=dblarr(nbins)
 for k=0,nbins-1 do begin    
   data1(*,k)=total(data(*,k*binning:(k+1)*binning-1),2)
  for j=0,nx-1 do ddata1[j,k]=stdev(data(j,k*binning:(k+1)*binning-1)) ;/sqrt(double(binning))
   y1[k]=y[float(k*binning+(k+1)*binning)*0.5]
 endfor
  obj=data1  &  dobj=ddata1  &   y=y1  & dy=dy*double(binning)
  ny=nbins
endif

  ccor=round((l*(z+1.)-l0)/dl)
  area=obj(ccor-d:ccor+d,*)  
  larea=lambda(ccor-d:ccor+d)  

aa=size(area)
; sence=median(sent(ccor-d:ccor+d))/sxpar(h,'GAIN')

if keyword_set(smoo) then begin
;  for i=0,aa[2]-1 do begin
   ; slice=reform(area(i,*),field[0],field[1])
;    area(*,i)=smooth(area(*,i),3)
;  endfor
endif

  set_plot,'x'

  flux=dblarr(ny)
  dflux=dblarr(ny)
  rms=dblarr(ny)
  fwhm=dblarr(ny)
  cont=dblarr(ny)
  cent=dblarr(ny)
  dfwhm=dblarr(ny)
  dcont=dblarr(ny)
  dcent=dblarr(ny)
  var=dblarr(ny)
  velt=dblarr(ny)
  var0=dblarr(ny)
  ston=dblarr(ny)
   arasm=dblarr(ny,aa[1])
;;   for k=0,aa[1]-1 do arasm(*,k)=area(k,*)   ;; marasm!

  for i=0,nbins-1 do begin
;    plot,larea,area(*,i)
    wplus=where(area(*,i) gt 0.,nplus)
    if nplus lt 1 then begin
      flux[i]=0.
      cent[i]=0.
      dcent[i]=w
      fwhm[i]=0.
      continue
    endif
    w1=where(larea gt l*(z+1.)-w/5. and larea lt l*(z+1.)+w/5.)
    mest=max(smooth(med_ext(area(w1,i),3),3),mcest)
    cest=larea(w1[mcest])
    if keyword_set(fixl) then cest=l*(z+1.)
    west=where(larea lt cest+w*.2 and larea gt cest-w*.2)
    nest=where(abs(larea-cest) ge w*.2 and area(*,i) le median(area(*,i)))

;    cont=lowess(larea(nest),area(nest,i),w/4.,1)
    contpars=goodpoly(larea(nest),area(nest,i),1,1.,afit)
    cont=poly(larea,contpars)
;    oplot,larea,cont,color=10000
 ;   gfit=gaussfit(larea(west),(area(west,i)-cont(west))/mest,gf,nterms=3,est=[mest,cest,6.])*mest
    flux[i]=int_tabulated(larea(west),area(west,i)-cont(west),/double)
    cent[i]=int_tabulated(larea(west),larea(west)*(area(west,i)-cont(west)),/double)/flux[i]
    fwhm[i]=sqrt(total((larea(west)-cent[i])^2*(area(west,i)-cont(west) > 0.)/flux[i]))*sqrt(8.*alog(2.))
    var[i]=stdev(area(nest,i)-cont(nest))*(double(n_elements(west)))         ;fwhm[i]/dl
    ston[i]=flux[i]/var[i]
    dcent[i]=fwhm[i]/ston[i]
  endfor

c=2.9979e+5                       ; c in km/s 
dflux=var
velt=(cent-l)*c/l   ;-double(dv)
dvelt=dcent*c/l

intflux=total(flux) & dinf=stdev(flux)

vmean=total(flux*velt)/intflux

posang=sxpar(h,'POSANG')

lof=floor(alog10(max(flux)))

set_plot,'ps'
; device,filename=out+'_fv.ps',xsize=field[0]+5.,ysize=field[1]+5.
device,filename=out+'_fv.ps',/colo,/port,xsize=21,ysize=27,yoffset=1,xoffset=0
loadct,0
  !p.multi=[0,1,2]
    plot,y,flux*(10D)^(-lof),yrange=[0,max(flux)]*(10D)^(-lof),psym=1,ytitle='flux, 10!U'+$
       strtrim(string(lof,format='(I8)'),2)+'!Nerg/cm!U2!Ns',xst=1,yst=1,charsize=2,$
       charthick=2.5,thick=2.
 
    oploterr,y,flux*(10D)^(-lof),dflux*(10D)^(-lof),1
  ;  xyouts,.8,.95,'Pos = '+strtrim(sxpar(h,'POSANG'),2),/norm

    plot,y,velt,yrange=[-100.,100.]+c*z+ddv,psym=1,ytitle='v, km/s', xtitle='y, arcsec',xst=1,yst=1,charsize=2.,$
       charthick=2.5,thick=2.
    oploterr,y,velt,dvelt,1
!p.multi=0
 device,/close
set_plot,'x'

message,'mean radial velosity: '+strtrim(string(vmean)),/cont

  h2=h
  fileout=out+'_F.fts'
  sxaddpar,h2,'FILE',fileout
  sxaddpar,h2,'INTFLUX',intflux,'erg/cm^2 s'
  sxaddpar,h2,'DFLUX',dinf,'erg/cm^2 s - integral flux uncertainty',after='INTFLUX'
  sxaddpar,h2,'MEANVEL',vmean,'km/s'
  sxaddpar,h2,'LINE',strtrim(string(l),2),' line wavelength'
  sxaddpar,h2,'IMAGETYP','flux map'
  sxaddpar,h2,'CDELT1',dy
  sxaddpar,h2,'CRVAL1',min(yrange)
  writefits,fileout,reform(flux),h2

  h2=h
  fileout=out+'_dF.fts'
  sxaddpar,h2,'FILE',fileout
  sxaddpar,h2,'INTFLUX',intflux,'erg/cm^2 s'
  sxaddpar,h2,'DFLUX',dinf,'erg/cm^2 s - integral flux uncertainty',after='INTFLUX'
  sxaddpar,h2,'MEANVEL',vmean,'km/s'
  sxaddpar,h2,'LINE',strtrim(string(l),2),' line wavelength'
  sxaddpar,h2,'IMAGETYP','flux uncertainty map'
  sxaddpar,h2,'CDELT1',dy
  sxaddpar,h2,'CRVAL1',min(yrange)
  writefits,fileout,reform(dflux),h2

  h2=h
  fileout=out+'_V.fts'
  sxaddpar,h2,'FILE',fileout
  sxaddpar,h2,'LINE',strtrim(string(l),2),' line wavelength'
  sxaddpar,h2,'IMAGETYP','velocity map'
  sxaddpar,h2,'CDELT1',dy
  sxaddpar,h2,'CRVAL1',min(yrange)
  writefits,fileout,reform(velt),h2

  h2=h
  fileout=out+'_dv.fts'
  sxaddpar,h2,'FILE',fileout
  sxaddpar,h2,'LINE',strtrim(string(l),2),' line wavelength'
  sxaddpar,h2,'IMAGETYP','velocity uncertainties map'
  sxaddpar,h2,'CDELT1',dy
  sxaddpar,h2,'CRVAL1',min(yrange)
  writefits,fileout,reform(dvelt),h2

fin:
loadct,0
end
