pro def_disp,LOGFILE,PLOT=plot
;+
; NAME:
;	DEF_DISP
; PURPOSE:
;	creation dispersion curve forSCORPIO-data in long-slit-mode
; DESCRIPTION:
;
; CALLING SEQUENCE:
;	DEF_DISP, LOGFILE, PLOT=shift
;
; CATEGORY:
;	reduction SCORPIO-data in long-slit-mode
;
; INPUTS:
;	LOGFILE = file name of LOG observation (in format FITS-header)
;
;
; OUTPUTS:
;	Result saved at working directory in file 'disper.fts'
;
;
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;	PLOT  if keyword preset then plotted control graphics
;
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;	Function  DEF_WDIR, DEF_RDIR, DEF_NAME, DEF_POS,READFILE, READ_FTS,
;		   SHIFT_IMAGE
;	Procedure SXADDPAR, REM_HITS,CROSS_N,FI_PEAK
;
; MODIFICATION HISTORY:
;       Written by Victor Afanasiev, Special Astrophysical Observatory RAS, Oct 2001
;-
on_error,2
swg=0
message,'creation dispersion curve',/cont
;determination working directory and file extention
wdir=def_wdir(logfile)
rdir=def_rdir(logfile)
 ext=def_ext(logfile)
;EXTRACTION SPECTRA NEON
;check size of frame
y_range=fix(str_sep(def_pos(logfile),','))
N_range=y_range(1)-y_range(0)+1
neon_name=DEF_NAME(LOGFILE,'neon',N_neon)
bias=read_fts(wdir+'bias_i.fts',/silent)
a=size(bias)  &  Nx=a(1)   &  Ny=a(2)
neon_s= fltarr(Nx)
tmp=readfile(rdir,neon_name(0),ext,h)
if sxpar(h,'NAXIS2') gt N_range then tmp=tmp(*,y_range(0):y_range(1))
neon=tmp-bias
W=10
neon_s=total(neon(*,Ny/2-w:Ny/2+w),2)

;*********************************************
;creation dispersion curve
x=findgen(Nx)
;find all peaks
fi_peak,x,neon_s,0,ipix,xpk,ypk,bkpk,ipk
;creation background
fon=INTERPOL( bkpk(1:ipk-1),xpk(1:ipk-1) ,x)
fon=smooth(fon,50,/edge_truncate)
ypk=ypk-fon(ipix)+20 & neon_s=neon_s-fon+100 & bkpk=bkpk-fon(ipix)+100

;previus identefication
vc=read_fts('neon_eta.fts',/silent)
dc=150  & xc=findgen(2*dc+1)-dc
gau=gaussfit(xc,cross_norm(neon_s,vc,dc),G)
lambda_shift=G(1)

pos=[163,240,352,564,659,962]+G(1)

pos=pos

line=[3888.65,4333.56,5015.66,6402.24,7032.20,9122.97]
f=poly_fit(line,pos,2,yfit)
lines=[3888.65,4044.42,4259.36,4336.00,4471.78,4712.5,4921.93,$
       5015.66,5338.50,5764.42,5852.48,5944.83,$
       6143.06,6266.49,6402.24,6506.53,$
       6598.95,6678.15,6929.47,7032.20,7245.17,$
       7383.98,7503.87,7635.11,7724.00,$
       7948.18,8014.79,8115.31,8264.52,8424.65,$
       8654.39,8782.40,9122.97,9224.50,9657.78]
pos_lines=lines
N_lines=N_elements(lines)
for k=0,N_lines-1 do begin
pos_lines(k)=f(0)+lines(k)*f(1)+f(2)*lines(k)^2
endfor
R=where(pos_lines lt Nx-20,ind)  & pos_lines=pos_lines(R)
lines=lines(R) & N_lines=N_elements(lines)
 d=3
for k=0,N_lines-1 do begin
I=max(neon(pos_lines(k)-d:pos_lines(k)+d),Nmax)
good_pos=fix(pos_lines(k))-d+Nmax
f=goodpoly(double(x(good_pos-d:good_pos+d)),double(neon(good_pos-d:good_pos+d)),2,3,Yfit)
good_pos=-f(1)/2/f(2)
pos_lines(k)=good_pos
endfor
print,'Number lines=',N_lines
P=3
f=goodpoly(pos_lines,lines,P,1.5,Yfit,xg,yg)
ng=N_elements(yg)

for j=0,P do begin
yg=yg-f(j)*xg^j
endfor
robomean,yg,3,0.5,avg,avg_rms

rms=string(avg_rms,format='(F4.2)')+' A'
if keyword_set(plot) then swg=1
if swg eq 1 then window,3,xsize=800,ysize=900
!P.multi=[0,1,3,0]
if swg eq 0 then begin
set_plot,'PS'
device,file=wdir+'ident.ps',xsize=18,ysize=26,xoffset=1,yoffset=1
endif
ymax=max(neon_s)
plot,neon_s,xst=1,yrange=[1E2,1E8],yst=1,charsize=2,/ylog
oplot,xpk,ypk,psym=6
for k=0,N_elements(pos_lines)-1 do begin
oplot,pos_lines(k)*[1,1],[neon_s(pos_lines(k))*10^0.1,neon_s(pos_lines(k))*9]
xyouts,pos_lines(k),neon_s(pos_lines(k))*10,string(lines(k),format='(F6.1)'),$
	/data,charsize=0.75,ORIENTATION=90
 endfor
plot,pos_lines,lines-Yfit,psym=6,yrange=[-10,10],charsize=2,$
	title='Accuracy approximation dispersion curve'+'  rms= '+rms,$
	xtitle='position lines, px',ytitle='error,A'
oplot,pos_lines,yfit*0
oplot,xg,yg,psym=1
print,'error dispersion curve='+rms
disp=f(1)+2*f(2)*pos_lines+3*f(3)*pos_lines^2

plot,pos_lines,disp,xst=1,yrange=[5,7.5],charsize=2,$
	xtitle='Position, px',ytitle='Linear dispersion, A/px',$
	title='SCORPIO, grating 300 1/mm'
if swg eq 0 then begin
device,/close
set_plot,'WIN'
endif
!P.multi=[0,1,1,0]
writefits,wdir+'disper.fts',f
cont:
end
