function DEF_SHI,image1,image2,coord,c,wx,wy

;+
; NAME:
;	DEF_SHIFT
; PURPOSE:
;	definition  shift between two image in any direction
; DESCRIPTION:
;
; CALLING SEQUENCE:
;	Result = DEF_SHI ( image1, image2, coord, c, wx, wy)
; CATEGORY:
;	reduction MPFS-data
;
;INPUTS :
;      image1 and image2 - images to be shifted
;      coord - string scalar define shift direction (value 'x' or 'y')
;      c - magnification for calculation cross-correlation
;      w - width band integration in cross-shift direction
;
; OUTPUTS:
;	Result = float scalar
;
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;	no
;
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;	CRSPROD
;
; MODIFICATION HISTORY:
;       Written by Victor Afanasiev, Special Astrophysical Observatory RAS, Jul 1999
;-

;on_error,2

a=size(image1)
Nx=a(1)  &  Ny=a(2)
if coord eq 'y' then begin
  vector2=total(image2(Nx/2-wx:Nx/2+wx,Ny/2-wy:Ny/2+wy),1)
  vector1=total(image1(Nx/2-wx:Nx/2+wx,Ny/2-wy:Ny/2+wy),1)
endif
if coord eq 'x' then begin
  vector2=total(image2(Nx/2-wx:Nx/2+wx,Ny/2-wy:Ny/2+wy),2)
  vector1=total(image1(Nx/2-wx:Nx/2+wx,Ny/2-wy:Ny/2+wy),2)
endif

if coord eq 'x' then m=c*(wx<50) else m=c*(wy<100)
;vector1=vector1-median(vector1)
;vector2=vector2-median(vector2)

y=dindgen(n_elements(vector1))

vecpolypars1=goodpoly(y,vector1,2,2.,vecpolyfit1)
vecpolypars2=goodpoly(y,vector2,2,2.,vecpolyfit2)

vector1=vector1-vecpolyfit1
vector2=vector2-vecpolyfit2

vector1=rebin(vector1,n_elements(vector1)*c)
vector2=rebin(vector2,n_elements(vector2)*c)
; CRSPROD,vector1,vector2,m,CORR
x=dindgen(2*m+1)-double(m)

CORR=c_correlate(vector1,vector2,x,/dou)

tmp=max(CORR,Nmax)
res=-x[Nmax]/double(c)

return,res
end
