; NEON line distortion measurements
; from SCORPIO long-slit data

pro geometry_neon_quasi3d,fileneon,filein,fileout,Ndeg=Ndeg,Nx_pos=Nx_pos,Ny_pos=Ny_pos,$
     PLOT=plot,w_dir=w_dir
;Ndeg	- degree polynomial approximation
;
ext='_i3.fts'
ext1='_ic3.fts'
if not(keyword_set(Ndeg)) then Ndeg=2
if not(keyword_set(Nx_pos)) then Nx_pos=30
if not(keyword_set(Ny_pos)) then Ny_pos=30
if not(keyword_set(w_dir)) then w_dir=exdir(fileneon)

; Read NEON
neon=readfits(w_dir+fileneon,header,/silent)
 if n_elements(neon) eq 1 then begin
  res=dialog_message('No file '+w_dir+fileneon)
  return
 endif

Nx=sxpar(header,'naxis1')
Ny=sxpar(header,'naxis2')

x=dindgen(Nx)

;creating background subtracted vectors

wy=Ny/(Ny_pos+1)  & y_pos=dindgen(Ny_pos)*wy+wy

pos_neon=dblarr(Nx,Ny_pos)

for k=0,Ny_pos-1 do begin
 y=total(neon(*,y_pos[k]-wy:y_pos[k]+wy)-min(neon),2)
; f=goodpoly(x,y,5,2,Yfit)
 yfit=lowess(x,y,100,1)
wwy=where(y lt yfit*1.5)
yfit=interpol(lowess(x(wwy),y(wwy),300,1),x(wwy),x)
 fi_peak,x,y-Yfit,0,ipix,xpk,ypk,bkpk,ipk
; fon=interpol(bkpk,xpk,x,/LSQUADRATIC)
 pos_neon(*,k)=y  ;-fon
; plot,x,y-yfit
endfor

;determination mean curvature lines
wx=Nx/(Nx_pos+1)-1 & x_pos=dindgen(Nx_pos)*wx+wx
pos=dblarr(Nx_pos,Ny_pos)
M=50 & x_cross=dindgen(2*M+1)-M
apod=cosin_apod(2*wx+1,10)

for i=0,Ny_pos-1 do begin
 for j=0,Nx_pos-1 do begin
   cross=CROSS_NORM(pos_neon(x_pos(j)-wx:x_pos(j)+wx,i)*apod,pos_neon(x_pos(j)-wx:x_pos(j)+wx,Ny_pos/2)*apod,M)
   gau=gaussfit(x_cross,cross,G)
   pos[j,i]=G[1]
 endfor 
endfor

coeff=dblarr(Nx_pos+2,Ndeg+1)
;fit repers along slit
for k=0,Nx_pos-1 do begin
  coeff(k+1,*)=goodpoly(double(y_pos),double(pos(k,*)),Ndeg,1,fit)
  ;maximum of curvature
  if k eq 0 then begin
    x_beg=abs(max(fit)-min(fit))+10
  endif
endfor
x_beg=0
;quadratic approximation dependance value polynomial coefficients along X
for k=0,Ndeg do begin
  FF=goodpoly(double(x_pos(*)),coeff(1:Nx_pos ,k),2,3,fit_coeff)
  coeff(1:Nx_pos,k)=fit_coeff
  coeff(Nx_pos+1,k)=FF(0)+FF(1)*(Nx-1)
  coeff(0,k)=FF(0)+FF(1)*x_beg
endfor
x_pos=[x_beg,x_pos,Nx-1]
;calculation good position
fit_pos=dblarr(Nx_pos+2,Ny_pos+2)
y_pos=[0,y_pos,Ny-1]
for k=0,Nx_pos+1 do begin
  for j=0,Ndeg do fit_pos(k,*)=fit_pos(k,*)+double(y_pos)^j*coeff(k,j)
endfor

;creating repers
x1=dblarr(Nx_pos+2,Ny_pos+2)  & y1=x1 & x0=x1
for i=0,Nx_pos+1 do begin
 for j=0,Ny_pos+1 do begin
   x1(*,j)=x_pos+fit_pos(*,j)
   x0(*,j)=x_pos
   y1(*,j)=make_array(Nx_pos+2,value=y_pos(j))
 endfor
endfor

 x1=reform(x1,(Nx_pos+2)*(Ny_pos+2))
 x0=reform(x0,(Nx_pos+2)*(Ny_pos+2))
 y1=reform(y1,(Nx_pos+2)*(Ny_pos+2))
 y0=y1
; Image transformations
if keyword_set(plot) then begin
  window,2,xsize=1000,ysize=805
  rms=stdev(neon(Nx/2-Nx/4:Nx/2+Nx/4,*),avg)
  tv,255-bytscl(congrid(neon,1000,400),0,avg+rms),0,0
  neon=WARP_TRI( X0, Y0, X1, Y1, neon)
  tv,255-bytscl(congrid(neon,1000,400),0,avg+rms),0,405
endif

n=n_elements(filein)

for i=0,n-1 do begin
  im=readfits(w_dir+filein[i]+ext,h,/silent)
  field=[sxpar(h,'FIELD1'),sxpar(h,'FIELD2')]

if max(im) eq -1 then continue
  a=size(im)

for q=0,field[0]-1 do begin
  imtmp=reform(im(*,indgen(field[1])*field[0]+q))
  imtmp=WARP_TRI( X0, Y0, X1, Y1, imtmp)    ; image correction
  im(*,indgen(field[1])*field[0]+q)=imtmp

endfor
  sxaddhist,'geometry distortion corrected',h
  writefits,w_dir+fileout[i]+ext1,im,h
endfor 
;fin:

END
