pro mean_bias,logfile,OVERSCAN=overscan,bias_par=bias_par

;+
; NAME:
;	mean_BIAS
; PURPOSE:
;	definition mean  bias at  SCORPIO-data in long-slit mode
; DESCRIPTION:
;
; CALLING SEQUENCE:
;	DEF_BIAS,FILELOG
;
; CATEGORY:
;	reduction SCORPIO-data in long-slit mode
;
; INPUTS:
;	LOGFILE = file name of LOG observation (in format FITS-header)
;
; OUTPUTS:
;	Result saved at working directory in file 'bias.fts'
;
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;	no
;
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;	Function  DEF_WDIR, DEF_RDIR, DEF_NAME, DEF_POS, READ_FILE, READ_FTS,
;	SXPAR,WRITEFITS
;
; MODIFICATION HISTORY:
;       Written by Victor Afanasiev, Special Astrophysical Observatory RAS, Oct 2001
;-
; on_error,2	;return to user

message,'REDUCTION BIAS ',/cont
;determination y-position
if not(keyword_set(overscan)) then overscan=[0,0]
if not(keyword_set(bias_par)) then bias_par=75
print,overscan
;determination working directory and file extention
wdir=def_wdir(logfile)
rdir=def_rdir(logfile)
ext=def_ext(logfile)
;reading file
bias_name=DEF_NAME(LOGFILE,'bias',N_bias)
tmp=readfits(rdir+bias_name[0]+ext,h_bias)
Nx=sxpar(h_bias,'NAXIS1') & Ny=sxpar(h_bias,'NAXIS2')
ima=fltarr(Nx,Ny,N_bias)
for k=0,N_bias-1 do begin
; tmp=READ_FILE(rdir,bias_name(k),ext,h)
 tmp=readfits(rdir+bias_name[k]+ext,h_bias)
 ima(*,*,k)=tmp(*,*)
endfor
;bias=rem_hits(ima)/N_bias
bias=med3d(ima,mask=mask)/N_bias
;check length of overscan
N=Nx/512
; if abs(overscan) ne Nx-(Nx/512)*512 then overscan=overscan/abs(overscan)*(Nx-(Nx/512)*512)
;remove overscan
;if overscan gt 0 then bias=bias(overscan:Nx-1,*) else $
;	bias=bias(0:Nx-1+overscan,*)
;determination position da columns
sb=size(bias)
bias=bias(overscan[0]>0:Nx+(overscan[0]<0)-1,overscan[1]>0:Ny+(overscan[1]<0)-1)
;if keyword_set(overscan_x) then bias=bias(overscan_x[0]>0:Nx+(overscan_x[1]<0)-1,overscan_y[0]>0:Ny+(overscan_y[1]<0)-1,*) 
y=total(bias,2)
N=N_elements(y)
x=dindgen(N)
y=y-lowess(x,y,20,2,3)
robomean,y,3,0.5,average,rms
w=2 & dw=1
par=bias_par
fi_peak,x,y,rms*par,ipix,xpk,ypk,bkpk,ipk
tmp=' '
for k=0,ipk-1 do begin
d=',' & if k eq ipk-1 then d=''
tmp=tmp+string(xpk(k),format='(I4)')+d
endfor

if tmp gt 0  then begin
s=size(bias)
nx=s[1] & ny=s[2]
xx=dindgen(nx) & yy=dindgen(ny)

for p=0,n_elements(tmp)-1 do begin
   wbc=where(xx ge tmp[p]-1 and xx le tmp[p]+1)
   www=where((xx gt tmp[p]-5 and xx lt tmp[p]-1) or (xx lt tmp[p]+5 and xx gt tmp[p]+1))

   for g=0,ny-1 do begin
      lin1=reform(bias(*,g))
      lin1(wbc)=median(lin1(www))
      bias(*,g)=lin1
   endfor
endfor
endif

tmp=strcompress(tmp)
sxaddpar,h_bias,'BAD_COL',tmp

robomean,bias(Nx/2-50:Nx/2+50,Ny/2-50:Ny/2+50),3,0.5,avg_bias,rms_bias
out_str='mean bias='+string(avg_bias,format='(F6.1)')$
	+' ADU  RON='+string(rms_bias,format='(F6.2)')+' ADU'
sxaddhist,out_str,h_bias
writefits,wdir+'bias.fts',bias,h_bias
end
