;reduction frames SCORPIO
PRO REDUCTION_FRAME,LOGFILE,IMAGE_TYPE,OVERSCAN=overscan,YLIMIT=ylimit,thresh=thresh
;+
; NAME:
;	reduction_obj
; PURPOSE:
;	bias and sky substraction,remove cosmic hits and
;	correction flexibility in set SCORPIO-data in long-slit-mode
; DESCRIPTION:
;	Routine make bias and substraction ,remove cosmic hits and
;	correction flexibility in set SCORPIO-data in long-slit mode
;	Routine compared two set frame objects in different position along slit
;	Routine modified FITS-header saved FITS-file
; CALLING SEQUENCE:
;	REDUCTION_OBJ, LOGFILE
;
; CATEGORY:
;	reduction SCORPIO-data in long-slit-mode
;
; INPUTS:
;	LOGFILE = file name of LOG observation (in format FITS-header)
;
;
; OUTPUTS:
;	Result saved at working directory in file 'obj.fts', in FITS-header
;   writes y-position spectra of object
;
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;	PLOT  if keyword preset then plotted control graphics
;
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;	Function  DEF_WDIR, DEF_RDIR, DEF_NAME, DEF_POS,READ_FILE, READ_FTS,
;		   SHIFT_IMAGE, SIGMA_FILTER, FILTER_IMAGE
;	Procedure SXADDPAR, REM_HITS
;
; MODIFICATION HISTORY:
;       Written by Victor Afanasiev, Special Astrophysical Observatory RAS, Oct 2001
;-
; on_error,2	;return to user

message,IMAGE_TYPE,/cont

if not(keyword_set(overscan)) then overscan=[0,0]

if not(keyword_set(thresh)) then thresh=3.
wy=10
if not(keyword_set(ylimit)) then ylimit='0,'+string(Ny-1,format='(I4)')
yL=fix(str_sep(ylimit,','))
print,yL(0),yL(1)
;definition position spectra
ypos=fix(str_sep(def_pos(logfile),','))
;determination working directory and file extention
wdir=def_wdir(logfile)
rdir=def_rdir(logfile)
 ext=def_ext(logfile)
obj_name=DEF_NAME(LOGFILE,IMAGE_TYPE,N_obj)

;read mean bias
bias=readfits(wdir+'bias.fts',h_bias,SILENT='silent')
Nx_bias=sxpar(h_bias,'NAXIS1') & Ny_bias=sxpar(h_bias,'NAXIS2')
;read bad columns position
xbad=fix(str_sep(sxpar(h_bias,'BAD_COL'),',')) & Nbad=N_elements(xbad) & wbad=0 & dwbad=1
;size image
tmp=READ_FILE(rdir,obj_name(0),EXT,h)
Nx=sxpar(h,'NAXIS1')-overscan[0] & Ny=sxpar(h,'NAXIS2')-overscan[1]
N_head=N_elements(h)
;
;check and correction size bias

if IMAGE_TYPE eq 'map' then bias=0
if IMAGE_TYPE eq 'map' then goto,cont1
bin_bias=fix(str_sep(sxpar(h_bias,'BINNING'),'x'))
bin_obj=fix(str_sep(sxpar(h,'BINNING'),'x'))
if bin_bias(0) ne bin_obj(0) then bias=rebin(bias,Nx_bias*bin_bias(0)/bin_obj(0),Ny_bias)

if Ny_bias ne Ny then begin
  y0=sxpar(h,'CRVAL2')
  bias=bias(*,y0/bin_bias(1):y0/bin_bias(1)+Ny*bin_obj(1)/bin_bias(1)-1)
endif

cont1:

ima_obj=dblarr(Nx+overscan[0],Ny+overscan[1])
print,N_head
head_tmp=strarr(N_head,N_obj)
ima_tmp=dblarr(Nx+overscan[0],Ny+overscan[1],N_obj)

for k=0,N_obj-1 do begin
  tmp=READ_FILE(rdir,obj_name[k],EXT,h)
  ima_tmp(*,*,k)=tmp
  head_tmp(*,k)=h
endfor
ima_tmp=reform(ima_tmp)
;remove overscan
;if overscan gt 0  then ima_tmp=ima_tmp(overscan:Nx-1,*,*) ELSE $
;	ima_tmp=ima_tmp(0:Nx-1+overscan,*,*)
atmp=size(ima_tmp)
  if atmp[0] gt 2 then ima_tmp=ima_tmp(overscan[0]:Nx-1+overscan[0],0:Ny-1,*) else $
     ima_tmp=ima_tmp(overscan[0]:Nx-1+overscan[0],0:Ny-1)

atmp=size(ima_tmp)
if atmp[0] gt 2 then ima_obj=total(ima_tmp,3) else ima_obj=ima_tmp
header=ADD_HEADER(head_tmp)

gain=sxpar(header,'GAIN')

; thresh=3.
if obj_name[0] ne 'neon' then begin
 if atmp[0] gt 2 then ima_obj=altermed(ima_tmp,0.,thresh,mask=mask,/med_clean)-bias else $
 	ima_obj=destroy(ima_tmp,mask,5.,20.,thresh,thresh*3.,wdir=wdir)-bias
endif else begin
    ima_obj=neores(ima_tmp,mask=mask,mf=thresh)-bias
endelse

set_plot,'ps'
device,filename=wdir+image_type+'_mask.ps',/port,/color,xsize=20, ysize=27
loadct,16
a=floor(size(ima_obj)/2.)
  tvscl,rebin(-alog(ima_obj(0:a[1]*2-1,0:a[2]*2-1)>1.),a[1],a[2]),2.,8.,xsize=12,ysize=8,/cent
loadct,0
  tvscl,rebin(mask(0:a[1]*2-1,0:a[2]*2-1) eq 0.,a[1],a[2]),2.,-1.,xsize=12,ysize=8,/cent
 xyouts,0,-2.,'cosmic particles graveyard for '+sxpar(header,'OBJECT')+'; threshold '+strtrim(string(thresh,format='(F5.1)'))+' used',/norm
device,/close
set_plot,'x'

;remove bad column
if xbad(0) gt 0 then begin
for i=0,Ny-1 do begin
  for k=0,Nbad-1 do begin
   ; ima_obj(xbad(k)-wbad:xbad(k)+wbad,i,j)=(ima_obj(xbad(k)-wbad-dwbad,i,j)+ima_obj(xbad(k)+wbad+dwbad,i,j))/2
  endfor
endfor
endif
;case reduction in one position
if IMAGE_TYPE eq 'obj' AND N_elements(ypos) eq 2 then goto,cont
if IMAGE_TYPE eq 'obj' then begin
message,'REDUCTION OBJECT IN ONE POSITION ',/cont

endif
ima_tmp=dblarr(Nx+overscan[0],Ny+overscan[1],N_obj)
head_tmp=strarr(N_head,N_obj)
for k=0,N_obj-1 do begin
  ima_tmp(*,*,k)=READ_FILE(rdir,obj_name(k),EXT,h)
  head_tmp(*,k)=h
endfor
;remove overscan
;if overscan gt 0 then ima_tmp=ima_tmp(overscan:Nx-1,*,*) ELSE $
;	ima_tmp=ima_tmp(0:Nx-1+overscan,*,*)
atmp=size(ima_tmp)
;if atmp[0] gt 2 then ima_tmp=ima_tmp(0:Nx-1-overscan[0],0:Ny-1-overscan[1],*) else $
;     ima_tmp=ima_tmp(0:Nx-1-overscan[0],0:Ny-1-overscan[1])

header=ADD_HEADER(head_tmp)
if IMAGE_TYPE eq 'map' then bias=0
; ima_obj=REMOVE_HITS(ima_tmp,/nosmooth,/norm)  ;-bias
;remove bad column
if image_type ne 'map' then begin
 if xbad(0) gt 0 then begin
  for i=0,Ny-1 do begin
   for k=0,Nbad-1 do begin
    ;ima_obj(xbad(k)-wbad:xbad(k)+wbad,i)=(ima_obj(xbad(k)-wbad-dwbad,i)+ima_obj(xbad(k)+wbad+dwbad,i))/2
   endfor&endfor
 endif
endif

cont:
;save result
if IMAGE_TYPE ne 'map' then begin
;if N_elements(ypos) eq 2 then ima_obj=ima_obj(*,yL(0):yL(1),*) ELSE ima_obj=ima_obj(*,yL(0):yL(1))
endif
if IMAGE_TYPE eq 'map' then begin
 hh=headfits(wdir+'obj_i.fts')
 N_y=sxpar(hh,'NAXIS2')
 bin_obj=str_sep(sxpar(hh,'BINNING'),'x') & bin_obj=bin_obj(1)
 bin_map=str_sep(sxpar(header,'BINNING'),'x') & bin_map=bin_map(1)

 d_y=sxpar(hh,'CRVAl2')
;ima_obj=ima_obj(*,d_y/bin_map:d_y/bin_map+N_y-1*bin_obj/bin_map)
endif
; if N_elements(ypos) eq 2 then ima_obj=ima_obj(*,yL(0):yL(1),*) ELSE ima_obj=ima_obj(*,yL(0):yL(1))

sxaddpar,header,'POSANG',sxpar(header,'PARANGLE')-sxpar(header,'ROTANGLE')+131.
sxaddpar,header,'FILE',IMAGE_TYPE+'_i.fts'
sxaddpar,header,'REDSHIFT',def_par(logfile,'REDSHIFT')
sxaddhist,'initial FITS-files:',header
sxaddhist,strcompress(obj_name),header
sxaddpar,header,'YPOS',def_pos(logfile),' POSITION OBJECT ALONG SLIT',BEFORE='DATE'
writefits,wdir+IMAGE_TYPE+'_i.fts',ima_obj,header

end
