function REMOVE_HITS,obj_spp,obj_clean,NOSMOOTH=nosmooth,NORM=norm

;+
; NAME:
;	REM_HITS
; PURPOSE:
;	remove cosmic hits on exposure frames
; DESCRIPTION:
;
; CALLING SEQUENCE:
;	Result = REM_HITS ( obj_spp, NOSMOOTH=nosmooth)
; CATEGORY:
;	reduction MPFS-data
;
;INPUTS :
;      obj_spp - set of analised images
;
; OUTPUTS:
;	Result = total image cleaned at cosmic hits
;
; OPTIONAL OUTPUT:
;	obj_spp - set of  images cleaned at cosmic hits
;
; OPTIONAL INPUT KEYWORDS:
;	NOSMOOTH if keyword present, data not smoothed along dispersion
;   NORM  if keywords present, data in different frames normalised
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;	ROBOMEAN
;
; MODIFICATION HISTORY:
;       Written by Victor Afanasiev, Special Astrophysical Observatory RAS, Jul 1999
;-
on_error,2
a=size(obj_spp)
n=a(1)-1
m=a(2)
n_exp=a(3) ; number of OBJ frames
n_replace=0
IF a(0)	eq 2 then goto,cont_1
message,'remove cosmic hits in '+string(n_exp,format='(I2)')+' image',/cont
thresh_sig=5 ; thresh. for sigma determination
thresh=4.2; thresh. for first iteration
im=fltarr(m,n_exp) ; array for each channels

n_events=max([1,n_exp-2]) ; number events for hit removing
list=bindgen(n_exp)
mean_1=findgen(n_exp-1)
;calculation total intensity in different frames
co=0
if KEYWORD_SET(norm) then begin
image_tmp=fltarr(n+1,m)
mean_bias=fltarr(n_exp)
total_int=fltarr(n_exp)
message,'normalisation frames',/cont
for m=0,n_exp-1 do begin
;median smoothing  images
image_tmp(*,*)=obj_spp(*,*,m)
vector_bias=total(image_tmp(2:6,*),1)/5.
robomean,vector_bias,2.,0.5,mean_b,avgdev,sigma
mean_bias(m)=mean_b
image_tmp=median(image_tmp-co*mean_bias(m),5)
total_int(m)=total(image_tmp)
print,'  total flux '+$
	string(total_int(m),format='(E11.2)')

endfor
total_norm=total(total_int)/n_exp
for m=0,n_exp-1 do begin
obj_spp(*,*,m)=(obj_spp(*,*,m)-co*mean_bias(m))*total_norm/total_int(m)
endfor

endif

;goto,cont_3
FOR ch=1,n-1 do BEGIN
im(*,*)=obj_spp(ch,*,*)
for i=0,n_exp-1 do BEGIN   ; clear CH in each spectra
 mask=0
 newlist=where(list NE i)
  for j=0,n_exp-2 do BEGIN ; mask creating
    resid=im(*,i)-im(*,newlist(j))
   ;mean_dis,resid,sigma=sigma,dif=dif,mean=mean
   robomean,resid,thresh_sig,0.5,mean_1(j),avgdev,sigma
    dif=resid-mean_1(j)
    rec=where(dif GT thresh*sigma)
    mask=[mask,rec]

                     ENDFOR  ; mask creating

    hist=histogram(mask,min=0)
    hit=where(hist GE n_events,count)

    if count GT 0 then BEGIN
     IF N_EXP GT 2 then av=total(im(*,newlist(*)),2)/(n_exp-1)+total(mean_1)/(n_exp-1) else $
          av=im(*,newlist(0))+mean_1(0)
	n_replace=n_replace+N_elements(hit)
     obj_spp(ch,hit,i)=av(hit)
                       ENDIF
                   ENDFOR  ; clear CH in each spectra
;if (ch/100-ch/100.) eq 0 then  print,'Vector:',ch
ENDFOR ; ch
if KEYWORD_SET(norm) then begin
for m=0,n_exp-1 do begin
obj_spp(*,*,m)=obj_spp(*,*,m)*total_int(m)/total_norm
endfor
endif
print,'ok'
result=total(obj_spp,3)
goto,cont_3
cont_1:
if KEYWORD_SET(nosmooth) then goto,cont_2
message,' image smoothed along dispersion',/cont
for  k=0,m-1 do begin
vector_tmp=float(obj_spp(*,k,0))
vector_tmp=median(vector_tmp,10)
obj_spp(*,k,0)=vector_tmp
endfor
cont_2:
result=obj_spp
cont_3:
if a(0) ne 2  then message,string(n_replace)+' points replaced',/cont
RETURN,result
END
