;; ***************************** Scaling changes:  ***************************

pro changescales_event,ev

;   *** event processing ***

   common drawfield, lrange, frange, screensize, scales, x0, x1, outtext, lineid, indraw
   common fide, curfilelist, dir, meds, runs, fact
   common specpars, dls, l0s, nxs, ll

   nf=n_elements(curfilelist)
   
   widget_control,ev.id,get_uvalue=uv
 ;  print,uv
   if uv eq 'Kill' then begin
       widget_control,ev.top,/destroy 
       plotfits,curfilelist
   endif

   if uv eq 'Vasto' then begin
       scales=replicate(1d,nf)
       widget_control,ev.top,/destroy 
       plotfits,curfilelist
   endif

   wk=where(curfilelist eq uv,nk)
   if nk eq 1 then scales[wk[0]]=ev.value
   if nk eq 1 then plotfits,curfilelist
end

;; *** scaling: main procedure ***

pro change_scales,ev
  common drawfield, lrange,  frange, screensize, scales
  common fide, curfilelist, dir, meds, runs, fact
 
  nf=n_elements(curfilelist)
  if n_elements(scales) eq 1 then scales=replicate(1d,nf)
  if n_elements(scales) lt nf then scales=[scales,replicate(1d,nf-n_elements(scales))]
  vasco=widget_base(title='Change spectra scales',xsize=screensize[0]*0.5,$
     ysize=screensize[1]*0.1*double(nf+1),group_lea=base0,/toolbar,$
     yoffset=screensize[1]*(.9-0.1*double(nf+1)),xoffset=screensize[0]*0.4)
  reset=widget_button(vasco,val='Set all to 1 and exit',uval='Vasto')
  quit=widget_button(vasco,val='Save scaling and exit',uval='Kill',xoffset=screensize[0]*0.2/double(nf+1))

  slither=uintarr(nf)  &  slid=curfilelist
  for k=0,nf-1 do begin  ;  xoffset=double(k)*screensize[0]*0.3/double(nf+1),
    slither[k]=cw_fslider(vasco,title=curfilelist[k],/drag,/edit,$
        uval=slid[k],val=scales[k],min=scales[k]*1.e-2,max=scales[k]*10.,xsize=screensize[1]*0.4,ysize=screensize[1]*0.025)
    ; scales[k]=val
  endfor
  widget_control,vasco,/realize

  xmanager,'changescales',vasco,group=vasco,/no_block
;widget_control,vasco,/destroy
  plotfits,curfilelist
end

;; ************************* Additional tool -- Gaussian line approximation

pro drawgauss,lc
  common drawfield, lrange,  frange, screensize, scales, x0, x1, outtext, lineid, indraw
  common fide, curfilelist, dir, meds, runs, fact
  common specpars, dls, l0s, nxs, ll

nf=n_elements(curfilelist)
w=20

for k=0,nf-1 do begin
  fk=readfits(curfilelist[k],hk,/sil)
  dl=sxpar(hk,'CDELT1')  &  l0=sxpar(hk,'CRVAL1')
   n=n_elements(fk)
   l=dindgen(n)*dl+l0
     ddl=min(abs(l-ll),wll)
   fkar=fk(wll-w>0:wll+w<n-1)
   lar=l(wll-w>0:wll+w<n-1)

   curw=!d.window
window,/free,xsize=600,ysize=350,retain=2
m=1.d17
gfit=gaussfit(lar,fkar*m,g,nterms=4,sigma=dg)/m
loadct,0,/sil
plot,lar,fkar,xtitle='wavelength,'+string("305B),ytitle='F!D!4k!3!N, erg cm!U-2!Ns!U-1!N'+string("305B)+'!U-1!N',$
  title=curfilelist[k]+', around !4k!3='+strtrim(string(ll,for='(F10.2)'),2)+string("305B),/yst
loadct,27,/sil
oplot,lar,gfit,color=1000
fluxscale=floor(alog10(g[2]*g[0]*sqrt(2.*!DPI)/m))

xyouts,0.5,0.9,'!4k!3 = '+strtrim(string(g[1],for='(F15.2)'),2)+string("261B)+strtrim(string(dg[1],for='(F6.2)'),2)+string("305B),/norm,color=111111,charsize=1.5,charthick=2.
xyouts,0.5,0.8,'FWHM = '+strtrim(string(g[2]*2.3548,for='(F10.2)'),2)+string("261B)+strtrim(string(dg[2]*2.3548,for='(F10.2)'),2)+string("305B),/norm,color=111111,charsize=1.5,charthick=2.
xyouts,0.4,0.7,'F = ('+strtrim(string(g[2]*g[0]*sqrt(2.*!DPI)/(10d)^fluxscale/m,for='(F5.2)'),2)+string("261B)+strtrim(string((dg[2]*g[0]+g[2]*dg[0])*sqrt(2.*!DPI)/(10d)^fluxscale/m,for='(F5.2)'),2)+') 10!U'+strtrim(string(fluxscale,for='(I10)'),2)+'!Nerg/cm!U2!N/s',/norm,color=111111,charsize=1.5,charthick=2.
loadct,0,/sil
wset,curw
endfor

end


; ************************** plot adjustment **********************************

pro change_drawproterties,ev
;
  common drawfield, lrange,  frange, screensize, scales, x0, x1, outtext, lineid, indraw
  common fide, curfilelist, dir, meds, runs, fact
  common specpars, dls, l0s, nxs, ll

;ll=ev.x*(max(lrange)-min(lrange))/(screensize[0]*0.89) ;; wavelength estimate

ll=(ev.x-x0)*(max(lrange)-min(lrange))/(x1-x0)+min(lrange)

  case ev.release of
     0:  widget_control,outtext[1],set_value='wavelength='+strtrim(string(ll,for='(F10.2)'),2)+string("305B) ;,/cont
     1:  if lineid eq 0 then lrange=ll+(lrange-median(lrange,/even))*0.95
     2:  lrange=0
     4:  lrange=ll+(lrange-median(lrange,/even))*1.15
     8:  lrange=lrange+abs(ll-(max(lrange)+min(lrange))/2.)*0.5
     16: lrange=lrange-abs(ll-(max(lrange)+min(lrange))/2.)*0.5
     else: message,ev.release
  endcase

if ev.release eq 1 and lineid gt 0 then drawgauss,ll

;  if ev.press gt 0 then 
  plotfits,curfilelist
end

;; ************************* Plotting ********************************

pro plotfits,flist,psout=psout

  common drawfield, lrange,  frange, screensize, scales, x0, x1, outtext, lineid, indraw
  common fide, curfilelist, dir, meds, runs, fact
  common specpars, dls, l0s, nxs, ll

colorsequence=[-1,170,111111,300000,-65000]

  nf=n_elements(flist)
  if scales[0] eq 0 and n_elements(scales) eq 1 then scales=replicate(1d,nf)
  if n_elements(scales) lt nf then scales=[scales,replicate(1d,nf-n_elements(scales))]

dls=dblarr(nf)  &  l0s=dblarr(nf)  &  nxs=dblarr(nf)  

if flist[0] eq '' then begin
  plot,[0.,1.],[0.,1.],/noda
  xyouts,0.5,0.5,'Zdes mogla byt vasha reklama!',charsize=1.8,charthick=2.
;  xyouts,0.5,0.3,'tribute K. Malewicz',charsize=1.2,charthick=1.5
  return
endif
 ; scales=replicate(1d,nf)


if keyword_set(psout) then begin
   set_plot,'ps'
   device,filename=psout,/land,xsize=28,ysize=21,/color,yoffset=28,xoffset=0
endif

  for k=0,nf-1 do begin
     sp=readfits(flist[k],hsp,/sil)
     a=size(sp)
     dl=sxpar(hsp,'CDELT1')   &  l0=sxpar(hsp,'CRVAL1')  &  nx=n_elements(sp)
     l=dindgen(nx)*dl+l0  &  dls[k]=dl  &  l0s[k]=l0  &  nxs[k]=nx
             if n_elements(lrange) lt 2 then lrange=[min(l),max(l)]
             if n_elements(frange) lt 2 then frange=[min([sp(where(finite(sp)))]),max(sp(where(finite(sp))))]*scales[0]
   if total(frange) eq 0. and k eq 0 then begin
       frange=[min(sp),max(sp)]*scales[0]                        
       lrange=[min(l),max(l)]
   endif
   ; print,lrange
     if k eq 0 then begin
       loadct,0,/sil
       plot,l,sp*scales[k],xrange=lrange,yrange=frange,xtitle='wavelength,'+string("305B),/xst,/yst,$
          ytitle='F!D!4k!3!N, erg cm!U-2!Ns!U-1!N'+string("305B)+'!U-1!N',charsize=1.2+double(keyword_set(psout)),$
          charthick=2.+1.*double(keyword_set(psout)),thick=1.+2.*double(keyword_set(psout)),color=-1+2.*double(keyword_set(psout))+(keyword_set(psout)),$
          noda=(meds[k]+runs[k] gt 0)
     endif else begin
       loadct,27,/sil
       if meds[k] eq 0 then oplot,l,sp*scales[k],color=colorsequence[k mod n_elements(colorsequence)],thick=1.+2.*double(keyword_set(psout)) ; ,noda=meds[k]
       loadct,0,/sil
     endelse   
     if meds[k]+runs[k] gt 0 then begin
        sp1=sp
       if meds[k] gt 1 then sp1=med_ext(sp1,meds[k]*2+1)
       if runs[k] gt 0 then sp1=gauss_convol(l,sp1,(runs[k])*dl/2.)
       loadct,27,/sil
        oplot,l,sp1*scales[k],color=colorsequence[k mod n_elements(colorsequence)],thick=2.+2.*double(keyword_set(psout))
     endif
  endfor
       loadct,0,/sil
      if lineid eq 1 then oplot,[ll,ll],frange,linestyle=1,thick=2.
       
if keyword_set(psout) then begin
   device,/close
  set_plot,'x'
endif

end


;  *** flux levels adjustment: events processing  ***

pro yzoom_event,ev
   common drawfield, lrange, frange, screensize, scales, x0, x1, outtext, lineid, indraw
   common fide, curfilelist, dir, meds, runs, fact
   common specpars, dls, l0s, nxs, ll

   nf=n_elements(curfilelist)
   
   widget_control,ev.id,get_uvalue=uv
 ;  print,uv
   if uv eq 'yzoomkill' then begin
       widget_control,ev.top,/destroy 
    ;   plotfits,curfilelist
   endif

   if uv eq 'yzoomdef' then begin
      frange=0
       widget_control,ev.top,/destroy 
   ;    plotfits,curfilelist
   endif

  if uv eq 'yzoommax' then frange=[min(frange),ev.value]
  if uv eq 'yzoommin' then frange=[ev.value,max(frange)]
 
   plotfits,curfilelist

end

;  *** flux levels adjustment:  main procedure

pro yzoom,ev
  common fide, curfilelist, dir, meds, runs, fact
  common drawfield, lrange, frange, screensize,scales, x0, x1, outtext, lineid, indraw
  common specpars, dls, l0s, nxs, ll

    izum=widget_base(title='Flux scale adjustment',xsize=screensize[0]*0.3,$
         ysize=screensize[1]*0.3,group_lea=base0,/toolbar,yoffset=screensize[1]*0.6)
    reset=widget_button(izum,val='Default ([Min, Max])',uval='yzoomdef')
    quit=widget_button(izum,val='Save scaling and exit',uval='yzoomkill',xoffset=screensize[0]*0.1) 

    slither=cw_fslider(izum,title='Maximum',/drag,/edit,scroll=1.e-3,$
        uval='yzoommax',val=max(frange),min=min(frange)-(max(frange)-min(frange))*2.,max=10.*max(frange),xsize=screensize[0]*0.25,ysize=screensize[1]*0.05)
    slither=cw_fslider(izum,title='Minimum',/drag,/edit,scroll=1.e-3,$
        uval='yzoommin',val=min(frange),min=min(frange)-(max(frange)-min(frange))*2.,max=10.*max(frange),xsize=screensize[0]*0.25,ysize=screensize[1]*0.05)

  widget_control,izum,/realize
  xmanager,'yzoom',izum,group=izum,/no_block

end

;  *** arbitrary smoothing 

pro smoothing_event,ev
   common drawfield, lrange, frange, screensize, scales, x0, x1, outtext, lineid, indraw
   common fide, curfilelist, dir, meds, runs, fact
   common specpars, dls, l0s, nxs, ll

   nf=n_elements(curfilelist)
   
   widget_control,ev.id,get_uvalue=uv
;   widget_control,ev.id,get_value=val
 ;  print,uv
   if uv eq 'exit' then begin
       widget_control,ev.top,/destroy 
    ;   plotfits,curfilelist
   endif

   if uv eq 'none' then begin
      meds[fact]=0
      runs[fact]=0
       widget_control,ev.top,/destroy 
   ;    plotfits,curfilelist
   endif
;if uv eq 'median' or uv eq 'running' then ev.value=floor(ev.value)
  if uv eq 'median' then meds[fact]=ev.value
; if uv eq 'slidertextmed' then if val gt 0 then meds[fact]=integer(val)
  if uv eq 'running' then runs[fact]=ev.value
;  if uv eq 'slidertextrun' then if val gt 0 then runs[fact]=integer(val)
;if uv eq 'median' then widget_control,slidertext1,set_value=ev.value
;if uv eq 'running' then widget_control,slidertext2,set_value=ev.value
  for k=0,nf-1 do if uv eq curfilelist[k] then fact=k

  plotfits,curfilelist

end

pro smoothspect,ev
  common fide, curfilelist, dir, meds, runs, fact
  common drawfield, lrange, frange, screensize,scales, x0, x1, outtext, lineid, indraw
  common specpars, dls, l0s, nxs, ll

nf=n_elements(curfilelist)
kk=0
    frame=widget_base(title='User-defined smoothing',xsize=screensize[0]*0.5,yoffset=screensize[1]*(0.7-0.1*(double(nf)-1. > 1.)),$
         ysize=screensize[1]*(0.20+0.125*(double(nf)-1. > 1.)),group_lea=base0,/toolbar)
    reset=widget_button(frame,val='None',uval='none')
    quit=widget_button(frame,val='Save & exit',uval='exit',xoffset=screensize[0]*0.1) 

actfile=uintarr(nf)
for k=0,nf-1 do actfile[k]=widget_button(frame,val=curfilelist[k],uval=curfilelist[k],xoffset=screensize[0]*0.4,$
   yoffset=screensize[1]*0.3*double(k)/double(nf+1))

    slidertext1=widget_text(frame,/all_eve,uvalue='slidertextmed',value='Median smoothing:')
    slither1=widget_slider(frame,title='median',/drag,$  ;/edit,$
        uval='median',val=meds[fact],min=0,max=100,xsize=screensize[0]*0.25,ysize=screensize[1]*0.05)
    slidertext2=widget_text(frame,/all_eve,uvalue='slidertextrun',value='Running average:')
    slither2=widget_slider(frame,title='running',/drag,$  ;/edit,$
        uval='running',val=runs[fact],min=0,max=100,xsize=screensize[0]*0.25,ysize=screensize[1]*0.05)

  widget_control,frame,/realize
  xmanager,'smoothing',frame,group=frame,/no_block

end

; *** base event processing ***

pro viewsp_event,ev
  common fide, curfilelist, dir, meds, runs, fact
  common drawfield, lrange, frange, screensize,scales, x0, x1, outtext, lineid, indraw
  common specpars, dls, l0s, nxs, ll

  widget_control,ev.id,get_uvalue=val

if dir eq '' then dir='/bufm/sco_tmp/'

  case val of
    'Load': curfilelist=dialog_pickfile(title='Select files with 1D-spectra',/multi,path=dir,filter=['*.fts','*.mt','*.fits','*.FTS'])   ;  dwid -- id of the drawing widget
    'Load extra': curfilelist=[curfilelist,dialog_pickfile(title='Select files with 1D-spectra',/multi,path=dir,filter=['*.fts','*.mt','*.fits','*.FTS'])]         ; potom pomenaem!
    'Clear': curfilelist=''              ;   black rectangle (tribute Malewicz!)
    'Print ps': plotfits,curfilelist,psout=dialog_pickfile(title='Select postscript output file!',path=dir,filter='*.ps')  ; PS output
    'Exit': widget_control,ev.top,/destroy
    'Y-coordinate zoom': yzoom,ev
    'Smooth': smoothspect,ev
    'lineid': if lineid eq 0 then begin & lineid=1 & widget_control,ev.id,set_value='Line ID (+)' & endif else begin & lineid=0 & widget_control,ev.id,set_value='Line ID (-)' & endelse
    'wkil': wkiller,/nowi
    'Change scales':  change_scales,ev
    'Draw': change_drawproterties,ev
    'Z-machine': if curfilelist[0] ne '' then zmachine,curfilelist,lrange=lrange,frange=frange,wayout=ev.top
    'base': plotfits,curfilelist
    else: print,val
  endcase

if val eq 'Z-machine' then wset,indraw

dir=exdir(curfilelist[0])
    wnull=where(curfilelist eq '',nnull)
    wnor=where(curfilelist ne '',nnor)
if nnor gt 0 then dir=exdir(curfilelist[0])
    if nnull gt 0 then begin
      if nnor gt 0 then curfilelist=curfilelist(wnor) else if val ne 'Exit' then val='Clear'
    endif
  if val  eq 'Load extra' and nnull eq 0 then meds=[meds,replicate(0.,n_elements(curfilelist)-n_elements(meds))]
  if val eq 'Load'  and nnull eq 0 then meds=replicate(0.,n_elements(curfilelist))
  if val  eq 'Load extra'  and nnull eq 0 then runs=[runs,replicate(0.,n_elements(curfilelist)-n_elements(runs))]
  if val eq 'Load'  and nnull eq 0 then runs=replicate(0.,n_elements(curfilelist))
;  if val eq 'Load' then meds=[0]
  if val eq 'Load' or val eq 'Load extra'  or val eq 'Clear' or val eq 'Change scales' then plotfits,curfilelist
  if val ne 'Exit' then widget_control,outtext[0],set_value='File(s) loaded: '+string(10B)+strjoin('		'+curfilelist,string(10B))
end

; ********************************** main procedure ****************************

pro view_sp,pars
  common fide, flist, dir, meds, runs, fact         ;; file list, directory, median and running average windows
  common drawfield, lrange,  frange, screensize, scales, x0, x1, outtext, lineid, indraw
  common specpars, dls, l0s, nxs, ll

lrange=[0.,1.]   &  frange=[0.,0.]
x0=72d  &  x1=951d
fact=0   &  ll=0  ;.; active file
lineid=0
meds=[0]  &  runs=[0]
 dir='/bufm/sco_tmp/ghoshpul_long/'
flist=''
set_plot,'x'
scales=[0]
  Device, GET_SCREEN_SIZE = screensize
screensize=[1024,768]

  strmenu=['Exit','Load','Load extra','Clear','Print ps','Y-coordinate zoom','Smooth',$
    'Change scales','Z-machine']
nstr=n_elements(strmenu)

  base=widget_base(title='LinV: 1D spectra viewer',xsize=screensize[0]*0.99,ysize=screensize[1]*0.55,$
    uvalue='base',/tracking_events)
  bbut=uintarr(nstr)

  for k=0,nstr-1 do bbut[k]=widget_button(base,val=strmenu[k],uval=strmenu[k],$
   xoffset=round(1.1*screensize[0]*double(k)/double(nstr+1)),xsize=1.09*round(screensize[0]*(1.+0.*double(k eq 0))/double(nstr+1)),$
   ysize=round(1.1*screensize[0]*(0.2+0.1*double(k eq 0))/double(nstr+1)))

;  bbgd=widget_button(base,val='',uval='Draw',xsize=screensize[0]*0.89,ysize=screensize[1]*0.445)

 ; widget_control,base,set_uval=
;  widget_control,menu2,set_uval=menu2
  widget_control,base,/realize
  draw=widget_draw(base,xsize=screensize[0]*0.95,ysize=screensize[1]*0.440,xoffset=screensize[0]*0.01,$
     yoffset=screensize[1]*0.05,uvalue='Draw',/button_events,/expose_ev,/keyboard_events) ;;,tooltip='Use central mouse click to restore the X-scale and right mouse click to expand the visible region')
outtext=uintarr(2)
  outtext[0]=widget_text(base,yoffset=screensize[1]*0.475,uvalue='tout',xsize=screensize[0]*0.4,$
    ysize=screensize[1]*0.05)  ;,title='Files loaded')
  outtext[1]=widget_text(base,yoffset=screensize[1]*0.475,xoffset=screensize[0]*0.75,uvalue='tout',xsize=screensize[0]*0.85,$
    ysize=screensize[1]*0.03,scr_xsize=screensize[0]*0.3,/wrap)  ;,title='wavelength,'+string("305B))
  wbut=widget_button(base,yoffset=screensize[1]*0.52,xoffset=screensize[0]*0.75,uvalue='lineid',xsize=screensize[0]*0.2,$
     ysize=screensize[1]*0.03,value='Line ID (-)',$
     tooltip='Toggles the left mouse click behavior between zooming in (-) and wavelength measurment (+)')

  wkilbut=widget_button(base,yoffset=screensize[1]*0.48,xoffset=screensize[0]*0.65,uvalue='wkil',xsize=screensize[0]*0.1,$
     ysize=screensize[1]*0.07,value='Kill windows',$
     tooltip='Closes all the windows created by Line ID tool')

 widget_control,draw,GET_VALUE=indraw
 wset,indraw
;  widget_control,base,set_uval=draw
  xmanager,'viewsp',base,group=base,/no_block
;  xmanager,'evdraw',base,group=base

end
